Package: FAfA
Title: Factor Analysis for All
Version: 1.1
Date: 2026-04-30
Authors@R: c(
    person("Abdullah Faruk", "KILIC", , "afarukkilic@trakya.edu.tr", role = c("aut", "cre")),
    person("Ahmet", "Caliskan", , "ahmetcaliskan1987@gmail.com", role = "aut")
  )
Description: Provides a comprehensive Shiny-based graphical user interface
    for conducting a wide range of factor analysis procedures. 'FAfA'
    (Factor Analysis for All) guides users through data uploading,
    assumption checking (descriptives, collinearity, multivariate
    normality, outliers), data wrangling (variable exclusion, data
    splitting), factor retention analysis (e.g., Parallel Analysis, Hull
    method, EGA), Exploratory Factor Analysis (EFA) with various rotation
    and extraction methods, Confirmatory Factor Analysis (CFA) for model
    testing, Reliability Analysis (e.g., Cronbach's Alpha, McDonald's
    Omega), Measurement Invariance testing across groups, and item
    weighting techniques. The application leverages established R packages
    such as 'lavaan' and 'psych' to perform these analyses, offering an
    accessible platform for researchers and students. Results are
    presented in user-friendly tables and plots, with options for
    downloading outputs.
License: MIT + file LICENSE
URL: https://github.com/AFarukKILIC/FAfA
BugReports: https://github.com/AFarukKILIC/FAfA/issues
Imports: Amelia, EFA.MRFA, EFA.dimensions, EFAtools, EGAnet, MBESS,
        bsicons, bslib, config, dplyr, gtools, ggcorrplot, ggplot2,
        golem, grDevices, graphics, haven, lavaan, magrittr, mctest,
        mice, missForest, moments, mvnormalTest, naniar, pastecs,
        psych, psychometric, readxl, rlang, semPlot, semTools, shiny,
        shinycssloaders, sirt, stats, tools, qgraph, utils
Suggests: spelling, testthat (>= 3.0.0)
Config/testthat/edition: 3
Encoding: UTF-8
Language: en-US
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-05-03 13:49:39 UTC; Faruk
Author: Abdullah Faruk KILIC [aut, cre],
  Ahmet Caliskan [aut]
Maintainer: Abdullah Faruk KILIC <afarukkilic@trakya.edu.tr>
Depends: R (>= 4.1.0)
Repository: CRAN
Date/Publication: 2026-05-03 14:50:07 UTC
Built: R 4.5.2; ; 2026-05-03 17:56:46 UTC; unix

Package: FAfA
Title: Factor Analysis for All
Version: 1.0
Date: 2026-04-29
Authors@R: c(
    person("Abdullah Faruk", "KILIC", , "afarukkilic@trakya.edu.tr", role = c("aut", "cre")),
    person("Ahmet", "Caliskan", , "ahmetcaliskan1987@gmail.com", role = "aut")
  )
Description: Provides a comprehensive Shiny-based graphical user interface
    for conducting a wide range of factor analysis procedures. 'FAfA'
    (Factor Analysis for All) guides users through data uploading,
    assumption checking (descriptives, collinearity, multivariate
    normality, outliers), data wrangling (variable exclusion, data
    splitting), factor retention analysis (e.g., Parallel Analysis, Hull
    method, EGA), Exploratory Factor Analysis (EFA) with various rotation
    and extraction methods, Confirmatory Factor Analysis (CFA) for model
    testing, Reliability Analysis (e.g., Cronbach's Alpha, McDonald's
    Omega), Measurement Invariance testing across groups, and item
    weighting techniques. The application leverages established R packages
    such as 'lavaan' and 'psych' to perform these analyses, offering an
    accessible platform for researchers and students. Results are
    presented in user-friendly tables and plots, with options for
    downloading outputs.
License: MIT + file LICENSE
URL: https://github.com/AFarukKILIC/FAfA
BugReports: https://github.com/AFarukKILIC/FAfA/issues
Imports: Amelia, EFA.MRFA, EFA.dimensions, EFAtools, EGAnet, ItemRest,
        MBESS, bsicons, bslib, config, dplyr, ggcorrplot, ggplot2,
        golem, grDevices, graphics, haven, lavaan, magrittr, mctest,
        mice, missForest, moments, mvnormalTest, naniar, pastecs,
        psych, psychometric, readxl, semPlot, semTools, shiny,
        shinycssloaders, sirt, stats, tools, utils
Suggests: spelling, testthat (>= 3.0.0)
Config/testthat/edition: 3
Encoding: UTF-8
Language: en-US
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-04-29 21:05:48 UTC; Faruk
Author: Abdullah Faruk KILIC [aut, cre],
  Ahmet Caliskan [aut]
Maintainer: Abdullah Faruk KILIC <afarukkilic@trakya.edu.tr>
Depends: R (>= 4.1.0)
Repository: CRAN
Date/Publication: 2026-04-29 21:40:09 UTC
Built: R 4.6.0; ; 2026-04-30 02:10:45 UTC; unix

# Generated by roxygen2: do not edit by hand

export("%||%")
export(about_server)
export(assumptions_server)
export(cfa_server)
export(cfa_ui)
export(data_selection_server)
export(efa_server_analysis)
export(efa_server_fac_ret)
export(efa_server_report)
export(inv_server)
export(item_weighting_server)
export(mod_itemrest_server)
export(mod_itemrest_ui)
export(mod_missing_server)
export(mod_missing_ui)
export(reliability_server)
export(run_app)
export(wrangling_server_ex_var)
import(bslib)
import(shiny)
importFrom(EFA.MRFA,hullEFA)
importFrom(EFA.MRFA,parallelMRFA)
importFrom(EFA.dimensions,EMPKC)
importFrom(EFA.dimensions,MAP)
importFrom(EFAtools,CD)
importFrom(EGAnet,EGA)
importFrom(MBESS,ci.reliability)
importFrom(bsicons,bs_icon)
importFrom(dplyr,arrange)
importFrom(dplyr,filter)
importFrom(dplyr,mutate)
importFrom(dplyr,select)
importFrom(ggplot2,element_text)
importFrom(ggplot2,theme)
importFrom(golem,activate_js)
importFrom(golem,add_resource_path)
importFrom(golem,bundle_resources)
importFrom(golem,favicon)
importFrom(golem,with_golem_options)
importFrom(grDevices,dev.off)
importFrom(grDevices,svg)
importFrom(graphics,image)
importFrom(graphics,plot)
importFrom(graphics,text)
importFrom(lavaan,cfa)
importFrom(lavaan,lavaanify)
importFrom(magrittr,"%>%")
importFrom(mctest,eigprop)
importFrom(mctest,mctest)
importFrom(moments,kurtosis)
importFrom(moments,skewness)
importFrom(mvnormalTest,mardia)
importFrom(naniar,mcar_test)
importFrom(naniar,vis_miss)
importFrom(pastecs,stat.desc)
importFrom(psych,alpha)
importFrom(psych,fa.parallel)
importFrom(psych,omega)
importFrom(psych,principal)
importFrom(psychometric,item.exam)
importFrom(semTools,reliability)
importFrom(shiny,shinyApp)
importFrom(shinycssloaders,withSpinner)
importFrom(sirt,stratified.cronbach.alpha)
importFrom(stats,as.formula)
importFrom(stats,cor)
importFrom(stats,cov)
importFrom(stats,lm)
importFrom(stats,mahalanobis)
importFrom(stats,median)
importFrom(stats,model.matrix)
importFrom(stats,na.omit)
importFrom(stats,pchisq)
importFrom(stats,var)
importFrom(utils,capture.output)
importFrom(utils,head)
importFrom(utils,write.csv)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# `{FAfA}`

<!-- badges: start -->

[![Lifecycle:
experimental](https://img.shields.io/badge/lifecycle-experimental-orange.svg)](https://lifecycle.r-lib.org/articles/stages.html#experimental)
<!-- badges: end -->

## Installation

You can install the development version of `{FAfA}` like so:

``` r
# FILL THIS IN! HOW CAN PEOPLE INSTALL YOUR DEV PACKAGE?
```

## Run

You can launch the application by running:

``` r
FAfA::run_app()
```

## About

You are reading the doc about version : 0.5

This README has been compiled on the

``` r
Sys.time()
#> [1] "2025-12-11 22:13:40 +03"
```

Here are the tests results and package coverage:

``` r
devtools::check(quiet = TRUE)
#> ℹ Loading FAfA
#> ── R CMD check results ─────────────────────────────────────────── FAfA 0.5 ────
#> Duration: 7.4s
#> 
#> ❯ checking package dependencies ... ERROR
#>   Namespace dependencies missing from DESCRIPTION Imports/Depends entries:
#>     'bsicons', 'bslib', 'ggplot2'
#>   
#>   Imports includes 27 non-default packages.
#>   Importing from so many packages makes the package vulnerable to any of
#>   them becoming unavailable.  Move as many as possible to Suggests and
#>   use conditionally.
#>   
#>   See section 'The DESCRIPTION file' in the 'Writing R Extensions'
#>   manual.
#> 
#> 1 error ✖ | 0 warnings ✔ | 0 notes ✔
#> Error: R CMD check found ERRORs
```

``` r
covr::package_coverage()
#> Error in loadNamespace(x): there is no package called 'covr'
```

#' ItemRest Analysis UI Module
#'
#' @param id Module namespace ID.
#' @import shiny
#' @import bslib
#' @importFrom bsicons bs_icon
#' @noRd
mod_itemrest_ui <- function(id) {
  ns <- NS(id)
  setting_note <- function(...) {
    helpText(class = "small text-muted mb-3", ...)
  }

  tagList(
    layout_columns(
      col_widths = c(4, 8),
      
      # Analysis Settings Card
      card(
        card_header(
          class = "bg-info text-white",
          "Analysis Settings", 
          bs_icon("sliders")
        ),
        card_body(
          p("Automated item removal strategies for EFA."),
          
          # Inputs based on itemrest function arguments [cite: 78-84]
          numericInput(
            ns("n_factors"),
            "Number of Factors (Optional):",
            value = NA, 
            min = 1,
            step = 1
          ),
          setting_note(
            "Leave empty to estimate the number of factors with parallel analysis. ",
            "Set a value when you already know the target factor count."
          ),
          
          selectInput(
            ns("cor_method"),
            "Correlation Method:",
            choices = c(
              "Pearson" = "pearson",
              "Spearman" = "spearman",
              "Polychoric" = "polychoric"
            ),
            selected = "pearson"
          ),
          setting_note(
            "Pearson is fast for continuous data. Spearman is rank-based. ",
            "Polychoric is usually better for ordinal/Likert items, but can be slower."
          ),

          selectInput(
            ns("search_method"),
            "Search Method:",
            choices = c(
              "All Subsets" = "all_subsets",
              "Stepwise" = "stepwise",
              "Backward" = "backward"
            ),
            selected = "all_subsets"
          ),
          setting_note(
            tags$b("All Subsets:"), " tests combinations of initially problematic items. ",
            tags$b("Stepwise:"), " re-detects problematic items after each removal round. ",
            tags$b("Backward:"), " removes one best item per round and is usually fastest."
          ),

          numericInput(
            ns("max_removed_per_iter"),
            "Max Items per Tested Removal:",
            value = 3,
            min = 1,
            step = 1
          ),
          setting_note(
            "For All Subsets and Stepwise, this limits how many items are tested ",
            "together in one removal combination. For example, 3 means the app can ",
            "try removing 1, 2, or 3 items at once. Backward ignores this setting ",
            "because it tests one item at a time."
          ),

          numericInput(
            ns("max_iter"),
            "Max Iterations (Stepwise / Backward):",
            value = 20,
            min = 1,
            step = 1
          ),
          setting_note(
            "Used by Stepwise and Backward. It limits how many rounds the algorithm can run. ",
            "A higher value allows more re-check/removal cycles but increases runtime."
          ),

          numericInput(
            ns("max_removed_total"),
            "Max Removed Total (Optional):",
            value = NA,
            min = 1,
            step = 1
          ),
          setting_note(
            "Optional global safety limit for total removed items. ",
            "Leave empty if you do not want to impose a total removal cap."
          ),

          numericInput(
            ns("min_loading"),
            "Minimum Loading:",
            value = 0.30,
            min = 0,
            max = 1,
            step = 0.05
          ),
          setting_note(
            "Items with no absolute loading at or above this threshold are flagged as low-loading."
          ),

          selectInput(
            ns("loading_diff_mode"),
            "Cross-loading Criterion:",
            choices = c(
              "Loading difference" = "numeric",
              "Howard (2016)" = "howard"
            ),
            selected = "numeric"
          ),
          setting_note(
            "Loading difference flags items whose strongest and second-strongest loadings are too close. ",
            "Howard (2016) applies a stricter rule: primary >= .40, secondary < .30, and difference >= .20."
          ),

          numericInput(
            ns("loading_diff"),
            "Minimum Loading Difference:",
            value = 0.10,
            min = 0,
            max = 1,
            step = 0.05
          ),
          setting_note(
            "Used when Cross-loading Criterion is Loading difference. ",
            "Smaller values are more permissive; larger values flag more potential cross-loadings."
          ),

          numericInput(
            ns("min_items_per_factor"),
            "Minimum Items per Factor:",
            value = 3,
            min = 1,
            step = 1
          ),
          setting_note(
            "Removal strategies that would leave fewer than this many items per factor are skipped."
          ),
          
          selectInput(
            ns("extraction_method"),
            "Extraction Method:",
            choices = c(
              "Unweighted Least Squares" = "uls",
              "Minimum Residual" = "minres",
              "Maximum Likelihood" = "ml",
              "Principal Axis" = "pa"
            ),
            selected = "uls"
          ),
          setting_note(
            "EFA extraction method passed to psych::fa. ULS is the default and is usually stable for this workflow."
          ),
          
          selectInput(
            ns("rotation_method"),
            "Rotation Method:",
            choices = c(
              "Oblimin" = "oblimin",
              "Varimax" = "varimax",
              "Promax" = "promax",
              "GeominQ" = "geominQ"
            ),
            selected = "oblimin"
          ),
          setting_note(
            "Rotation method passed to psych::fa. Oblique rotations such as Oblimin/Promax allow factors to correlate."
          ),
          
          actionButton(
            ns("run_itemrest"),
            "Run ItemRest Analysis",
            class = "btn-info w-100",
            icon = icon("play")
          ),
          tags$p(
            class = "text-muted small mt-3 mb-0",
            "Recommended items are shown on the right. To remove them from analyses, use the Exclude Variables tab."
          )
        )
      ),
      
      # Optimal Strategy Results (styled card UI)
      card(
        card_header("Optimal Strategy", bs_icon("trophy")),
        card_body(
          uiOutput(ns("optimal_strategy_ui")),
          tags$p(class = "text-muted small mt-2 mb-0",
                 "The strategy above represents the cleanest factor structure found.")
        )
      )
    ),
    
    # Detailed Summary
    card(
      card_header("Comparative Removal Strategies"),
      card_body(
        tableOutput(ns("removal_summary_table")),
        p(em("This table compares model fit and structure across different item removal thresholds."))
      )
    )
  )
}

#' Shared Helper: Compute correlation matrix
#' Reused across modules; cached via reactive in callers.
#' @noRd
compute_cor_matrix <- function(data, kind = c("pea", "poly")) {
  kind <- match.arg(kind)
  d <- stats::na.omit(data)
  if (kind == "pea") stats::cor(d)
  else suppressWarnings(psych::polychoric(d)$rho)
}

#' EFA Factor Retention Server Module
#' @param id Module namespace ID.
#' @param data Input data (reactive).
#' @noRd
efa_server_fac_ret <- function(id, data) {
  moduleServer(id, function(input, output, session) {

    # Show placeholder until Run is pressed
    results_rv <- reactiveValues(table = NULL, scree = NULL, ready = FALSE)

    numeric_complete_data <- function(d) {
      if (is.null(d)) return(NULL)
      d <- as.data.frame(d)
      d <- d[, sapply(d, is.numeric), drop = FALSE]
      if (ncol(d) < 2) return(NULL)
      d <- stats::na.omit(d)
      if (nrow(d) < 3) return(NULL)
      d
    }

    # Invalidate results when method or data change so stale results
    # are not shown after a new selection
    observeEvent(list(input$dimension_methods, input$fac_ret_cor_kind, data()), {
      results_rv$ready <- FALSE
      results_rv$table <- NULL
      results_rv$scree <- NULL
    }, ignoreInit = TRUE)

    # Run only when button is pressed - no reactive dependency on inputs inside
    observeEvent(input$run_factor_ret, {
      d <- data()
      req(d)

      # Snapshot inputs at the moment the button is pressed
      method <- isolate(input$dimension_methods)

      progress_id <- showNotification("Running factor retention analysis...",
                                      type = "message", duration = NULL,
                                      id = "fac_ret_progress")

      tryCatch({
        d_clean <- numeric_complete_data(d)
        if (is.null(d_clean)) {
          stop("At least 2 numeric variables and 3 complete rows are required.")
        }

        if (identical(method, "scree_plot")) {
          ev <- eigen(stats::cor(d_clean), symmetric = TRUE, only.values = TRUE)$values
          results_rv$scree <- data.frame(
            Factor = seq_along(ev),
            Eigenvalue = ev
          )
          tbl <- data.frame(
            Result = "Scree plot generated. Inspect the elbow in the plot below."
          )
        } else {
          tbl <- if (exists("factor_ret")) {
            factor_ret(d_clean, method = method)
          } else {
            data.frame(Error = "Function factor_ret not found.")
          }
          results_rv$scree <- NULL
        }

        results_rv$table <- tbl
        results_rv$ready <- TRUE
        removeNotification("fac_ret_progress")
        showNotification("Factor retention analysis complete.",
                         type = "message", duration = 4)
      }, error = function(e) {
        removeNotification("fac_ret_progress")
        showNotification(paste("Error:", e$message),
                         type = "error", duration = 8)
      })
    })

    # Table: reads from reactiveValues, no dependency on input$dimension_methods
    output$dim_ret_results <- renderTable({
      if (!results_rv$ready) {
        validate(need(FALSE, "Select a method and press Run Analysis."))
      }
      results_rv$table
    }, rownames = TRUE)

    output$scree_plot <- renderPlot({
      validate(need(results_rv$ready, "Press Run Analysis to create the scree plot."))
      scr <- results_rv$scree
      validate(need(
        !is.null(scr) && nrow(scr) > 0,
        "Select 'Scree Plot' and press Run Analysis to generate the scree plot."
      ))

      graphics::plot(
        scr$Factor, scr$Eigenvalue,
        type = "b", pch = 19,
        xlab = "Factor", ylab = "Eigenvalue",
        main = "Scree Plot"
      )
      graphics::abline(h = 1, lty = 2, col = "gray50")
    })
  })
}

#' EFA Analysis Server Module
#' Returns a list with `efa_object` and `settings` reactives.
#' @param id Module namespace ID.
#' @param data Input data (reactive).
#' @noRd
efa_server_analysis <- function(id, data) {
  moduleServer(id, function(input, output, session) {
    efa_res <- reactiveVal(NULL)

    # Centralised settings reactive (replaces the broken cross-namespace pattern)
    settings <- reactive({
      list(
        number_factor   = input$number_factor,
        rotating_method = input$rotating_method,
        fact_method     = input$fact_method,
        cor_kind        = input$cor_kind
      )
    })

    observeEvent(list(data(), settings()), {
      efa_res(NULL)
    }, ignoreInit = TRUE)

    observeEvent(input$run_efa, {
      d <- data()
      req(d)
      err <- validate_data(d, min_n = 30, min_p = 3, require_numeric = TRUE)
      if (!is.null(err)) {
        showNotification(err, type = "error", duration = 8)
        return()
      }
      showNotification("Running EFA...", type = "message")

      tryCatch({
        cor_type_arg <- if (input$cor_kind == "pea") "cor" else "poly"

        res <- psych::fa(
          r        = d,
          nfactors = as.numeric(input$number_factor),
          rotate   = input$rotating_method,
          fm       = input$fact_method,
          cor      = cor_type_arg
        )

        efa_res(res)
        showNotification("EFA Completed!", type = "message")
      }, error = function(e) {
        showNotification(paste("EFA Failed:", e$message),
                         type = "error", duration = 8)
        efa_res(NULL)
      })
    })

    list(efa_object = efa_res, settings = settings)
  })
}

#' EFA Reporting Server Module
#' @param id Module namespace ID.
#' @param data Input data (reactive).
#' @param efa_output_reactive Reactive containing the EFA results.
#' @param efa_settings_reactive Reactive containing the EFA settings.
#' @noRd
efa_server_report <- function(id, data, efa_output_reactive, efa_settings_reactive) {
  moduleServer(id, function(input, output, session) {

    # ---- A2 fix: cache correlation matrix; recomputed only when data or
    #              cor_kind change. Used by KMO, Bartlett, heatmap, and range.
    cor_matrix_rv <- reactive({
      req(data())
      tryCatch(
        compute_cor_matrix(data(), efa_settings_reactive()$cor_kind),
        error = function(e) NULL
      )
    })

    # ---- KMO ----
    output$kmo_result <- renderUI({
      d <- data(); req(d)
      tryCatch({
        cm <- cor_matrix_rv(); req(cm)
        kmo_res <- psych::KMO(cm)
        val <- round(kmo_res$MSA, 3)
        color <- if (val >= 0.8) "green" else if (val >= 0.6) "orange" else "red"
        HTML(paste0("<b style='font-size:1.4rem;color:", color, ";'>", val, "</b>"))
      }, error = function(e) tags$span(class = "text-danger",
                                        paste("Error:", e$message)))
    })

    # ---- Bartlett ----
    output$bartlett <- renderTable({
      d <- data(); req(d)
      tryCatch({
        cm <- cor_matrix_rv(); req(cm)
        test <- psych::cortest.bartlett(cm, n = nrow(d))
        data.frame(
          Statistic = round(test$chisq, 2),
          df        = test$df,
          p_value   = if (test$p.value < 0.001) "< .001"
                      else format(round(test$p.value, 3), nsmall = 3)
        )
      }, error = function(e) data.frame(Error = e$message))
    })

    # ---- Loadings ----
    output$efa_result_str <- renderTable({
      req(efa_output_reactive())
      res <- efa_output_reactive()
      loadings_df <- as.data.frame(unclass(res$loadings))
      if (!is.null(res$communality)) loadings_df$h2 <- res$communality
      loadings_df
    }, rownames = TRUE, digits = 3)

    # ---- Variance ----
    output$efa_result_expl_var <- renderTable({
      req(efa_output_reactive())
      res <- efa_output_reactive()
      if (is.null(res$Vaccounted)) {
        return(data.frame(Message = "Variance table not available."))
      }
      as.data.frame(unclass(res$Vaccounted))
    }, rownames = TRUE, digits = 3)

    # ---- Phi ----
    output$efa_result_interf_cor <- renderTable({
      req(efa_output_reactive())
      res <- efa_output_reactive()
      if (is.null(res$Phi)) {
        return(data.frame(
          Info = "Correlations not available. Reasons: 1) Orthogonal rotation used, or 2) Only 1 factor extracted."
        ))
      }
      as.data.frame(unclass(res$Phi))
    }, rownames = TRUE, digits = 3)

    # ---- Heatmap with palette options ----
    palettes <- list(
      "blue_red"   = c(low = "#0072B2", mid = "white",   high = "#D55E00"),
      "viridis"    = c(low = "#440154", mid = "#21908C", high = "#FDE725"),
      "warm"       = c(low = "#FFFFCC", mid = "#FD8D3C", high = "#800026"),
      "cool"       = c(low = "#F7FBFF", mid = "#6BAED6", high = "#08306B"),
      "grayscale"  = c(low = "#FFFFFF", mid = "#999999", high = "#000000")
    )

    output$heat_map <- renderPlot({
      req(data())
      cm <- cor_matrix_rv(); req(cm)
      pal <- palettes[[input$heatmap_palette %||% "blue_red"]]
      lab_show <- isTRUE(input$heatmap_show_values) && ncol(cm) < 25
      suppressWarnings(
        ggcorrplot::ggcorrplot(
          cm,
          type     = "lower",
          lab      = lab_show,
          colors   = c(pal[["low"]], pal[["mid"]], pal[["high"]]),
          tl.cex   = 10,
          lab_size = 3
        )
      )
    })

    # ---- Off-diagonal correlation summary ----
    output$cor_range_text <- renderText({
      cm <- cor_matrix_rv(); req(cm)
      tri <- cm[upper.tri(cm)]
      paste0(
        "Off-diagonal correlations - ",
        "Min: ",    round(min(tri),  3), "  |  ",
        "Max: ",    round(max(tri),  3), "  |  ",
        "Mean: ",   round(mean(tri), 3), "  |  ",
        "Median: ", round(stats::median(tri), 3)
      )
    })

    # ---- Download loadings ----
    output$download_efa_loadings <- downloadHandler(
      filename = "efa_loadings.csv",
      content  = function(file) {
        req(efa_output_reactive())
        write.csv(unclass(efa_output_reactive()$loadings), file)
      }
    )
  })
}

#' ItemRest Analysis Server Module
#'
#' Reads the structured `itemrest_result` object returned by the internal
#' itemrest() engine. The relevant fields are:
#'   * descriptive_stats : list(n_items, n_obs, min_value, max_value)
#'   * initial_efa       : list(efa, alpha, explained_var, loadings)
#'   * problem_items     : list(cross_3, cross_2, low_loading) - character vectors
#'   * removal_summary   : data frame of all tested removal combinations,
#'                         columns: Removed_Items, Total_Explained_Var,
#'                         Factor_Loading_Range, Cronbachs_Alpha, Cross_Loading
#'   * recommended       : recommended strategy list, or NULL
#'   * settings          : list(n_factors, cor_method, extract, rotate, auto_n_factors)
#'
#' @param id Module namespace ID.
#' @param data Reactive containing the input dataset.
#' @import shiny
#' @importFrom stats na.omit
#' @noRd
mod_itemrest_server <- function(id, data) {
  moduleServer(id, function(input, output, session) {

    analysis_results <- reactiveVal(list(res = NULL))

    observeEvent(data(), {
      req(data())
      analysis_results(list(res = NULL))
    }, ignoreNULL = TRUE)

    observeEvent(input$run_itemrest, {
      req(data())

      df <- data()
      if (!all(sapply(df, is.numeric))) {
        showNotification("Item drop-out requires all variables to be numeric.",
                         type = "error", duration = 8)
        return()
      }
      df_clean <- stats::na.omit(df)
      num_factors_arg <- if (is.na(input$n_factors)) NULL else input$n_factors
      max_removed_total_arg <- if (is.na(input$max_removed_total)) {
        NULL
      } else {
        as.integer(input$max_removed_total)
      }
      loading_diff_arg <- if (identical(input$loading_diff_mode, "howard")) {
        "howard"
      } else {
        input$loading_diff
      }

      progress_id <- showNotification(
        "Running item drop-out search. With many problem items this may take a while...",
        type = "message", duration = NULL, id = "ir_progress"
      )

      tryCatch({
        ir_result <- suppressMessages(suppressWarnings(
          itemrest(
            data                  = df_clean,
            cor_method            = input$cor_method,
            n_factors             = num_factors_arg,
            extract               = input$extraction_method,
            rotate                = input$rotation_method,
            min_loading           = input$min_loading,
            loading_diff          = loading_diff_arg,
            search                = input$search_method,
            max_iter              = as.integer(input$max_iter),
            max_removed_total     = max_removed_total_arg,
            max_removed_per_iter  = as.integer(input$max_removed_per_iter),
            min_items_per_factor  = as.integer(input$min_items_per_factor)
          )
        ))
        analysis_results(list(res = ir_result))
        removeNotification("ir_progress")
        showNotification("Analysis Complete!", type = "message", duration = 4)
      }, error = function(e) {
        removeNotification("ir_progress")
        showNotification(paste("Analysis Failed:", e$message),
                         type = "error", duration = 8)
      })
    })

    metric_box <- function(label, val, theme = "secondary") {
      if (is.null(val) || (is.character(val) && nchar(trimws(val)) == 0))
        return(NULL)
      div(
        class = paste0("p-2 rounded text-center border-start border-3 border-",
                       theme, " bg-light"),
        tags$div(class = "small text-muted text-uppercase",
                 style = "letter-spacing:0.05em", label),
        tags$div(class = "fw-bold", style = "font-size:1.05rem;", val)
      )
    }

    fmt_pct <- function(x) {
      if (is.null(x) || is.na(x)) return(NA_character_)
      paste0(formatC(100 * x, digits = 2, format = "f"), "%")
    }
    fmt_num <- function(x, d = 3) {
      if (is.null(x) || is.na(x)) return(NA_character_)
      formatC(x, digits = d, format = "f")
    }

    output$optimal_strategy_ui <- renderUI({
      out_data <- analysis_results()
      res <- out_data$res

      if (is.null(res)) {
        return(div(
          class = "text-muted text-center p-4",
          bsicons::bs_icon("hourglass-split", size = "2em"),
          tags$p(class = "mt-2 mb-0",
                 "Run the analysis to see the result.")
        ))
      }

      ds        <- res$descriptive_stats
      eff       <- res$initial_efa
      settings  <- res$settings
      problems  <- res$problem_items
      all_probs <- res$all_problem_items
      recommended <- res$recommended
      removal   <- res$removal_summary    # full df or NULL

      n_items    <- ds$n_items   %||% NA
      n_obs      <- ds$n_obs     %||% NA
      n_factors  <- settings$n_factors %||% NA
      auto_flag  <- isTRUE(settings$auto_n_factors)
      init_alpha <- if (!is.null(eff$alpha)) eff$alpha else NA
      init_var   <- if (!is.null(eff$explained_var)) eff$explained_var else NA

      n_lowload <- length(problems$low_loading)
      n_cross2  <- length(problems$cross_2)
      n_cross3  <- length(problems$cross_3)
      n_total_problems <- length(all_probs)

      rec_row <- NULL
      rec_status <- "none"   # "none", "optimal", "fallback"

      if (n_total_problems == 0) {
        rec_status <- "none"
      } else if (!is.null(recommended) && !is.null(recommended$row)) {
        rec_row <- recommended$row[1, , drop = FALSE]
        rec_status <- "optimal"
      } else if (is.data.frame(removal) && nrow(removal) >= 1) {
        rec_row <- removal[1, , drop = FALSE]
        rec_status <- "fallback"
      }

      # Extract recommendation fields safely
      rec_removed   <- if (!is.null(rec_row)) as.character(rec_row$Removed_Items)        else NULL
      rec_var       <- if (!is.null(rec_row)) rec_row$Total_Explained_Var                 else NULL
      rec_loadrange <- if (!is.null(rec_row)) as.character(rec_row$Factor_Loading_Range) else NULL
      rec_alpha     <- if (!is.null(rec_row)) rec_row$Cronbachs_Alpha                    else NULL
      rec_cross     <- if (!is.null(rec_row)) as.character(rec_row$Has_Cross_Loading)    else NULL
      rec_low       <- if (!is.null(rec_row)) as.character(rec_row$Has_Low_Loading)      else NULL

      # -------- Top alert --------
      top_alert <- switch(
        rec_status,
        "none" = div(
          class = "alert alert-success d-flex align-items-center mb-3 p-3",
          bsicons::bs_icon("check-circle-fill", size = "1.4em"),
          div(class = "ms-3",
              tags$div(class = "small text-uppercase fw-bold",
                       style = "letter-spacing:0.05em;", "Result"),
              tags$div(style = "font-size:1.05rem;",
                       "No problematic items detected. The current item set is clean."))
        ),
        "optimal" = div(
          class = "alert alert-success d-flex align-items-center mb-3 p-3",
          bsicons::bs_icon("trophy-fill", size = "1.4em"),
          div(class = "ms-3",
              tags$div(class = "small text-uppercase fw-bold",
                       style = "letter-spacing:0.05em;", "Optimal Strategy"),
              tags$div(style = "font-size:1.1rem;",
                       tags$b("Remove: "), rec_removed),
              tags$div(class = "small text-muted mt-1",
                       "Use the Exclude Variables tab to remove these items from the active dataset."))
        ),
        "fallback" = div(
          class = "alert alert-warning d-flex align-items-center mb-3 p-3",
          bsicons::bs_icon("exclamation-triangle-fill", size = "1.4em"),
          div(class = "ms-3",
              tags$div(class = "small text-uppercase fw-bold",
                       style = "letter-spacing:0.05em;", "Best Available Strategy"),
              tags$div(style = "font-size:1.1rem;",
                       tags$b("Remove: "), rec_removed),
              tags$div(class = "small mt-1",
                       "No fully clean removal combination was found. ",
                       "The top row of the comparison table is shown. Use Exclude Variables ",
                       "if you decide to remove these items."))
        )
      )

      # -------- Settings note (auto vs manual factor count) --------
      settings_note <- div(
        class = "small text-muted mb-3",
        bsicons::bs_icon("info-circle"), " ",
        if (auto_flag) {
          paste0("Number of factors determined automatically by parallel analysis: ",
                 n_factors, ".")
        } else {
          paste0("Number of factors set manually: ", n_factors, ".")
        },
        " Extraction: ", toupper(settings$extract %||% "-"),
        " | Rotation: ", tools::toTitleCase(settings$rotate %||% "-"),
        " | Correlation: ", tools::toTitleCase(settings$cor_method %||% "-"),
        " | Search: ", gsub("_", " ", tools::toTitleCase(settings$search %||% "-")), "."
      )

      # -------- Dataset overview row --------
      overview_row <- div(
        class = "row g-2 mb-3",
        div(class = "col-md-3", metric_box("Items",         n_items,            "primary")),
        div(class = "col-md-3", metric_box("Sample (n)",    n_obs,              "primary")),
        div(class = "col-md-3", metric_box("Factors",       n_factors,          "primary")),
        div(class = "col-md-3", metric_box("Initial Alpha", fmt_num(init_alpha), "info"))
      )

      # -------- Comparison row (only when a removal is recommended) --------
      comparison_row <- if (rec_status %in% c("optimal", "fallback")) {
        div(
          class = "row g-2 mb-3",
          div(class = "col-md-3",
              metric_box("Variance (original)", fmt_pct(init_var), "secondary")),
          div(class = "col-md-3",
              metric_box("Variance (after removal)", fmt_pct(rec_var), "success")),
          div(class = "col-md-3",
              metric_box("Alpha (after removal)", fmt_num(rec_alpha), "success")),
          div(class = "col-md-3",
              metric_box("Loading range",
                         rec_loadrange,
                         if (!is.null(rec_cross) &&
                             tolower(rec_cross) == "no" &&
                             !is.null(rec_low) &&
                             tolower(rec_low) == "no") "success" else "warning"))
        )
      } else {
        div(
          class = "row g-2 mb-3",
          div(class = "col-md-6",
              metric_box("Variance Explained", fmt_pct(init_var), "info")),
          div(class = "col-md-6",
              metric_box("Initial Alpha", fmt_num(init_alpha), "info"))
        )
      }

      # -------- Problem-item breakdown (always shown when problems exist) --------
      problems_row <- if (n_total_problems > 0) {
        div(
          class = "row g-2 mb-3",
          div(class = "col-md-4", metric_box("Cross-loading (>=3 factors)",
                                             n_cross3, "danger")),
          div(class = "col-md-4", metric_box("Cross-loading (>=2 factors)",
                                             n_cross2, "warning")),
          div(class = "col-md-4", metric_box("Low-loading items",
                                             n_lowload, "warning"))
        )
      } else NULL

      tagList(top_alert, settings_note, overview_row, comparison_row, problems_row)
    })

    output$removal_summary_table <- renderTable({
      req(analysis_results())
      res <- analysis_results()$res
      if (is.null(res)) return(NULL)
      tab <- res$removal_summary
      if (is.null(tab)) {
        return(data.frame(
          Status = "No removal combinations were tested (no problematic items detected)."
        ))
      }

      display_cols <- c(
        "Iteration",
        "Removed_This_Step",
        "Removed_Items",
        "N_Removed",
        "N_Remaining",
        "Total_Explained_Var",
        "Factor_Loading_Range",
        "Cronbachs_Alpha",
        "Has_Cross_Loading",
        "Cross_Loading_Items",
        "Has_Low_Loading",
        "Low_Loading_Items"
      )
      tab <- tab[, intersect(display_cols, names(tab)), drop = FALSE]
      if ("Total_Explained_Var" %in% names(tab)) {
        tab$Total_Explained_Var <- round(tab$Total_Explained_Var, 3)
      }
      if ("Cronbachs_Alpha" %in% names(tab)) {
        tab$Cronbachs_Alpha <- round(tab$Cronbachs_Alpha, 3)
      }
      tab
    }, striped = TRUE, hover = TRUE, digits = 3)
  })
}

#' EFA Replicability Server Module
#' @param id Module namespace ID.
#' @param data Input data (reactive).
#' @noRd
efa_server_replicability <- function(id, data) {
  moduleServer(id, function(input, output, session) {
    result <- reactiveVal(NULL)

    clean_numeric_complete <- function(d) {
      if (is.null(d)) return(NULL)
      d <- as.data.frame(d)
      d <- d[, sapply(d, is.numeric), drop = FALSE]
      if (ncol(d) < 3) return(NULL)
      d <- stats::na.omit(d)
      if (nrow(d) < 60) return(NULL)
      zero_var <- vapply(d, stats::var, numeric(1), na.rm = TRUE) == 0
      if (any(zero_var)) d <- d[, !zero_var, drop = FALSE]
      if (ncol(d) < 3) return(NULL)
      d
    }

    run_efa <- function(d, nfactors, rotate, fm, cor_kind) {
      psych::fa(
        r        = d,
        nfactors = nfactors,
        rotate   = rotate,
        fm       = fm,
        cor      = if (cor_kind == "pea") "cor" else "poly"
      )
    }

    loading_matrix <- function(fa_obj, nfactors) {
      loads <- as.matrix(unclass(fa_obj$loadings))
      loads <- loads[, seq_len(nfactors), drop = FALSE]
      colnames(loads) <- paste0("F", seq_len(nfactors))
      loads
    }

    align_loadings <- function(load_1, load_2) {
      nfactors <- ncol(load_1)
      if (nfactors == 1) {
        if (sum(load_1[, 1] * load_2[, 1], na.rm = TRUE) < 0) {
          load_2[, 1] <- -load_2[, 1]
        }
        return(list(load_2 = load_2, congruence = data.frame(Factor = "F1", Congruence = psych::factor.congruence(load_1, load_2)[1, 1])))
      }

      congr <- psych::factor.congruence(load_1, load_2)
      pool <- as.data.frame(as.table(abs(congr)), stringsAsFactors = FALSE)
      names(pool) <- c("Factor_1", "Factor_2", "Abs_Congruence")
      pool <- pool[order(pool$Abs_Congruence, decreasing = TRUE), , drop = FALSE]

      matched_1 <- character(0)
      matched_2 <- character(0)
      pairs <- data.frame(Factor_1 = character(0), Factor_2 = character(0), stringsAsFactors = FALSE)
      for (i in seq_len(nrow(pool))) {
        p1 <- as.character(pool$Factor_1[i])
        p2 <- as.character(pool$Factor_2[i])
        if (!(p1 %in% matched_1) && !(p2 %in% matched_2)) {
          pairs <- rbind(pairs, data.frame(Factor_1 = p1, Factor_2 = p2, stringsAsFactors = FALSE))
          matched_1 <- c(matched_1, p1)
          matched_2 <- c(matched_2, p2)
        }
      }

      ordered_2 <- load_2[, pairs$Factor_2, drop = FALSE]
      colnames(ordered_2) <- pairs$Factor_1
      ordered_2 <- ordered_2[, colnames(load_1), drop = FALSE]

      for (factor_name in colnames(load_1)) {
        if (sum(load_1[, factor_name] * ordered_2[, factor_name], na.rm = TRUE) < 0) {
          ordered_2[, factor_name] <- -ordered_2[, factor_name]
        }
      }

      aligned_congr <- psych::factor.congruence(load_1, ordered_2)
      congr_tbl <- data.frame(
        Factor = colnames(load_1),
        Congruence = diag(aligned_congr),
        stringsAsFactors = FALSE
      )

      list(load_2 = ordered_2, congruence = congr_tbl)
    }

    build_comparison <- function(load_1, load_2) {
      out <- data.frame(Item = rownames(load_1), stringsAsFactors = FALSE)
      diffs <- matrix(NA_real_, nrow = nrow(load_1), ncol = ncol(load_1))

      for (j in seq_len(ncol(load_1))) {
        factor_name <- colnames(load_1)[j]
        s1_name <- paste0("Split1_", factor_name)
        s2_name <- paste0("Split2_", factor_name)
        diff_name <- paste0("Diff_", factor_name)

        out[[s1_name]] <- load_1[, j]
        out[[s2_name]] <- load_2[, j]
        out[[diff_name]] <- abs(load_1[, j] - load_2[, j])
        diffs[, j] <- out[[diff_name]]
      }

      out$MaxAbsDiff <- apply(diffs, 1, max, na.rm = TRUE)
      out$Primary_Split1 <- colnames(load_1)[max.col(abs(load_1), ties.method = "first")]
      out$Primary_Split2 <- colnames(load_2)[max.col(abs(load_2), ties.method = "first")]
      out$Same_Primary <- out$Primary_Split1 == out$Primary_Split2
      out
    }

    observeEvent(list(data(), input$cor_kind, input$number_factor, input$fact_method,
                      input$rotating_method, input$split_prop, input$seed), {
      result(NULL)
    }, ignoreInit = TRUE)

    observeEvent(input$run_replicability, {
      d <- clean_numeric_complete(data())
      if (is.null(d)) {
        showNotification(
          "Replicability requires at least 60 complete cases and 3 numeric variables after cleaning.",
          type = "error", duration = 8
        )
        return()
      }

      nfactors <- as.numeric(input$number_factor)
      if (nfactors < 1 || nfactors >= ncol(d)) {
        showNotification("Number of factors must be smaller than the number of variables.", type = "error", duration = 8)
        return()
      }

      split_n <- floor(nrow(d) * as.numeric(input$split_prop))
      if (min(split_n, nrow(d) - split_n) < 30) {
        showNotification("Each split must contain at least 30 complete cases.", type = "error", duration = 8)
        return()
      }

      showNotification("Running EFA replicability analysis...", type = "message", duration = NULL, id = "rep_progress")

      tryCatch({
        set.seed(as.integer(input$seed))
        idx_1 <- sample(seq_len(nrow(d)), split_n)
        d1 <- d[idx_1, , drop = FALSE]
        d2 <- d[-idx_1, , drop = FALSE]

        fa_1 <- run_efa(d1, nfactors, input$rotating_method, input$fact_method, input$cor_kind)
        fa_2 <- run_efa(d2, nfactors, input$rotating_method, input$fact_method, input$cor_kind)

        load_1 <- loading_matrix(fa_1, nfactors)
        load_2 <- loading_matrix(fa_2, nfactors)
        aligned <- align_loadings(load_1, load_2)
        comp <- build_comparison(load_1, aligned$load_2)

        summary <- data.frame(
          Metric = c(
            "Total complete cases",
            "Split 1 cases",
            "Split 2 cases",
            "Variables",
            "Factors",
            "Correlation",
            "Extraction",
            "Rotation",
            "Mean absolute loading difference",
            "Items with same primary factor"
          ),
          Value = c(
            nrow(d),
            nrow(d1),
            nrow(d2),
            ncol(d),
            nfactors,
            if (input$cor_kind == "pea") "Pearson" else "Polychoric",
            input$fact_method,
            input$rotating_method,
            round(mean(comp$MaxAbsDiff, na.rm = TRUE), 3),
            paste0(sum(comp$Same_Primary, na.rm = TRUE), " / ", nrow(comp))
          ),
          stringsAsFactors = FALSE
        )

        result(list(
          comparison = comp,
          summary = summary,
          congruence = aligned$congruence
        ))

        removeNotification("rep_progress")
        showNotification("Replicability analysis complete.", type = "message", duration = 4)
      }, error = function(e) {
        removeNotification("rep_progress")
        result(NULL)
        showNotification(paste("Replicability failed:", e$message), type = "error", duration = 10)
      })
    })

    output$loadings_comparison <- renderTable({
      req(result())
      result()$comparison
    }, digits = 3, striped = TRUE, bordered = TRUE)

    output$summary_table <- renderTable({
      req(result())
      result()$summary
    }, striped = TRUE, bordered = TRUE)

    output$factor_congruence <- renderTable({
      req(result())
      congr <- result()$congruence
      congr$Congruence <- round(congr$Congruence, 3)
      congr
    }, striped = TRUE, bordered = TRUE)

    output$download_replicability <- downloadHandler(
      filename = "efa_replicability_loadings.csv",
      content = function(file) {
        req(result())
        utils::write.csv(result()$comparison, file, row.names = FALSE)
      }
    )
  })
}

#' EFA UI Modules
#' @import shiny
#' @import bslib
#' @importFrom bsicons bs_icon
#' @noRd

# 1. Factor Retention
efa_ui_fac_ret <- function(id) {
  ns <- NS(id)
  tagList(
    layout_columns(
      col_widths = c(4, 8),
      card(
        card_header("Retention Methods", class = "bg-primary text-white", bs_icon("gear")),
        card_body(
          radioButtons(ns("fac_ret_cor_kind"), "Correlation:",
                       choices  = c("Pearson" = "pea", "Polychoric" = "poly"),
                       selected = "poly", inline = TRUE),
          selectInput(ns("dimension_methods"), "Select Method:",
            choices = c(
              "Optimal Parallel Analysis (MRFA)" = "pa_mrfa",
              "Traditional Parallel Analysis"    = "pa_traditional",
              "Hull Method"                       = "hull_method",
              "MAP (Original)"                    = "map_method_tra",
              "MAP (Revised)"                     = "map_method_rev",
              "EGA (TMFG)"                        = "EGA_tmfg",
              "EGA (Glasso)"                      = "EGA_glasso",
              "Empirical Kaiser (EKC)"            = "EK_C",
              "Comparison Data (CD)"              = "comp_data_method",
              "Scree Plot"                        = "scree_plot"
            ), selected = "hull_method"),
          actionButton(ns("run_factor_ret"), "Run Analysis",
                       icon = icon("play"), class = "btn-success w-100")
        )
      ),
      card(
        card_header("Results"),
        card_body(
          div(
            class = "alert alert-secondary p-2 mb-3",
            style = "font-size:0.82rem;",
            bsicons::bs_icon("play-circle"), " ",
            "Select a method and correlation type, then press ",
            tags$b("Run Analysis"), " to compute results."
          ),
          tableOutput(ns("dim_ret_results")) %>% withSpinner(type = 8, color = "#2C3E50"),
          plotOutput(ns("scree_plot"), height = "300px")
        )
      )
    )
  )
}

# 2. EFA Setup
efa_ui_analysis <- function(id) {
  ns <- NS(id)
  tagList(
    card(
      card_header("EFA Configuration", class = "bg-primary text-white", bs_icon("sliders")),
      card_body(
        style = "overflow: visible; min-height: 620px;",
        div(
          class = "d-grid gap-3",
          div(
            class = "p-3 rounded",
            style = "background:#f8fafc; border:1px solid #e2e8f0;",
            radioButtons(
              ns("cor_kind"), "Correlation:",
              choices = c("Pearson" = "pea", "Polychoric" = "poly"),
              selected = "poly",
              inline = TRUE
            ),
            tags$p(
              class = "text-muted small mb-0",
              "Use Polychoric for ordinal/Likert items; Pearson is faster for continuous data."
            )
          ),
          numericInput(ns("number_factor"), "No. of Factors:", value = 1, min = 1),
          selectizeInput(
            ns("fact_method"), "Extraction:",
            choices = c(
              "Minimum Residuals"         = "minres",
              "Maximum Likelihood"        = "ml",
              "Principal Axis"            = "pa",
              "Unweighted Least Squares"  = "uls",
              "Weighted Least Squares"    = "wls",
              "Minimum Rank"              = "minrank",
              "Minimum Chi-Square"        = "minchi",
              "Generalized Least Squares" = "gls"
            ),
            selected = "minres",
            options = list(dropdownParent = "body")
          ),
          selectizeInput(
            ns("rotating_method"), "Rotation:",
            choices = list(
              "None" = list("None" = "none"),
              "Oblique" = list(
                "Oblimin"    = "oblimin",
                "Promax"     = "promax",
                "Quartimin"  = "quartimin",
                "BiquartMin" = "biquartimin",
                "GeominQ"    = "geominQ",
                "BentlerQ"   = "bentlerQ",
                "Simplimax"  = "simplimax",
                "Cluster"    = "cluster"
              ),
              "Orthogonal" = list(
                "Varimax"    = "varimax",
                "Quartimax"  = "quartimax",
                "Equamax"    = "equamax",
                "BentlerT"   = "bentlerT",
                "GeominT"    = "geominT",
                "Bifactor"   = "bifactor"
              )
            ),
            selected = "oblimin",
            options = list(dropdownParent = "body")
          )
        ),
        hr(),
        actionButton(ns("run_efa"), "Run EFA",
                     icon = icon("play"), class = "btn-success w-100 btn-lg")
      )
    )
  )
}

# 3. EFA Report
efa_ui_report <- function(id) {
  ns <- NS(id)
  tagList(
    card(
      card_header("Sampling Adequacy & Sphericity", bs_icon("check-circle")),
      card_body(
        layout_columns(
          col_widths = c(5, 7),
          div(
            tags$p(class = "text-muted small mb-1", "KMO Measure of Sampling Adequacy"),
            uiOutput(ns("kmo_result"))
          ),
          div(
            tags$p(class = "text-muted small mb-1", "Bartlett's Test of Sphericity"),
            tableOutput(ns("bartlett"))
          )
        )
      )
    ),
    card(
      card_header("Factor Solution & Visualisation", bs_icon("table")),
      navset_card_tab(
        nav_panel(
          title = "Heatmap", icon = bs_icon("grid-3x3"),
          layout_columns(
            col_widths = c(4, 8),
            div(
              selectInput(ns("heatmap_palette"), "Color Palette:",
                          choices = c(
                            "Blue - Red (default)" = "blue_red",
                            "Viridis"               = "viridis",
                            "Warm"                  = "warm",
                            "Cool (Blue)"           = "cool",
                            "Grayscale"             = "grayscale"
                          ),
                          selected = "blue_red"),
              checkboxInput(ns("heatmap_show_values"),
                            "Show numeric values", value = TRUE),
              p(class = "text-muted small",
                "Values shown only if number of variables < 25.")
            ),
            plotOutput(ns("heat_map"), height = "480px") %>%
              withSpinner(type = 8, color = "#2C3E50")
          ),
          div(
            class = "mt-2 p-2 rounded",
            style = "background:#f1f5f9; font-size:0.82rem; color:#475569;",
            bs_icon("info-circle"), " ",
            textOutput(ns("cor_range_text"), inline = TRUE)
          )
        ),
        nav_panel(
          title = "Loadings", icon = bs_icon("list-ol"),
          tableOutput(ns("efa_result_str")),
          downloadButton(ns("download_efa_loadings"), "Download CSV", class = "btn-sm mt-2")
        ),
        nav_panel("Variance Explained", tableOutput(ns("efa_result_expl_var"))),
        nav_panel("Factor Correlations (Phi)", tableOutput(ns("efa_result_interf_cor")))
      )
    )
  )
}

#' Measurement Invariance Server Module
#' @param id Module namespace ID.
#' @param data Input data (reactive)
#' @noRd
inv_server <- function(id, data) {
  moduleServer(id, function(input, output, session) {

    analysis_results_rv <- reactiveValues(
      fit_measures_df = NULL,
      model_comparison_df = NULL
    )


    observe({
      req(data())
      current_data <- data()
      col_names <- names(current_data)


      updateSelectizeInput(session, "builder_items", choices = col_names)
      updateSelectizeInput(session, "builder_cov_items", choices = col_names)

      is_cat <- sapply(current_data, function(x) is.factor(x) || is.character(x) || (is.numeric(x) && length(unique(stats::na.omit(x))) <= 10))
      updateSelectInput(session, "grouping_variable_select", choices = c("", col_names[is_cat]))
    })

    append_syntax <- function(new_text) {
      current <- input$inv_model_syntax
      if (is.null(current) || current == "") {
        updateTextAreaInput(session, "inv_model_syntax", value = new_text)
      } else {
        updateTextAreaInput(session, "inv_model_syntax", value = paste(current, new_text, sep = "\n"))
      }
    }

    observeEvent(input$btn_add_to_model, {
      req(input$builder_factor_name, input$builder_items)
      f_name <- trimws(input$builder_factor_name)
      items_str <- paste(input$builder_items, collapse = " + ")
      new_line <- paste0(f_name, " =~ ", items_str)
      append_syntax(new_line)

      updateTextInput(session, "builder_factor_name", value = "")
      updateSelectizeInput(session, "builder_items", selected = character(0))
    })

    observeEvent(input$btn_add_cov, {
      req(input$builder_cov_items)
      if(length(input$builder_cov_items) != 2) {
        showNotification("Select exactly 2 variables for covariance.", type = "warning")
        return()
      }
      new_line <- paste0(input$builder_cov_items[1], " ~~ ", input$builder_cov_items[2])
      append_syntax(new_line)

      updateSelectizeInput(session, "builder_cov_items", selected = character(0))
    })

    observeEvent(input$correlation_matrix_type, {
      req(input$correlation_matrix_type)
      estimator_choices <- if (input$correlation_matrix_type == "pea") {
        c("Default (MLR)" = "default", "MLR", "ML", "GLS")
      } else {
        c("Default (WLSMV)" = "default", "WLSMV", "ULSMV", "DWLS")
      }
      updateSelectInput(
        session,
        "invariance_estimator_select",
        choices = estimator_choices,
        selected = "default"
      )
    }, ignoreInit = FALSE)

    observeEvent(input$run_invariance_button, {
      validate(
        need(data(), "Upload data."),
        need(input$inv_model_syntax, "Define model."),
        need(input$grouping_variable_select, "Select grouping var."),
        need(length(input$invariance_levels_checkbox) > 0, "Select levels.")
      )

      clean_res <- clean_missing_data(data())
      dat <- clean_res$cleaned_data

      grp <- input$grouping_variable_select
      dat[[grp]] <- as.factor(dat[[grp]])

      is_poly <- (input$correlation_matrix_type %||% "pea") == "poly"
      est_sel <- input$invariance_estimator_select %||% "default"
      est <- if (est_sel == "default") {
        if(isTRUE(is_poly)) "WLSMV" else "MLR"
      } else {
        est_sel
      }
      manifest_vars <- unique(unlist(lavaan::lavaanify(input$inv_model_syntax)$rhs[lavaan::lavaanify(input$inv_model_syntax)$op == "=~"]))
      ordered_arg <- if(isTRUE(is_poly)) manifest_vars else FALSE

      extract_fit_indices <- function(model, model_name) {
        fm <- tryCatch(lavaan::fitMeasures(model), error = function(e) NULL)
        if(is.null(fm)) return(NULL)

        chi <- fm["chisq.scaled"] %||% fm["chisq"]
        df  <- fm["df.scaled"] %||% fm["df"]
        p   <- fm["pvalue.scaled"] %||% fm["pvalue"]
        cfi <- fm["cfi.scaled"] %||% fm["cfi.robust"] %||% fm["cfi"]
        tli <- fm["tli.scaled"] %||% fm["tli.robust"] %||% fm["tli"]
        rmsea     <- fm["rmsea.scaled"] %||% fm["rmsea.robust"] %||% fm["rmsea"]
        rmsea_low <- fm["rmsea.ci.lower.scaled"] %||% fm["rmsea.ci.lower.robust"] %||% fm["rmsea.ci.lower"]
        rmsea_upp <- fm["rmsea.ci.upper.scaled"] %||% fm["rmsea.ci.upper.robust"] %||% fm["rmsea.ci.upper"]
        srmr <- fm["srmr"] %||% fm["srmr_bentler"]

        data.frame(
          Model = model_name,
          ChiSq = round(chi, 2),
          df = df,
          p = ifelse(is.na(p), NA, format.pval(p, digits = 3, eps = 0.001)),
          CFI = round(cfi, 3),
          TLI = round(tli, 3),
          RMSEA = round(rmsea, 3),
          RMSEA_90_CI_Lower = round(rmsea_low, 3),
          RMSEA_90_CI_Upper = round(rmsea_upp, 3),
          SRMR = round(srmr, 3),
          check.names = FALSE
        )
      }

      showNotification("Running Analysis...", type="message")

      tryCatch({
        analysis_results_rv$fit_measures_df <- NULL
        analysis_results_rv$model_comparison_df <- NULL

        mods <- list()
        levels <- input$invariance_levels_checkbox

        run_mod <- function(lvl, eq=NULL) {
          tryCatch(
            lavaan::cfa(
              input$inv_model_syntax,
              data = dat,
              group = grp,
              group.equal = eq,
              estimator = est,
              ordered = ordered_arg,
              missing = "listwise",
              mimic = "Mplus"
            ),
            error=function(e) NULL
          )
        }

        metric_constraints <- "loadings"
        scalar_constraints <- if (isTRUE(is_poly)) {
          c("loadings", "thresholds")
        } else {
          c("loadings", "intercepts")
        }
        strict_constraints <- if (isTRUE(is_poly)) {
          c("loadings", "thresholds", "residuals")
        } else {
          c("loadings", "intercepts", "residuals")
        }

        if("configural" %in% levels) mods$configural <- run_mod("Configural")
        if("metric" %in% levels)     mods$metric     <- run_mod("Metric", metric_constraints)
        if("scalar" %in% levels)     mods$scalar     <- run_mod("Scalar", scalar_constraints)
        if("strict" %in% levels)     mods$strict     <- run_mod("Strict", strict_constraints)

        # Fit Measures Table
        res_list <- lapply(names(mods), function(n) {
          m <- mods[[n]]
          if(is.null(m)) return(NULL)
          extract_fit_indices(m, n)
        })
        analysis_results_rv$fit_measures_df <- do.call(rbind, res_list)

        # Comparison
        valid_mods <- Filter(Negate(is.null), mods)
        if(length(valid_mods) > 1) {
          lrt <- tryCatch(as.data.frame(do.call(lavaan::lavTestLRT, unname(valid_mods))), error = function(e) NULL)
          fit_df <- analysis_results_rv$fit_measures_df
          comp_df <- fit_df
          comp_df$Compared_To <- c(NA, head(comp_df$Model, -1))
          comp_df$Delta_CFI <- round(c(NA, diff(comp_df$CFI)), 3)
          comp_df$Delta_TLI <- round(c(NA, diff(comp_df$TLI)), 3)
          comp_df$Delta_RMSEA <- round(c(NA, diff(comp_df$RMSEA)), 3)
          comp_df$Delta_SRMR <- round(c(NA, diff(comp_df$SRMR)), 3)

          if(!is.null(lrt) && nrow(lrt) == nrow(comp_df)) {
            lrt_cols <- intersect(c("Chisq diff", "Df diff", "Pr(>Chisq)"), names(lrt))
            if(length(lrt_cols) > 0) {
              comp_df <- cbind(comp_df, lrt[, lrt_cols, drop = FALSE])
            }
          }

          analysis_results_rv$model_comparison_df <- comp_df[, c(
            "Model", "Compared_To", "CFI", "TLI", "RMSEA", "SRMR",
            "Delta_CFI", "Delta_TLI", "Delta_RMSEA", "Delta_SRMR",
            setdiff(names(comp_df), c("Model", "Compared_To", "CFI", "TLI", "RMSEA", "SRMR",
                                      "Delta_CFI", "Delta_TLI", "Delta_RMSEA", "Delta_SRMR"))
          )]
        }

      }, error = function(e) showNotification(e$message, type="error"))
    })

    output$invariance_fit_measures_table <- renderTable({
      analysis_results_rv$fit_measures_df
    }, rownames=FALSE, striped=TRUE, digits=3)

    output$model_comparison_table <- renderTable({
      tab <- analysis_results_rv$model_comparison_df
      delta_cols <- intersect(
        c("Delta_CFI", "Delta_TLI", "Delta_RMSEA", "Delta_SRMR"),
        names(tab)
      )
      for (col in delta_cols) {
        tab[[col]] <- ifelse(
          is.na(tab[[col]]),
          NA,
          formatC(tab[[col]], digits = 3, format = "f")
        )
      }
      tab
    }, rownames=FALSE, striped=TRUE, digits=3)

    output$download_fit_measures_button <- downloadHandler(
      filename="inv_fit.csv", content=function(f) {
        req(analysis_results_rv$fit_measures_df)
        write.csv(analysis_results_rv$fit_measures_df, f, row.names=FALSE)
      }
    )
    output$download_model_comparison_button <- downloadHandler(
      filename="inv_comp.csv", content=function(f) {
        req(analysis_results_rv$model_comparison_df)
        write.csv(analysis_results_rv$model_comparison_df, f, row.names=FALSE)
      }
    )
  })
}

#' Data Selection Server Logic (Modern)
#' @param id Module namespace ID.
#' @param data Input data (reactive).
#' @importFrom rlang .data
#' @noRd
data_selection_server <- function(id, data) {
  moduleServer(id, function(input, output, session) {

    analyzed_data <- eventReactive(input$analyze_data, {
      req(data())
      data()
    })

    # ---- Preview Table ----
    output$mydatatable <- renderTable({
      validate(need(data(), "Please upload your dataset."))
      utils::head(data(), 10)
    })

    # ---- Helper: numeric subset ----
    get_numeric_data <- reactive({
      req(analyzed_data())
      df <- analyzed_data()
      df[, sapply(df, is.numeric), drop = FALSE]
    })

    # ---- Value boxes (top row) ----
    output$n_var_box <- renderUI({
      req(analyzed_data())
      value_box(title = "Variables", value = ncol(analyzed_data()),
                showcase = bsicons::bs_icon("columns"), theme = "primary")
    })

    output$n_obs_box <- renderUI({
      req(analyzed_data())
      value_box(title = "Sample Size", value = nrow(analyzed_data()),
                showcase = bsicons::bs_icon("people"), theme = "success")
    })

    output$n_missing_box <- renderUI({
      req(analyzed_data())
      n_miss <- sum(is.na(analyzed_data()))
      pct    <- round(100 * n_miss / (nrow(analyzed_data()) * ncol(analyzed_data())), 1)
      value_box(title = "Missing Cells",
                value = paste0(n_miss, " (", pct, "%)"),
                showcase = bsicons::bs_icon("question-octagon"),
                theme    = if (pct > 5) "warning" else "info")
    })

    output$n_complete_box <- renderUI({
      req(analyzed_data())
      n_complete <- sum(stats::complete.cases(analyzed_data()))
      pct <- round(100 * n_complete / nrow(analyzed_data()), 1)
      value_box(title = "Complete Cases",
                value = paste0(n_complete, " (", pct, "%)"),
                showcase = bsicons::bs_icon("check-circle"),
                theme    = "secondary")
    })

    # ---- Variable type breakdown ----
    output$var_types_table <- renderTable({
      req(analyzed_data())
      df <- analyzed_data()
      types <- sapply(df, function(x) {
        if (is.numeric(x))  "Numeric"
        else if (is.factor(x))  "Factor"
        else if (is.character(x)) "Character"
        else if (is.logical(x))  "Logical"
        else class(x)[1]
      })
      tab <- as.data.frame(table(Type = types), responseName = "Count")
      tab$Percent <- paste0(round(100 * tab$Count / sum(tab$Count), 1), "%")
      tab
    })

    # ---- Per-variable summary (single pass per column) ----
    output$variable_summary_table <- renderTable({
      req(analyzed_data())
      df <- analyzed_data()
      # Compute all stats in ONE iteration over columns
      summarise_col <- function(x) {
        is_num <- is.numeric(x)
        no_na  <- if (any(is.na(x))) x[!is.na(x)] else x
        list(
          Type     = class(x)[1],
          N_Unique = length(unique(no_na)),
          Missing  = length(x) - length(no_na),
          Min      = if (is_num && length(no_na)) min(no_na)             else NA,
          Max      = if (is_num && length(no_na)) max(no_na)             else NA,
          Mean     = if (is_num && length(no_na)) round(mean(no_na), 2)  else NA,
          SD       = if (is_num && length(no_na) > 1)
                       round(stats::sd(no_na), 2) else NA
        )
      }
      stats_list <- lapply(df, summarise_col)
      out <- data.frame(
        Variable = names(df),
        do.call(rbind.data.frame, stats_list),
        check.names = FALSE, stringsAsFactors = FALSE, row.names = NULL
      )
      out
    }, striped = TRUE, bordered = TRUE, na = "-", digits = 2)

    # ---- Categories distribution: counts per unique value level ----
    output$category_dist_plot <- renderPlot({
      req(analyzed_data())
      df <- analyzed_data()
      num_df <- df[, sapply(df, is.numeric), drop = FALSE]
      if (ncol(num_df) == 0) {
        graphics::plot.new(); graphics::title("No numeric variables.")
        return()
      }

      # Detect "categorical-like" variables: integer-like with <= 10 unique levels
      is_likert <- sapply(num_df, function(x) {
        u <- unique(stats::na.omit(x))
        length(u) <= 10 && all(u == round(u))
      })

      if (any(is_likert)) {
        cat_df <- num_df[, is_likert, drop = FALSE]
        # build long data frame: variable + value
        long <- do.call(rbind, lapply(names(cat_df), function(v) {
          data.frame(Variable = v, Value = cat_df[[v]], stringsAsFactors = FALSE)
        }))
        long <- long[!is.na(long$Value), ]
        ggplot2::ggplot(long, ggplot2::aes(x = factor(.data[["Value"]]))) +
          ggplot2::geom_bar(fill = "#2563EB", alpha = 0.85) +
          ggplot2::facet_wrap(~ Variable, scales = "free_y") +
          ggplot2::labs(x = "Category", y = "Count",
                        title = "Response Distribution per Variable") +
          ggplot2::theme_minimal(base_size = 11) +
          ggplot2::theme(strip.text = ggplot2::element_text(face = "bold"))
      } else {
        # Continuous case: show histograms
        long <- do.call(rbind, lapply(names(num_df), function(v) {
          data.frame(Variable = v, Value = num_df[[v]], stringsAsFactors = FALSE)
        }))
        long <- long[!is.na(long$Value), ]
        ggplot2::ggplot(long, ggplot2::aes(x = .data[["Value"]])) +
          ggplot2::geom_histogram(fill = "#16A34A", alpha = 0.85, bins = 25) +
          ggplot2::facet_wrap(~ Variable, scales = "free") +
          ggplot2::labs(x = NULL, y = "Count",
                        title = "Distribution per Variable") +
          ggplot2::theme_minimal(base_size = 11)
      }
    })
  })
}

#' Assumptions UI Module
#' @import shiny
#' @import bslib
#' @importFrom bsicons bs_icon
#' @noRd
assumptions_ui <- function(id) {
  ns <- NS(id)
  tagList(
    # Descriptive Statistics
    card(
      card_header("Descriptive Statistics", class = "bg-info text-white", bs_icon("table")),
      card_body(
        actionButton(ns("run_descriptives_button"), "Calculate Descriptives", icon = icon("calculator"), class = "btn-info mb-3"),
        tableOutput(ns("descriptives_table_output")) %>% withSpinner(type = 8, color = "#2C3E50"),
        downloadButton(ns("download_descriptives_button"), "Download CSV")
      )
    ),

    # Collinearity & Normality Side-by-Side
    layout_columns(
      col_widths = c(6, 6),
      card(
        card_header("Collinearity", bs_icon("link-45deg")),
        card_body(
          p("Check for multicollinearity using VIF, Tolerance (TOL), and Condition Index (CI)."),
          actionButton(ns("run_collinearity_button"), "Run Collinearity Check", class = "btn-secondary w-100 mb-2"),
          tableOutput(ns("collinearity_table_output")),
          div(
            class = "mt-2 p-2 rounded",
            style = "background:#f1f5f9; font-size:0.78rem; color:#475569; line-height:1.6;",
            tags$b("Interpretation guidelines:"), tags$br(),
            tags$b("VIF:"), " < 5 acceptable, < 10 maximum threshold.", tags$br(),
            tags$b("TOL:"), " > 0.20 acceptable, > 0.10 minimum threshold.", tags$br(),
            tags$b("CI:"), " < 10 no collinearity, 10-30 moderate, > 30 severe."
          )
        )
      ),
      card(
        card_header("Multivariate Normality", bs_icon("graph-up-arrow")),
        card_body(
          p("Mardia's Skewness & Kurtosis, Energy Test."),
          actionButton(ns("run_normality_tests_button"), "Run Normality Tests", class = "btn-secondary w-100 mb-2"),
          tableOutput(ns("multivariate_normality_table_output"))
        )
      )
    )
  )
}

#' The application server-side
#' @import shiny
#' @noRd
app_server <- function(input, output, session) {

  # --- Initial Data Loading ---
  shared_data_reactive <- eventReactive(input[["data_selection-analyze_data"]], {
    inFile <- input[["data_selection-file1"]]
    user_has_header <- input[["data_selection-has_header_checkbox"]] %||% TRUE

    if (is.null(inFile)) return(NULL)

    tryCatch({
      ext <- tools::file_ext(tolower(inFile$name))
      df <- switch(ext,
                   "csv"  = utils::read.csv(inFile$datapath, header = user_has_header,
                                            na.strings = c("NA", "", " ", ".", "na", "NaN")),
                   "xlsx" = readxl::read_excel(inFile$datapath, col_names = user_has_header,
                                               na = c("NA", "", " ")),
                   "sav"  = haven::read_sav(inFile$datapath),
                   utils::read.table(inFile$datapath, header = user_has_header,
                                     na.strings = c("NA", "", " "))
      )
      cleaned_result <- clean_missing_data(df, remove_na = FALSE)
      if (!is.null(cleaned_result$cleaned_data)) return(cleaned_result$cleaned_data)
      return(NULL)
    }, error = function(e) {
      showNotification(paste("Error:", e$message), type = "error")
      return(NULL)
    })
  }, ignoreNULL = FALSE)

  # --- Data Selection ---
  data_selection_server("data_selection", data = shared_data_reactive)

  # --- Missing Value ---
  processed_missing_data <- mod_missing_server("missing_val", data = shared_data_reactive)

  # --- Wrangling Chain ---
  data_after_exclusion_reactive <- wrangling_server_ex_var(
    "wrangling_ex_var", data = processed_missing_data
  )

  data_for_outliers_module_input <- reactive({
    data_after_exclusion_reactive()
  })

  data_without_outliers_reactive <- wrangling_server_outliers(
    "wrangling_outliers", data = data_for_outliers_module_input
  )

  final_wrangled_data_reactive <- reactive({
    d <- data_without_outliers_reactive()
    if (!is.null(d)) return(d)
    data_after_exclusion_reactive()
  })

  wrangling_server_split("wrangling_split", data = final_wrangled_data_reactive)

  # --- Analysis Modules ---
  assumptions_server("assumptions", data = final_wrangled_data_reactive)
  mod_itemrest_server("item_rest", data = final_wrangled_data_reactive)
  efa_server_replicability("efa_replicability", data = final_wrangled_data_reactive)
  efa_server_fac_ret("efa_fac_ret", data = final_wrangled_data_reactive)

  # efa_server_analysis now returns a list with both efa_object and settings.
  # The previous cross-namespace moduleServer pattern is removed.
  efa_out <- efa_server_analysis("efa_analysis", data = final_wrangled_data_reactive)

  efa_server_report(
    "efa_report",
    data                   = final_wrangled_data_reactive,
    efa_output_reactive    = efa_out$efa_object,
    efa_settings_reactive  = efa_out$settings
  )

  ega_server("ega",                 data = final_wrangled_data_reactive)
  cfa_server("cfa",                 data = final_wrangled_data_reactive)
  inv_server("inv",                 data = final_wrangled_data_reactive)
  reliability_server("reliability", data = final_wrangled_data_reactive)
  item_weighting_server("item_weighting", data = final_wrangled_data_reactive)
  about_server("about")
}

#' EGA UI Module
#' @noRd
ega_ui <- function(id) {
  ns <- NS(id)
  tagList(
    layout_columns(
      col_widths = c(4, 8),

      # Sol Taraf: Ayarlar
      card(
        card_header("EGA Setup", class = "bg-primary text-white", bs_icon("diagram-2")),
        card_body(
          selectInput(ns("ega_estimation_method_select"), "Network Model:",
                      choices = c("TMFG" = "TMFG", "Glasso" = "glasso"), selected = "TMFG"),
          selectInput(ns("ega_algorithm_select"), "Community Detection:",
                      choices = c(
                        "Walktrap (default)"  = "walktrap",
                        "Louvain"             = "louvain",
                        "Leiden"              = "leiden",
                        "Fast Greedy"         = "fast_greedy",
                        "Edge Betweenness"    = "edge_betweenness",
                        "Label Propagation"   = "label_prop"
                      ), selected = "walktrap"),
          radioButtons(ns("ega_correlation_type_radio"), "Correlation:",
                       choices = c("Auto" = "cor_auto", "Pearson" = "pearson"), inline = TRUE),
          actionButton(ns("run_ega_button"), "Run EGA",
                       icon = icon("project-diagram"), class = "btn-success w-100")
        )
      ),

      # Sağ Taraf: Plot
      card(
        card_header("Network Plot"),
        plotOutput(ns("ega_network_plot_output"), height = "500px") %>% withSpinner(type = 8, color = "#2C3E50"),
        downloadButton(ns("download_ega_plot_button"), "Download Plot")
      )
    ),

    card(
      card_header("Dimensions & Structure"),
      navset_card_tab(
        nav_panel(
          title = "Item Allocation", icon = bs_icon("diagram-3"),
          verbatimTextOutput(ns("ega_dimensionality_summary_output")),
          tableOutput(ns("ega_item_community_table_output")) %>%
            withSpinner(type = 8, color = "#2C3E50")
        ),
        nav_panel(
          title = "Adjacency Matrix", icon = bs_icon("table"),
          div(
            style = "overflow-x: auto;",
            tableOutput(ns("ega_network_table_output"))
          ),
          downloadButton(ns("download_ega_network_button"), "Download CSV", class = "btn-sm mt-2")
        )
      )
    )
  )
}

#' Reliability Analysis Server Module
#' @param id Module namespace ID.
#' @param data Input data (reactive)
#' @noRd
reliability_server <- function(id, data) {
  moduleServer(id, function(input, output, session) {

    reliability_output_rv <- reactiveVal(NULL)

    # Cache results keyed by (method, items, cor_kind, model_str, strata) tuple
    # Avoids recomputing when user re-selects a previously calculated coefficient
    cache_rv <- reactiveValues(store = list())
    strata_choice_cache <- reactiveValues(values = list())

    # Update item selector when data changes; clear cache too
    observeEvent(data(), {
      req(data())
      updateSelectizeInput(session, "reliability_item_select", choices = names(data()))
      cache_rv$store <- list()
    }, ignoreNULL = TRUE)

    active_reliability_items <- reactive({
      req(data())
      selected <- input$reliability_item_select
      if (!is.null(selected) && length(selected) > 0) {
        selected
      } else {
        names(data())
      }
    })

    output$strata_item_selectors <- renderUI({
      items <- active_reliability_items()
      n_factors <- input$strata_factor_count %||% 2
      n_factors <- max(1, as.integer(n_factors))

      tagList(lapply(seq_len(n_factors), function(i) {
        selectizeInput(
          session$ns(paste0("strata_items_", i)),
          paste0("Factor ", i, " Items:"),
          choices = items,
          selected = character(0),
          multiple = TRUE,
          options = list(
            placeholder = paste0("Select items for factor ", i, "..."),
            plugins = list("remove_button")
          )
        )
      }))
    })

    strata_selections <- reactive({
      n_factors <- input$strata_factor_count %||% 2
      n_factors <- max(1, as.integer(n_factors))
      lapply(seq_len(n_factors), function(i) {
        input[[paste0("strata_items_", i)]] %||% character(0)
      })
    })

    observeEvent(
      list(active_reliability_items(), input$strata_factor_count, strata_selections()),
      {
        items <- active_reliability_items()
        n_factors <- input$strata_factor_count %||% 2
        n_factors <- max(1, as.integer(n_factors))
        selections <- strata_selections()

        for (i in seq_len(n_factors)) {
          current_selected <- intersect(selections[[i]] %||% character(0), items)
          selected_elsewhere <- unique(unlist(selections[-i], use.names = FALSE))
          selected_elsewhere <- intersect(selected_elsewhere, items)

          available_choices <- c(
            current_selected,
            setdiff(items, unique(c(current_selected, selected_elsewhere)))
          )

          input_id <- paste0("strata_items_", i)
          cached_choices <- strata_choice_cache$values[[input_id]] %||% character(0)

          if (!identical(cached_choices, available_choices)) {
            freezeReactiveValue(input, input_id)
            updateSelectizeInput(
              session,
              input_id,
              choices = available_choices,
              selected = current_selected
            )
            strata_choice_cache$values[[input_id]] <- available_choices
          }
        }
      },
      ignoreInit = FALSE
    )

    build_strata_definition <- function(items, n_factors) {
      assignments <- rep(NA_integer_, length(items))
      names(assignments) <- items
      duplicate_items <- character(0)

      for (i in seq_len(n_factors)) {
        selected <- input[[paste0("strata_items_", i)]] %||% character(0)
        selected <- intersect(selected, items)
        if (length(selected) == 0) next

        duplicate_items <- union(duplicate_items, selected[!is.na(assignments[selected])])
        assignments[selected] <- i
      }

      unassigned_items <- names(assignments)[is.na(assignments)]
      if (length(duplicate_items) > 0) {
        stop(
          "Each item can belong to only one factor. Duplicate item(s): ",
          paste(duplicate_items, collapse = ", ")
        )
      }
      if (length(unassigned_items) > 0) {
        stop(
          "Assign every item to a factor before calculating Stratified Alpha. Missing item(s): ",
          paste(unassigned_items, collapse = ", ")
        )
      }

      paste(assignments, collapse = ",")
    }

    observeEvent(input$run_reliability_button, {
      req(data(), input$reliability_coefficient_select)

      raw_data <- data()

      # If user picked specific items, subset; otherwise use all variables
      if (!is.null(input$reliability_item_select) &&
          length(input$reliability_item_select) > 0) {
        current_data <- raw_data[, input$reliability_item_select, drop = FALSE]
      } else {
        current_data <- raw_data
      }

      if (!all(sapply(current_data, is.numeric))) {
        showNotification(
          "Warning: Non-numeric columns present. They will be excluded automatically if possible.",
          type = "warning", duration = 6
        )
      }

      strata_define_arg <- NULL
      if (input$reliability_coefficient_select == "s_alpha") {
        n_factors <- max(1, as.integer(input$strata_factor_count %||% 2))
        strata_error <- NULL
        tryCatch({
          strata_define_arg <- build_strata_definition(names(current_data), n_factors)
        }, error = function(e) {
          strata_error <<- e$message
        })

        if (!is.null(strata_error)) {
          showNotification(strata_error, type = "error", duration = 8)
          reliability_output_rv(paste("Error:", strata_error))
          return()
        }
      }

      # Pick the right correlation argument for each method
      cor_arg <- if (input$reliability_coefficient_select == "cr") {
        input$cr_correlation_type_radio %||% "cor"
      } else {
        input$correlation_type_radio %||% "cor"
      }

      # Build cache key ? uniquely identifies this computation
      cache_key <- paste(
        input$reliability_coefficient_select,
        paste(sort(names(current_data)), collapse = ","),
        cor_arg,
        digest_str(input$cfa_model_for_reliability_input %||% ""),
        strata_define_arg %||% "",
        sep = "|"
      )

      # Cache hit: return immediately
      if (!is.null(cache_rv$store[[cache_key]])) {
        reliability_output_rv(cache_rv$store[[cache_key]])
        showNotification("Result retrieved from cache.", type = "message", duration = 4)
        return()
      }

      progress_id <- showNotification("Calculating...", duration = NULL, type = "message")
      on.exit(removeNotification(progress_id), add = TRUE)

      tryCatch({
        res <- reliability_func(
          x                 = current_data,
          method            = input$reliability_coefficient_select,
          cor_kind          = cor_arg,
          defined_structure = input$cfa_model_for_reliability_input,
          strata_define     = strata_define_arg
        )
        cache_rv$store[[cache_key]] <- res
        reliability_output_rv(res)
      }, error = function(e) {
        reliability_output_rv(paste("Error:", e$message))
      })
    })

    output$reliability_result_output <- renderText({
      reliability_output_rv() %||% "Result will appear here."
    })
  })
}

# Lightweight string hash for cache keys (avoids dependency on digest package)
digest_str <- function(s) {
  if (is.null(s) || nchar(s) == 0) return("EMPTY")
  paste0("h", abs(sum(utf8ToInt(s) * seq_along(utf8ToInt(s)))))
}

#' Access files in the current app
#'
#' NOTE: If you manually change your package name in the DESCRIPTION,
#' don't forget to change it here too, and in the config file.
#' For a safer name change mechanism, use the `golem::set_golem_name()` function.
#'
#' @param ... character vectors, specifying subdirectory and file(s)
#' within your package. The default, none, returns the root of the app.
#'
#' @noRd
app_sys <- function(...) {
  system.file(..., package = "FAfA")
}


#' Read App Config
#'
#' @param value Value to retrieve from the config file.
#' @param config GOLEM_CONFIG_ACTIVE value. If unset, R_CONFIG_ACTIVE.
#' If unset, "default".
#' @param use_parent Logical, scan the parent directory for config file.
#' @param file Location of the config file
#'
#' @noRd
get_golem_config <- function(
  value,
  config = Sys.getenv(
    "GOLEM_CONFIG_ACTIVE",
    Sys.getenv(
      "R_CONFIG_ACTIVE",
      "default"
    )
  ),
  use_parent = TRUE,
  # Modify this if your config file is somewhere else
  file = app_sys("golem-config.yml")
) {
  config::get(
    value = value,
    config = config,
    file = file,
    use_parent = use_parent
  )
}

test_that("app ui", {
  ui <- app_ui()
  golem::expect_shinytaglist(ui)
  # Check that formals have not been removed
  fmls <- formals(app_ui)
  for (i in c("request")) {
    expect_true(i %in% names(fmls))
  }
})

test_that("app server", {
  server <- app_server
  expect_type(server, "closure")
  # Check that formals have not been removed
  fmls <- formals(app_server)
  for (i in c("input", "output", "session")) {
    expect_true(i %in% names(fmls))
  }
})

test_that(
  "app_sys works",
  {
    expect_true(
      app_sys("golem-config.yml") != ""
    )
  }
)

test_that(
  "golem-config works",
  {
    config_file <- app_sys("golem-config.yml")
    skip_if(config_file == "")

    expect_true(
      get_golem_config(
        "app_prod",
        config = "production",
        file = config_file
      )
    )
    expect_false(
      get_golem_config(
        "app_prod",
        config = "dev",
        file = config_file
      )
    )
  }
)

# Configure this test to fit your need.
# testServer() function makes it possible to test code in server functions and modules, without needing to run the full Shiny application
testServer(app_server, {

  # Set and test an input
  session$setInputs(x = 2)
  expect_equal(input$x, 2)

  # Example of tests you can do on the server:
  # - Checking reactiveValues
  # expect_equal(r$lg, 'EN')
  # - Checking output
  # expect_equal(output$txt, "Text")
})

# Configure this test to fit your need
test_that(
  "app launches",
  {
    golem::expect_running(sleep = 5)
  }
)

#' Data Selection UI Module
#' @import shiny
#' @import bslib
#' @importFrom bsicons bs_icon
#' @importFrom shinycssloaders withSpinner
#' @noRd
data_selection_ui <- function(id) {
  ns <- NS(id)

  tagList(
    # ---- Top: Upload + Quick Summary ----
    layout_columns(
      col_widths = c(4, 8),
      card(
        card_header("Data Upload", class = "bg-primary text-white", bs_icon("upload")),
        card_body(
          markdown("
Please upload your data file. Supported formats: **.csv, .txt, .dat, .sav, .xlsx, .xls**.

*Ensure missing values are represented as NA.*
"),
          fileInput(ns("file1"), label = "Choose Data File:",
                    accept = c(".csv", ".txt", ".dat", ".sav", ".xlsx", ".xls"),
                    placeholder = "No file selected"),
          checkboxInput(ns("has_header_checkbox"),
                        label = "My data has a header row", value = TRUE),
          actionButton(ns("analyze_data"), "Analyze Data",
                       icon = icon("play"), class = "btn-success w-100")
        )
      ),
      card(
        card_header("Dataset Overview", bs_icon("speedometer2")),
        card_body(
          layout_columns(
            col_widths = c(3, 3, 3, 3),
            uiOutput(ns("n_var_box")),
            uiOutput(ns("n_obs_box")),
            uiOutput(ns("n_missing_box")),
            uiOutput(ns("n_complete_box"))
          )
        )
      )
    ),

    # ---- Tabs: Preview / Variable Summary / Distributions ----
    navset_card_tab(
      nav_panel(
        title = "Data Preview", icon = bs_icon("table"),
        tableOutput(ns("mydatatable")) %>% withSpinner(type = 8, color = "#2C3E50")
      ),
      nav_panel(
        title = "Variable Types", icon = bs_icon("diagram-2"),
        layout_columns(
          col_widths = c(4, 8),
          div(
            h6("Type Breakdown", class = "text-muted"),
            tableOutput(ns("var_types_table"))
          ),
          div(
            h6("Per-Variable Detail", class = "text-muted"),
            div(style = "max-height:480px; overflow-y:auto;",
                tableOutput(ns("variable_summary_table"))
            )
          )
        )
      ),
      nav_panel(
        title = "Distributions", icon = bs_icon("bar-chart-line"),
        p(class = "text-muted small",
          "Likert-like variables (integer-valued, <= 10 unique levels) shown as bar charts. ",
          "Continuous variables shown as histograms."),
        plotOutput(ns("category_dist_plot"), height = "550px") %>%
          withSpinner(type = 8, color = "#2C3E50")
      )
    )
  )
}

#' Reliability UI
#' @noRd
reliability_ui <- function(id) {
  ns <- NS(id)
  tagList(
    layout_columns(
      col_widths = c(5, 7),
      card(
        card_header("Reliability Setup", class = "bg-info text-white"),
        card_body(
          selectizeInput(
            ns("reliability_item_select"),
            "Select Items (leave empty for all):",
            choices  = NULL,
            multiple = TRUE,
            options  = list(placeholder = "Select specific variables...")
          ),
          hr(),
          radioButtons(ns("reliability_coefficient_select"), "Coefficient:",
                       choices = c(
                         "Cronbach's Alpha"                  = "alpha",
                         "McDonald's Omega Total"            = "omega",
                         "McDonald's Omega Hierarchical"     = "omega_h",
                         "Armor's Theta"                     = "theta",
                         "Stratified Alpha"                  = "s_alpha",
                         "Composite Reliability & AVE (CFA)" = "cr"
                       )
          ),

          # Theta: correlation type
          conditionalPanel("input.reliability_coefficient_select == 'theta'", ns = ns,
                           radioButtons(ns("correlation_type_radio"), "Correlation:",
                                        choices = c("Pearson" = "cor", "Polychoric" = "poly"), inline = TRUE)
          ),

          # Stratified Alpha: strata definition
          conditionalPanel("input.reliability_coefficient_select == 's_alpha'", ns = ns,
                           textInput(ns("strata_definition_input"), "Strata (e.g., 1,1,2,2,3):",
                                     placeholder = "Comma-separated item strata numbers...")
          ),

          # CR & AVE: requires CFA model syntax
          conditionalPanel("input.reliability_coefficient_select == 'cr'", ns = ns,
                           textAreaInput(ns("cfa_model_for_reliability_input"),
                                         "CFA Model Syntax (lavaan):",
                                         rows = 5,
                                         placeholder = "e.g.\nF1 =~ item1 + item2 + item3\nF2 =~ item4 + item5"),
                           radioButtons(ns("cr_correlation_type_radio"), "Data Type:",
                                        choices = c("Continuous (Pearson)" = "cor", "Ordinal (Polychoric)" = "poly"),
                                        inline = TRUE)
          ),

          actionButton(ns("run_reliability_button"), "Calculate",
                       icon = icon("calculator"), class = "btn-info w-100")
        )
      ),
      card(
        card_header("Result"),
        card_body(
          verbatimTextOutput(ns("reliability_result_output")) %>%
            withSpinner(type = 8, color = "#2C3E50")
        )
      )
    )
  )
}

#' About Server Module
#'
#' @param id Module namespace ID.
#' @noRd
about_server <- function(id) {
  moduleServer(id, function(input, output, session) {

    # Output for the application description
    output$application_description_html <- renderUI({
      tagList(
        h4(strong("FAfA: Factor Analysis for All")),

        p(strong("Aim:")),
        p("The FAfA (Factor Analysis for All) Shiny application is a powerful and user-friendly tool designed to simplify Exploratory Factor Analysis (EFA), Confirmatory Factor Analysis (CFA), and Measurement Invariance workflows for researchers. Developed with R and Shiny, FAfA aims to unify these psychometric procedures within a single, intuitive interface, reducing the need for multiple software tools or complex manual preprocessing steps. It enables users to diagnose and handle missing data, validate assumptions, perform random dataset splits, conduct EFA replicability checks across two random half-samples, conduct comprehensive reliability analyses (including Stratified Alpha), apply automated item drop-out strategies, and utilize item weighting techniques to enhance construct validity."),
        br(),

        h5(strong("Overview")),
        p("FAfA provides a comprehensive suite of tools for psychometric analysis. It leverages established R packages such as 'psych', 'lavaan', 'missForest', and 'EGAnet', ensuring that statistical analyses are accurate and reliable. Users can easily upload their datasets (e.g., CSV, Excel, SAV, DAT), perform advanced diagnostics for missing values and outliers, configure key parameters for their analyses (like estimator types, rotation methods), and interpret results through a responsive user interface."),
        br(),

        h5(strong("Key Features and Technical Details")),
        tags$ul(
          tags$li(strong("Unified EFA, CFA & Invariance Workflow:"), " Conduct EFA, CFA, and Measurement Invariance testing within the same environment using a seamless workflow."),
          tags$li(strong("Advanced Missing Data Handling:"), " Analyze missingness patterns, test for MCAR, and apply robust imputation methods such as MICE and missForest (Random Forest)."),
          tags$li(strong("Item Drop Out Analysis:"), " Utilize automated strategies to identify and remove problematic items, optimizing scale length and factor structure."),
          tags$li(strong("EFA Replicability:"), " Randomly split the dataset into two half-samples, run the same EFA configuration on both samples, and compare item-level factor loadings side by side."),
          tags$li(strong("Reliability Analysis:"), " Evaluate internal consistency using Cronbach's Alpha, McDonald's Omega, Armor's Theta, and Stratified Alpha for multidimensional scales."),
          tags$li(strong("Interactive Model Builder:"), " Easily define factor structures and covariances for CFA and Measurement Invariance without manually writing complex syntax."),
          tags$li(strong("Assumption Checking & Wrangling:"), " Built-in diagnostics for multivariate normality, outliers (Mahalanobis Distance), and multicollinearity."),
          tags$li(strong("Random Dataset Splitting:"), " Supports rigorous validation by allowing users to randomly split datasets (e.g., for EFA on one half, CFA on the other)."),
          tags$li(strong("Interactive and Reproducible Results:"), " Provides real-time updates with outputs like path diagrams, scree plots, and fit statistics (CFI, TLI, RMSEA). Supports exporting results for publication.")
        )
      )
    })

    # Output for developer information
    output$developer_info_html <- renderUI({
      tagList(
        p(
          strong("Lead Application Developer:"), br(),
          "Abdullah Faruk KILIC, PhD, Assoc. Prof.", br(),
          "Trakya University, Faculty of Education", br(),
          "Department of Educational Science", br(),
          "Division of Measurement and Evaluation in Education"
        ),
        p(strong("Email 1:"), tags$a(href = "mailto:abdullahfarukkilic@gmail.com", "abdullahfarukkilic@gmail.com")),
        p(strong("Email 2:"), tags$a(href = "mailto:afarukkilic@trakya.edu.tr", "afarukkilic@trakya.edu.tr")),
        div(
          class = "d-flex flex-wrap gap-2 mt-2 mb-3",
          tags$a(
            href = "https://www.instagram.com/afarukkilic/",
            target = "_blank", rel = "noopener",
            class = "btn btn-sm",
            style = "background: linear-gradient(45deg,#f09433,#e6683c,#dc2743,#cc2366,#bc1888); color:#fff; border:none;",
            tags$i(class = "bi bi-instagram"), " Instagram"
          ),
          tags$a(
            href = "https://www.researchgate.net/profile/Abdullah-Kilic-2",
            target = "_blank", rel = "noopener",
            class = "btn btn-sm",
            style = "background:#00CCBB; color:#fff; border:none;",
            tags$i(class = "bi bi-file-earmark-text"), " ResearchGate"
          ),
          tags$a(
            href = "https://scholar.google.com/citations?user=AP7LlpoAAAAJ&hl=en",
            target = "_blank", rel = "noopener",
            class = "btn btn-sm btn-primary",
            tags$i(class = "bi bi-mortarboard"), " Google Scholar"
          )
        ),

        hr(),

        p(
          strong("Author (Item Drop Out & Missing Data):"), br(),
          "Ahmet CALISKAN", br(),
          "Trakya University, Faculty of Education", br(),
          "Department of Educational Science", br(),
          "Division of Measurement and Evaluation in Education"
        ),
        p(strong("Email:"), tags$a(href = "mailto:ahmetcaliskan@trakya.edu.tr", "ahmetcaliskan@trakya.edu.tr")),
        p(em("Contributed to the development of the Item Drop Out module and provided insights on missing data handling.")),

        br(),
        p(em("Please feel free to contact us regarding any errors or feedback on the application."))
      )
    })

    # Output for contributors and version information
    output$contributors_version_html <- renderUI({
      tagList(
        p(strong("Acknowledgements:")),
        p("A big thank you to the following researchers for their valuable feedback on the initial versions of FAfA:"),
        tags$ul(
          tags$li(strong("Seda Nur SAKAR:"), tags$a(href = "https://avesis.hacettepe.edu.tr/sedasakar", target = "_blank", "Researcher's Profile")),
          tags$li(strong("Tugay KACAK:"), tags$a(href = "https://personel.trakya.edu.tr/tugaykacak/", target = "_blank", "Researcher's Profile")),
          tags$li(strong("Basak ERDEM KARA:"), tags$a(href = "https://avesis.anadolu.edu.tr/basakerdem", target = "_blank", "Researcher's Profile")),
          tags$li(strong("Meltem ACAR GUVENDIR:"), tags$a(href = "https://personel.trakya.edu.tr/meltemacar/", target = "_blank", "Researcher's Profile")),
          tags$li(strong("Alperen YANDI:"), tags$a(href = "https://www.linkedin.com/in/alperen-yandi-36404891", target = "_blank", "Researcher's Profile")),
          tags$li(strong("Murat Dogan SAHIN:"), tags$a(href = "https://avesis.anadolu.edu.tr/mdsahin", target = "_blank", "Researcher's Profile"))
        ),
        br(),
        p(strong("What's New in Version 1.1:")),
        tags$ul(
          tags$li(strong("Performance improvements:"),
            " Polychoric correlations cached across KMO, Bartlett, heatmap and report views (3x faster on large datasets). Assumptions results cached across descriptives, collinearity and normality. Variable summary table reduced from seven column scans to a single pass."),
          tags$li(strong("Reliability caching:"),
            " Reliability coefficients are cached by method, item set, and correlation type. Re-running the same calculation returns instantly."),
          tags$li(strong("CFA Higher-Order and Bifactor builders:"),
            " New tabs in the Model Builder for second-order and bifactor models. Bifactor builder generates orthogonality constraints automatically. Templates added for correlated, higher-order and bifactor specifications."),
          tags$li(strong("EFA enhancements:"),
            " Eight extraction methods (added WLS, Minimum Rank, Minimum Chi-Square, GLS); fourteen rotation methods grouped into Oblique and Orthogonal. KMO and Bartlett moved to a compact horizontal card. Heatmap supports five colour palettes including grayscale, with off-diagonal correlation summary."),
          tags$li(strong("EFA Replicability module:"),
            " Added random split-half EFA replication with the same correlation, extraction, factor number, and rotation options used in EFA. Item-level factor loadings are reported in a comparative table with factor congruence and primary factor agreement summaries."),
          tags$li(strong("Factor Retention correlation choice:"),
            " Pearson and Polychoric options added."),
          tags$li(strong("EGA community detection:"),
            " Six algorithms available (Walktrap, Louvain, Leiden, Fast Greedy, Edge Betweenness, Label Propagation). Results redesigned with a tabbed layout."),
          tags$li(strong("Reliability expanded:"),
            " Added McDonald's Omega Hierarchical and Composite Reliability with AVE (CFA-based)."),
          tags$li(strong("Data Selection redesigned:"),
            " New tabs for Data Preview, Variable Types, and Distributions. Per-variable summary table with Type, N_Unique, Missing, Min, Max, Mean and SD. Likert-style and continuous variables visualised separately."),
          tags$li(strong("Exclude Variables module redesigned:"),
            " Dual-panel checkbox layout with real-time variable counters. Selective recovery and full reset supported."),
          tags$li(strong("Item Drop Out / ItemRest module:"),
            " The ItemRest package was used as a foundation and modified for FAfA. A search algorithm was added, and the optimal strategy is presented as a structured card with metric badges and fit indices instead of raw text."),
          tags$li(strong("Imputation iteration controls:"),
            " MICE and missForest iteration counts configurable via sliders."),
          tags$li(strong("CFA path diagram fix for ordinal data:"),
            " Overlapping residual arrows hidden for polychoric/WLSMV analyses (residuals fixed to 1.0 by identification constraints)."),
          tags$li(strong("Modern UI redesign:"),
            " Dark sidebar navigation, Inter typography, gradient card headers, refined buttons and tables, color-coded notifications. Multicollinearity interpretation thresholds (VIF, TOL, CI) shown below the collinearity table."),
          tags$li(strong("Outlier index drift fix:"),
            " Outlier removal snapshots the dataset and refuses if upstream data has changed since detection."),
          tags$li(strong("About page:"),
            " Instagram, ResearchGate, and Google Scholar profile links added as interactive buttons.")
        ),
        br(),
        p(em("Current Version: 1.1"))
      )
    })

    # Output for citation information
    output$citation_info_html <- renderUI({
      tagList(
        tags$p("If you use this package in your work, please cite it as follows:"),
        tags$br(),
        tags$p(
          style = "font-family: monospace; margin-left: 20px;",
          "Kilic, A. F. (2024). FAfA: Factor analysis for all: An R package to conduct factor analysis with RShiny application. ",
          tags$em("Journal of Measurement and Evaluation in Education and Psychology, 15"),
          HTML("(4), 446&ndash;451. "),
          tags$a(href = "https://doi.org/10.21031/epod.1555805", target = "_blank", "https://doi.org/10.21031/epod.1555805")
        ),
        tags$br(),
        tags$p("BibTeX citation:"),
        tags$pre(
          style = "font-family: monospace; background-color: #f0f0f0; padding: 10px; border-radius: 5px; white-space: pre-wrap; word-wrap: break-word;",
          paste(
            "@article{Kilic2024FAfA,",
            "  author    = {Kilic, Abdullah Faruk},",
            "  title     = {FAfA: Factor analysis for all An R package to conduct factor analysis with RShiny application},",
            "  journal   = {Journal of Measurement and Evaluation in Education and Psychology},",
            "  volume    = {15},",
            "  number    = {4},",
            " pages = {446\u2013451},",
            "  year      = {2024},",
            "  doi       = {10.21031/epod.1555805},",
            "  URL       = {https://doi.org/10.21031/epod.1555805}",
            "}",
            sep = "\n"
          )
        )
      )
    })

  })
}

#' Run the Shiny Application
#'
#' @description This function launches the Shiny application.
#' It uses default options for `shinyApp` and golem.
#'
#' @export
#' @importFrom shiny shinyApp
#' @importFrom golem with_golem_options
run_app <- function() {
  with_golem_options(
    app = shinyApp(
      ui = app_ui,
      server = app_server,
      onStart = NULL,         
      options = list(),        
      enableBookmarking = NULL, 
      uiPattern = "/"         
    ),
    golem_opts = list()
  )
}

#' Default Value Operator
#'
#' Helper for default NULL values (coalesce)
#'
#' @name op_null_or
#' @rdname op_null_or
#' @param x Left hand side
#' @param y Right hand side
#' @export
`%||%` <- function(x, y) if (is.null(x) || length(x) == 0 || all(is.na(x))) y else x

#' Clean Missing Data
#' Handles various NA representations and optionally removes rows with NAs.
#' @param data A data frame.
#' @param remove_na Logical. If TRUE, removes rows with NA (listwise deletion).
#' @return A list with `cleaned_data` (data.frame) and `removed_rows` (integer).
#' @importFrom stats na.omit
#' @noRd
clean_missing_data <- function(data, remove_na = TRUE) {
  if (!is.data.frame(data)) {
    data <- as.data.frame(data)
  }

  # 1. Standartlaştırma: Farklı karakterleri NA'ya çevir
  data_std_na <- as.data.frame(lapply(data, function(col_item) {
    if(is.character(col_item)) {
      col_item[col_item %in% c("", " ", "NA", "N/A", "na", "n/a", ".", "-", "?", "missing")] <- NA
    } else if (is.factor(col_item)) {
      char_col <- as.character(col_item)
      char_col[char_col %in% c("", " ", "NA", "N/A", "na", "n/a", ".", "-", "?", "missing")] <- NA
      col_item <- char_col
    }

    # Karışık tipler için ek kontrol
    if(is.character(col_item) || is.factor(col_item)){
      potential_na_mask <- is.na(col_item) | suppressWarnings(is.na(as.numeric(as.character(col_item))))
      col_item[potential_na_mask & !is.na(col_item) & !(as.character(col_item) %in% c("", " "))] <- NA
    }
    return(col_item)
  }))

  # 2. Nümeriğe Çevirme
  data_numeric <- as.data.frame(lapply(data_std_na, function(col_item) {
    suppressWarnings(as.numeric(as.character(col_item)))
  }))

  # 3. NA Silme İşlemi (Parametreye Bağlı)
  original_nrow <- nrow(data_numeric)

  if (remove_na) {
    cleaned_data_final <- stats::na.omit(data_numeric)
    removed_rows_count <- original_nrow - nrow(cleaned_data_final)
  } else {
    cleaned_data_final <- data_numeric
    removed_rows_count <- 0 # Silme yapılmadı
  }

  return(list(
    cleaned_data = cleaned_data_final,
    removed_rows = removed_rows_count
  ))
}

# ---------------------------------------------------------------------------
# assumptions
# ---------------------------------------------------------------------------
#' Calculate Statistical Assumptions
#'
#' Calculates descriptives, multicollinearity, Mahalanobis distance, and Mardia's tests.
#' @param x A numeric data frame.
#' @param mah_p_threshold P-value threshold for Mahalanobis outliers.
#' @return A list of assumption check results.
#' @importFrom pastecs stat.desc
#' @importFrom moments skewness kurtosis
#' @importFrom mctest mctest eigprop
#' @importFrom mvnormalTest mardia
#' @importFrom dplyr mutate filter select arrange
#' @importFrom magrittr %>%
#' @importFrom stats na.omit lm as.formula model.matrix mahalanobis pchisq cov median cor
#' @noRd
assumptions <- function(x, mah_p_threshold = 0.001) { # Added mah_p_threshold argument
  if (!is.data.frame(x)) {
    x <- as.data.frame(x)
  }
  if (ncol(x) == 0) {
    stop("Input data 'x' has 0 columns for assumptions checking.")
  }

  # Ensure data is numeric (as per previous robust versions)
  x_numeric <- x
  if (!all(sapply(x_numeric, is.numeric))) {
    warning("Not all columns are numeric in data passed to assumptions(). Attempting conversion.")
    x_numeric <- as.data.frame(lapply(x, function(col) {
      if(!is.numeric(col)) suppressWarnings(as.numeric(as.character(col))) else col
    }))
    if (!all(sapply(x_numeric, is.numeric))) {
      stop("Failed to convert all columns to numeric for assumptions checking.")
    }
  }
  x <- x_numeric # Use the numeric version

  # --- Descriptive Statistics (User's Logic) ---
  descr <- as.data.frame(matrix(NA, nrow = ncol(x), ncol = 8 ))
  colnames(descr) <- c("N", "N (missing)", "Min", "Max", "Median", "Mean", "Skewness", "Kurtosis")
  rownames(descr) <- colnames(x) # Assign row names for clarity

  if (nrow(x) > 0) { # Proceed only if data has rows
    missing_mean <- function(var) mean(var, na.rm = TRUE) # Corrected to take single var

    # Use tryCatch for pastecs in case of issues with very small/odd data
    descriptives_pastecs <- tryCatch(
      pastecs::stat.desc(x, basic = TRUE, desc = TRUE, norm = FALSE),
      error = function(e) {
        warning(paste("pastecs::stat.desc failed:", e$message)); NULL
      }
    )

    if (!is.null(descriptives_pastecs) && is.matrix(descriptives_pastecs)) {
      if("nbr.val" %in% rownames(descriptives_pastecs)) descr[,"N"] <- descriptives_pastecs["nbr.val", ]
      if("min" %in% rownames(descriptives_pastecs)) descr[,"Min"] <- descriptives_pastecs["min", ]
      if("max" %in% rownames(descriptives_pastecs)) descr[,"Max"] <- descriptives_pastecs["max", ]
      if("median" %in% rownames(descriptives_pastecs)) descr[,"Median"] <- descriptives_pastecs["median", ]
    }

    descr[,"N"] <- nrow(x)
    descr[,"N (missing)"] <- colSums(apply(x, 2, is.na))
    descr[,"Min"] <- apply(na.omit(x), 2, min)
    descr[,"Max"] <- apply(na.omit(x), 2, max)
    descr[,"Median"] <- apply(na.omit(x), 2, median)
    descr[,"Mean"] <- apply(x, 2, missing_mean) # User's original

    # Skewness and Kurtosis from 'moments' package
    if(nrow(x) >= 1 && ncol(x) >=1){
      skew_vals <- tryCatch(moments::skewness(x, na.rm = TRUE), error = function(e) rep(NA_real_, ncol(x)))
      kurt_vals <- tryCatch(moments::kurtosis(x, na.rm = TRUE) - 3, error = function(e) rep(NA_real_, ncol(x))) #kurtosis
      if(length(skew_vals) == ncol(x)) descr[,"Skewness"] <- as.numeric(skew_vals)
      if(length(kurt_vals) == ncol(x)) descr[,"Kurtosis"] <- as.numeric(kurt_vals)
    }
  } else {
    warning("Input data for 'assumptions' has 0 rows. Descriptive statistics will be NA.")
  }

  # --- Data for multicollinearity, Mahalanobis, Mardia needs to be complete cases ---
  x_complete <- stats::na.omit(x) # Use the already numeric 'x'

  # Initialize results for parts that depend on x_complete
  mc_control <- data.frame(VIF_min=NA_real_, VIF_max=NA_real_, TOL_min=NA_real_, TOL_max=NA_real_, CI_min=NA_real_, CI_max=NA_real_)
  Mah_significant <- data.frame(Row_Number=integer(), MD=numeric(), MD_p=numeric(), stringsAsFactors = FALSE)

  # Default structure for mardia results
  mardia_default_row <- data.frame(Test="N/A", Statistic=NA_real_, "p-value"=NA_real_, Result=NA_character_, check.names = FALSE, stringsAsFactors = FALSE)
  mardia_kurt_result_df <- mardia_default_row[1, , drop=FALSE]; mardia_kurt_result_df$Test <- "Kurtosis"
  mardia_skew_result_df <- mardia_default_row[1, , drop=FALSE]; mardia_skew_result_df$Test <- "Skewness"
  n_outlier <- 0

  if (nrow(x_complete) >= 2 && ncol(x_complete) >= 1 && nrow(x_complete) > ncol(x_complete)) {
    # --- Multicollinearity (User's Logic) ---
    if (ncol(x_complete) >= 2) {
      x_new <- x_complete
      x_new$rn_dummy_dependent_var <- 1:nrow(x_new) # User's variable name
      model_for_collinearity <- tryCatch(
        lm(stats::as.formula(paste(colnames(x_new)[ncol(x_new)], "~",
                                   paste(colnames(x_new)[1:(ncol(x_new)-1)], collapse = "+"))),
           data=x_new),
        error = function(e) {warning(paste("LM for multicollinearity failed:", e$message)); NULL}
      )

      if (!is.null(model_for_collinearity)) {
        mc_VIF_TOL_list <- tryCatch(mctest::mctest(model_for_collinearity, type = "i"), error = function(e) NULL)
        mc_CI_list <- tryCatch(mctest::eigprop(mod = model_for_collinearity), error = function(e) NULL)

        if (!is.null(mc_VIF_TOL_list) && is.list(mc_VIF_TOL_list) && "idiags" %in% names(mc_VIF_TOL_list)) {
          mc_VIF_TOL_df <- as.data.frame(mc_VIF_TOL_list$idiags[,1:2]) # Ensure it's a data frame
          mc_control$VIF_min <- if("VIF" %in% colnames(mc_VIF_TOL_df) && sum(!is.na(mc_VIF_TOL_df$VIF)) > 0) min(mc_VIF_TOL_df$VIF, na.rm=TRUE) else NA_real_
          mc_control$VIF_max <- if("VIF" %in% colnames(mc_VIF_TOL_df) && sum(!is.na(mc_VIF_TOL_df$VIF)) > 0) max(mc_VIF_TOL_df$VIF, na.rm=TRUE) else NA_real_
          mc_control$TOL_min <- if("TOL" %in% colnames(mc_VIF_TOL_df) && sum(!is.na(mc_VIF_TOL_df$TOL)) > 0) min(mc_VIF_TOL_df$TOL, na.rm=TRUE) else NA_real_
          mc_control$TOL_max <- if("TOL" %in% colnames(mc_VIF_TOL_df) && sum(!is.na(mc_VIF_TOL_df$TOL)) > 0) max(mc_VIF_TOL_df$TOL, na.rm=TRUE) else NA_real_
        }

        if (!is.null(mc_CI_list) && is.list(mc_CI_list) && "ci" %in% names(mc_CI_list) && length(mc_CI_list$ci) > 0) {
          mc_control$CI_min <- min(mc_CI_list$ci, na.rm=TRUE)
          mc_control$CI_max <- max(mc_CI_list$ci, na.rm=TRUE)
        }
      }
    }

    # --- Mahalanobis Distance (User's Logic, adapted for mah_p_threshold) ---
    if (nrow(x_complete) > ncol(x_complete) && ncol(x_complete) > 0) {
      distance <- tryCatch(as.matrix(stats::mahalanobis(x_complete, colMeans(x_complete), cov(x_complete))), error = function(e) NULL)
      if (!is.null(distance)) {
        Mah_significant <- x_complete %>% # Use x_complete here
          dplyr::mutate(Row_Number_In_Complete_Data = 1:nrow(x_complete), # Row number within x_complete
                        MD = as.numeric(distance), # Ensure MD is numeric
                        MD_p = stats::pchisq(MD, df = (ncol(x_complete)), lower.tail = FALSE)) %>%
          dplyr::filter(MD_p <= mah_p_threshold) %>%
          dplyr::select(Row_Number_In_Complete_Data, MD, MD_p) %>% # Select only relevant columns
          dplyr::arrange(MD_p,  dplyr::desc(MD))
        n_outlier <- nrow(Mah_significant)
      }
    } else {
      warning("Not enough observations relative to variables (N <= P) for Mahalanobis distance.")
    }

    # --- Mardia's Kurtosis and Skewness (User's Logic with Correction) ---
    if (nrow(x_complete) >= 2 && ncol(x_complete) >= 2) {
      mardia_test_output <- tryCatch(mvnormalTest::mardia(as.matrix(x_complete)), error = function(e) {
        warning(paste("Mardia test failed:", e$message)); NULL
      })
      if (!is.null(mardia_test_output) && !is.null(mardia_test_output$mv.test) && is.data.frame(mardia_test_output$mv.test)) {
        mardia_full_df <- mardia_test_output$mv.test

        # Process Skewness
        skew_row <- mardia_full_df[mardia_full_df$Test == "Skewness",, drop = FALSE]
        if (nrow(skew_row) == 1) {
          mardia_skew_result_df <- data.frame(
            Test = "Skewness",
            Statistic = as.numeric(as.character(skew_row[1, "Statistic"])),
            "p-value" = as.numeric(as.character(skew_row[1, "p-value"])),
            Result = as.character(skew_row[1, "Result"]),
            check.names = FALSE, stringsAsFactors = FALSE
          )
        }

        # Process Kurtosis
        kurt_row <- mardia_full_df[mardia_full_df$Test == "Kurtosis", , drop = FALSE]
        if (nrow(kurt_row) == 1) {
          mardia_kurt_result_df <- data.frame(
            Test = "Kurtosis",
            Statistic = as.numeric(as.character(kurt_row[1, "Statistic"])),
            "p-value" = as.numeric(as.character(kurt_row[1, "p-value"])),
            Result = as.character(kurt_row[1, "Result"]),
            check.names = FALSE, stringsAsFactors = FALSE
          )
        }
      }
    } else {
      warning("Not enough data (N<2 or P<2) for Mardia's tests.")
    }
  } else {
    warning("Not enough complete cases (N < 2 or N <= P) for some assumption checks.")
  }

  # MVN table: Mardia Skewness + Kurtosis
  mvn_table <- rbind(mardia_skew_result_df, mardia_kurt_result_df)
  rownames(mvn_table) <- NULL

  # --- Return List ---
  return(list(
    descriptives      = round(descr, 2),
    multicollinearity = round(mc_control, 2),
    Mah_significant   = Mah_significant,
    n_outlier         = n_outlier,
    Mardia_Kurtosis   = mardia_kurt_result_df,
    Mardia_Skewness   = mardia_skew_result_df,
    mvn_table         = mvn_table          # All MVN tests in one data frame
  ))
}


# ---------------------------------------------------------------------------
# factor_ret
# ---------------------------------------------------------------------------
#' Factor Retention Methods
#'
#' Applies methods to suggest the number of factors to retain.
#' @param x A numeric data frame or matrix.
#' @param method Character string for the retention method.
#' @return A data frame with suggested number of factors.
#' @importFrom stats na.omit
#' @importFrom EFA.MRFA parallelMRFA hullEFA
#' @importFrom psych fa.parallel principal
#' @importFrom EFA.dimensions MAP EMPKC
#' @importFrom EGAnet EGA
#' @importFrom EFAtools CD
#' @noRd
factor_ret <- function(x, method = "hull_method") {
  if (!is.data.frame(x) && !is.matrix(x)) {
    x <- as.data.frame(x)
  }
  if (!all(sapply(x, is.numeric))) {
    x_numeric_cols <- sapply(x, is.numeric)
    non_numeric_cols <- colnames(x)[!x_numeric_cols]
    warning(paste0("DEBUG factor_ret: Non-numeric columns found and will be excluded: ", paste(non_numeric_cols, collapse=", ")))
    x <- x[, x_numeric_cols, drop=FALSE]
    if(ncol(x) < 2) {
      return(data.frame(Suggested_Factors = NA, row.names = paste("Error in", method, ": Requires at least 2 numeric columns.")))
    }
  }

  x_complete <- stats::na.omit(x)

  if (nrow(x_complete) < max(3, ncol(x_complete) +1 ) || ncol(x_complete) < 2 ) {
    return(data.frame(Suggested_Factors = NA, row.names = paste("Error in", method, ": Insufficient data (N=",nrow(x_complete),", P=",ncol(x_complete),")")))
  }

  col_variances <- apply(x_complete, 2, var)
  if (any(col_variances == 0, na.rm = TRUE)) {
    zero_var_cols <- colnames(x_complete)[col_variances == 0]
    return(data.frame(Suggested_Factors = NA, row.names = paste("Error in", method, ": Zero variance in column(s)")))
  }

  if (method == "pa_mrfa") {
    return(tryCatch({
      op_pa_analysis <- EFA.MRFA::parallelMRFA(x_complete)
      data.frame(Suggested_Factors = op_pa_analysis$N_factors_percentiles, row.names = "Optimal Parallel Analysis (MRFA)")
    }, error = function(e) {
      data.frame(Suggested_Factors = NA, row.names = paste("Error in pa_mrfa:", conditionMessage(e)))
    }))
  } else if (method == "pa_traditional") {
    return(tryCatch({
      tra_pa_analysis <- psych::fa.parallel(x_complete, fa = "fa", plot = FALSE, show.legend = FALSE, error.bars = FALSE, fm="pa")
      data.frame(Suggested_Factors = tra_pa_analysis$nfact, row.names = "Traditional Parallel Analysis (FA based)")
    }, error = function(e) {
      data.frame(Suggested_Factors = NA, row.names = paste("Error in pa_traditional:", conditionMessage(e)))
    }))
  } else if (method == "hull_method") {
    return(tryCatch({
      hull_analysis <- EFA.MRFA::hullEFA(x_complete, display = FALSE)
      data.frame(Suggested_Factors = hull_analysis$n_factors, row.names = "Hull Method (EFA)")
    }, error = function(e) {
      data.frame(Suggested_Factors = NA, row.names = paste("Error in hull_method:", conditionMessage(e)))
    }))
  } else if (method == "map_method_tra") {
    return(tryCatch({
      map_analysis <- suppressMessages(EFA.dimensions::MAP(x_complete, corkind = "pearson", verbose = FALSE))
      data.frame(Suggested_Factors = map_analysis$NfactorsMAP, row.names = "Minimum Average Partial (MAP - Original)")
    }, error = function(e) {
      data.frame(Suggested_Factors = NA, row.names = paste("Error in map_method_tra:", conditionMessage(e)))
    }))
  } else if (method == "map_method_rev") {
    return(tryCatch({
      map_analysis <- suppressMessages(EFA.dimensions::MAP(x_complete, corkind = "pearson", verbose = FALSE))
      data.frame(Suggested_Factors = map_analysis$NfactorsMAP4, row.names = "Minimum Average Partial (MAP - Revised)")
    }, error = function(e) {
      data.frame(Suggested_Factors = NA, row.names = paste("Error in map_method_rev:", conditionMessage(e)))
    }))
  } else if (method == "EGA_tmfg") {
    return(tryCatch({
      ega_analysis_tmfg <- EGAnet::EGA(data = x_complete, model = "TMFG", plot.EGA = FALSE, verbose = FALSE)
      data.frame(Suggested_Factors = ega_analysis_tmfg$n.dim, row.names = "Exploratory Graph Analysis (TMFG)")
    }, error = function(e) {
      data.frame(Suggested_Factors = NA, row.names = paste("Error in EGA_tmfg:", conditionMessage(e)))
    }))
  } else if (method == "EGA_glasso") {
    return(tryCatch({
      ega_analysis_glasso <- EGAnet::EGA(data = x_complete, model = "glasso", plot.EGA = FALSE, verbose = FALSE)
      data.frame(Suggested_Factors = ega_analysis_glasso$n.dim, row.names = "Exploratory Graph Analysis (Glasso)")
    }, error = function(e) {
      data.frame(Suggested_Factors = NA, row.names = paste("Error in EGA_glasso:", conditionMessage(e)))
    }))
  } else if (method == "EK_C") {
    return(tryCatch({
      emkpc_analysis <- suppressMessages(EFA.dimensions::EMPKC(x_complete, corkind = "pearson", verbose = FALSE))
      data.frame(Suggested_Factors = emkpc_analysis$NfactorsEMPKC, row.names = "Empirical Kaiser Criterion (EKC)")
    }, error = function(e) {
      data.frame(Suggested_Factors = NA, row.names = paste("Error in EK_C:", conditionMessage(e)))
    }))
  } else if (method == "comp_data_method") {
    return(tryCatch({
      cd_analysis <- EFAtools::CD(x_complete)
      data.frame(Suggested_Factors = cd_analysis$n_factors, row.names = "Comparison Data (CD)")
    }, error = function(e) {
      data.frame(Suggested_Factors = NA, row.names = paste("Error in comp_data_method:", conditionMessage(e)))
    }))
  } else {
    return(data.frame(Suggested_Factors = NA, row.names = paste("Error: Unknown method", method)))
  }
}

# ---------------------------------------------------------------------------
# reliability_func
# ---------------------------------------------------------------------------
#' Calculate Reliability Coefficients
#'
#' Calculates various reliability coefficients like Alpha, Omega, Theta, etc.
#' @param x A numeric data frame or matrix.
#' @param method Reliability method ("alpha", "omega", "theta", "structure", "s_alpha").
#' @param cor_kind Correlation type (for "theta").
#' @param defined_structure Lavaan model syntax (for "structure").
#' @param strata_define Strata definition string (for "s_alpha").
#' @return A string with the formatted reliability coefficient or an error message.
#' @importFrom psych alpha omega principal
#' @importFrom MBESS ci.reliability
#' @importFrom lavaan lavaanify cfa
#' @importFrom semTools reliability
#' @importFrom sirt stratified.cronbach.alpha
#' @importFrom stats na.omit cov
#' @noRd
reliability_func <- function(x, method = "alpha", cor_kind = "cor", defined_structure = NULL, strata_define = NULL) {
  if (!is.data.frame(x) && !is.matrix(x)) x <- as.data.frame(x)
  if (!all(sapply(x, is.numeric))) {
    x_numeric_cols <- sapply(x, is.numeric)
    warning(paste0("Non-numeric columns found and will be excluded from reliability analysis: ", paste(colnames(x)[!x_numeric_cols], collapse=", ")))
    x <- x[, x_numeric_cols, drop=FALSE]
    if(ncol(x) < 2) stop("At least two numeric columns required for reliability analysis.")
  }
  x_complete <- stats::na.omit(x)
  if (nrow(x_complete) < 2 || ncol(x_complete) < 2 ) {
    stop("Not enough data (rows/columns) after NA removal and numeric filtering for reliability analysis.")
  }

  result_value <- tryCatch({
    if (method == "alpha") {
      alpha_analysis <- psych::alpha(x_complete, check.keys = FALSE)
      alpha_total_df <- as.data.frame(alpha_analysis$total)
      std_alpha_val <- alpha_total_df["std.alpha",1] %||% alpha_total_df["raw_alpha",1] %||% alpha_total_df[1,1] %||% NA_real_
      as.numeric(std_alpha_val)
    } else if (method == "omega") {
      omega_psych <- tryCatch(psych::omega(x_complete, plot=FALSE, fm="pa"), error = function(e) {
        warning(paste("psych::omega failed:", e$message, "Trying MBESS if N > P.")); NULL
      })
      if(!is.null(omega_psych) && "omega.tot" %in% names(omega_psych)){
        return(omega_psych$omega.tot)
      } else if (nrow(x_complete) > ncol(x_complete)) {
        warning("psych::omega did not provide omega.tot, attempting MBESS::ci.reliability (requires N > P).")
        cov_matrix <- stats::cov(x_complete)
        omega_analysis_mbess <- MBESS::ci.reliability(S = cov_matrix, N = nrow(x_complete), type = "omega")
        return(omega_analysis_mbess$est)
      } else {
        warning("Omega calculation failed with psych::omega and N <= P for MBESS.")
        return(NA_real_)
      }
    } else if (method == "theta") {
      armor_theta_calc <- function(data_in, correlation_type_internal = "cor") {
        num_items <- ncol(data_in)
        if (num_items < 2) return(NA_real_)
        valid_cor_types <- c("cor", "cov", "poly", "tet")
        if(!(correlation_type_internal %in% valid_cor_types)) {
          warning(paste("Invalid correlation_type '", correlation_type_internal, "' for psych::principal. Defaulting to 'cor'."))
          correlation_type_internal <- "cor"
        }
        pca_res <- psych::principal(data_in, nfactors = 1, rotate = "none", cor = correlation_type_internal)
        first_eigenvalue <- pca_res$Vaccounted[1, 1]
        if (is.na(first_eigenvalue) || (first_eigenvalue <= 1 && num_items > 1) || num_items <= 1) {
          return(NA_real_)
        }
        theta_val <- (num_items / (num_items - 1)) * (1 - (1 / first_eigenvalue))
        return(theta_val)
      }
      armor_theta_calc(x_complete, correlation_type_internal = cor_kind)
    } else if (method == "structure") {
      if (is.null(defined_structure) || nchar(trimws(defined_structure)) == 0) {
        stop("CFA model structure must be defined for 'structural' reliability.")
      }
      manifest_vars_rel <- unique(unlist(lavaan::lavaanify(defined_structure)$rhs[lavaan::lavaanify(defined_structure)$op == "=~"]))
      ordered_arg_rel <- if (cor_kind == "poly" && length(manifest_vars_rel) > 0) manifest_vars_rel else FALSE
      estimator_rel <- ifelse(any(as.logical(ordered_arg_rel)), "WLSMV", "ML")

      cfa_model_fit <- lavaan::cfa(model = defined_structure, data = x_complete, ordered = ordered_arg_rel, estimator = estimator_rel, warn = FALSE)
      structural_rel_matrix <- semTools::reliability(cfa_model_fit)
      return(sprintf("%.3f", structural_rel_matrix[5, 1]))
    } else if (method == "s_alpha") {
      if (is.null(strata_define) || nchar(trimws(strata_define)) == 0) {
        stop("Strata definition must be provided for 'stratified alpha'.")
      }
      strata_num_vector <- suppressWarnings(as.numeric(unlist(strsplit(strata_define, ","))))
      if(any(is.na(strata_num_vector))) stop("Strata definition contains non-numeric values.")
      if (length(strata_num_vector) != ncol(x_complete)) {
        stop("Length of strata definition does not match the number of items.")
      }
      item_names_strata <- colnames(x_complete)
      strata_matrix_sirt <- data.frame(item = item_names_strata, stratum = strata_num_vector)

      s_alpha_results <- sirt::stratified.cronbach.alpha(dat = x_complete, itemstrata = strata_matrix_sirt)
      s_alpha_results$alpha.stratified[1]
    } else if (method == "omega_h") {
      omega_res <- tryCatch(
        psych::omega(x_complete, plot = FALSE, fm = "pa"),
        error = function(e) NULL
      )
      if (!is.null(omega_res) && "omega_h" %in% names(omega_res)) {
        return(omega_res$omega_h)
      }
      warning("omega_h: psych::omega failed or omega_h not found.")
      return(NA_real_)

    } else if (method == "cr") {
      if (is.null(defined_structure) || nchar(trimws(defined_structure)) == 0) {
        stop("A lavaan CFA model syntax is required for Composite Reliability (CR) and AVE.")
      }
      manifest_vars_cr <- unique(unlist(
        lavaan::lavaanify(defined_structure)$rhs[lavaan::lavaanify(defined_structure)$op == "=~"]
      ))
      ordered_arg_cr <- if (cor_kind == "poly" && length(manifest_vars_cr) > 0) manifest_vars_cr else FALSE
      estimator_cr   <- ifelse(any(as.logical(ordered_arg_cr)), "WLSMV", "ML")

      cfa_fit_cr <- lavaan::cfa(
        model = defined_structure, data = x_complete,
        ordered = ordered_arg_cr, estimator = estimator_cr, warn = FALSE
      )
      std_sol_cr  <- lavaan::standardizedSolution(cfa_fit_cr)
      load_rows   <- std_sol_cr[std_sol_cr$op == "=~", , drop = FALSE]
      factors     <- unique(load_rows$lhs)

      lines <- vapply(factors, function(f) {
        l       <- load_rows[load_rows$lhs == f, "est.std"]
        l       <- l[!is.na(l)]
        cr_val  <- sum(l)^2 / (sum(l)^2 + sum(1 - l^2))
        ave_val <- sum(l^2) / (sum(l^2) + sum(1 - l^2))
        sprintf("%s  ->  CR = %.3f  |  AVE = %.3f", f, cr_val, ave_val)
      }, character(1))

      header <- paste0(rep("-", 42), collapse = "")
      return(paste(c(header, lines, header), collapse = "\n"))

    } else {
      stop(paste("Unknown reliability method:", method))
    }
  }, error = function(e) {
    warning(paste("Error in reliability_func method '", method, "': ", e$message, sep=""))
    return(NA_real_)
  })

  if (is.na(result_value)) {
    return("Calculation failed or N/A.")
  } else {
    return(sprintf("%.3f", as.numeric(result_value)))
  }
}

# ---------------------------------------------------------------------------
# item_weighting
# ---------------------------------------------------------------------------
#' Item Weighting Function
#'
#' Applies a specific item weighting algorithm.
#' @param x A numeric data frame.
#' @return A data frame with weighted scores.
#' @importFrom psychometric item.exam
#' @importFrom stats na.omit
#' @importFrom psychometric item.exam
#' @noRd
item_weighting <- function(x) {
  item_stats <- psychometric::item.exam(x = x, discrim = TRUE)
  item_diff <- item_stats$Difficulty / max(x)
  ind_averages <- rowSums(x) / (ncol(x) * max(x))
  new_data <- data.frame(matrix(NA, nrow = nrow(x), ncol = ncol(x)))
  colnames(new_data) <- paste("V", seq(1:ncol(x)), sep = "")
  weighted_data <- data.frame(matrix(NA, nrow = nrow(x), ncol = ncol(x)))
  colnames(weighted_data) <- paste("V", seq(1:ncol(x)), sep = "")
  for (i in 1:ncol(x)) {
    for (t in 1:nrow(x)) {
      new_data[t, i] <- ind_averages[t] + item_diff[i]
    }
  }
  for (i in 1:ncol(x)) {
    for (t in 1:nrow(x)) {
      if (new_data[t, i] >= 1) {
        weighted_data[t, i] <- x[t, i] + item_stats$Item.Rel.woi[i]
      } else {
        weighted_data[t, i] <- x[t, i]
      }
    }
  }
  return(weighted_data)
}

#' Missing Value Handling UI Module
#'
#' @param id Module namespace ID.
#' @import shiny
#' @import bslib
#' @importFrom bsicons bs_icon
#' @export
mod_missing_ui <- function(id) {
  ns <- NS(id)
  tagList(
    layout_columns(
      col_widths = c(4, 8),

      # Configuration Card
      card(
        card_header(
          class = "bg-primary text-white",
          "Imputation Settings",
          bs_icon("gear")
        ),
        card_body(
          p("Analyze missing data patterns and apply advanced imputation strategies."),
          selectInput(
            ns("imputation_method"),
            "Choose Imputation Method:",
            choices = c(
              "Visualize Only (No Action)" = "none",
              "Listwise Deletion (Remove Rows)" = "listwise",
              "Mean Imputation (Simple)" = "mean",
              "Median Imputation (Simple)" = "median",
              "Amelia II (Expectation-Maximization)" = "amelia",
              "MICE (Predictive Mean Matching)" = "mice",
              "Random Forest (Categorical / Integer)" = "missForest_cat", # YENİ
              "Random Forest (Continuous / Decimal)" = "missForest_cont"  # YENİ
            ),
            selected = "none"
          ),
          helpText(
            icon("info-circle"),
            "Note: Use 'RF (Categorical)' for Likert scales (1,2,3...) to preserve integer structure. Use 'RF (Continuous)' for decimal values."
          ),
          actionButton(
            ns("apply_imputation"),
            "Apply Imputation",
            class = "btn-success w-100",
            icon = icon("check")
          ),
          hr(),
          downloadButton(
            ns("download_data"),
            "Download Processed Data",
            class = "btn-secondary w-100"
          )
        )
      ),

      # Visualization Card
      card(
        card_header("Missingness Map"),
        card_body(
          plotOutput(ns("missing_plot"), height = "400px")
        )
      )
    ),

    # Summary Table & MCAR Test Card
    card(
      card_header("Missing Data Summary & Diagnostics"),
      card_body(
        layout_columns(
          col_widths = c(6, 6),

          # Sol Kolon: MCAR Testi
          div(
            h5("Little's MCAR Test"),
            p("Test whether missing data is Missing Completely at Random (MCAR)."),
            actionButton(ns("run_mcar_button"), "Run MCAR Test", icon = icon("play-circle"), class = "btn-warning mb-2"),
            verbatimTextOutput(ns("mcar_output"))
          ),

          # Sag Kolon: Ozet Tablo
          div(
            h5("Missingness by Variable"),
            tableOutput(ns("missing_summary_table"))
          )
        )
      )
    )
  )
}

#' Confirmatory Factor Analysis (CFA) Server Module
#'
#' @param id Module namespace ID.
#' @param data Reactive containing the input dataset.
#' @importFrom utils head
#' @noRd
cfa_server <- function(id, data) {
  moduleServer(id, function(input, output, session) {

    cfa_analysis_results_rv <- reactiveValues(
      lavaan_object          = NULL,
      fit_measures_df        = NULL,
      factor_loadings_df     = NULL,
      modification_indices_df = NULL
    )

    build_bifactor_layout <- function(fit) {
      load_rows <- tryCatch(lavaan::parameterTable(fit), error = function(e) NULL)
      if (is.null(load_rows)) return(NULL)

      load_rows <- load_rows[load_rows$op == "=~", c("lhs", "rhs"), drop = FALSE]
      if (nrow(load_rows) == 0) return(NULL)

      latent_names <- unique(load_rows$lhs)
      item_names <- unique(load_rows$rhs)
      if (length(latent_names) < 2 || length(item_names) < 3) return(NULL)

      loading_counts <- stats::setNames(
        vapply(latent_names, function(f) sum(load_rows$lhs == f), integer(1)),
        latent_names
      )
      general_name <- names(which.max(loading_counts))
      if (loading_counts[[general_name]] < max(3, ceiling(0.75 * length(item_names)))) {
        return(NULL)
      }

      group_names <- setdiff(latent_names, general_name)
      if (length(group_names) == 0) return(NULL)

      group_items <- lapply(group_names, function(g) load_rows$rhs[load_rows$lhs == g])
      names(group_items) <- group_names

      ordered_items <- unique(unlist(group_items, use.names = FALSE))
      ordered_items <- c(ordered_items, setdiff(item_names, ordered_items))
      item_x <- stats::setNames(seq(-1.35, 0.35, length.out = length(ordered_items)), ordered_items)

      sem_model <- tryCatch(semPlot::semPlotModel(fit), error = function(e) NULL)
      if (is.null(sem_model)) return(NULL)
      node_names <- sem_model@Vars$name

      layout <- matrix(0, nrow = length(node_names), ncol = 2)
      rownames(layout) <- node_names

      for (item in intersect(ordered_items, node_names)) {
        layout[item, ] <- c(item_x[[item]], 1)
      }

      group_y <- seq(-0.85, 0.45, length.out = length(group_names))
      for (i in seq_along(group_names)) {
        g <- group_names[[i]]
        if (!(g %in% node_names)) next
        x_vals <- item_x[intersect(group_items[[g]], names(item_x))]
        layout[g, ] <- c(mean(x_vals, na.rm = TRUE), group_y[[i]])
      }

      if (general_name %in% node_names) {
        layout[general_name, ] <- c(1.25, 0)
      }

      layout
    }

        # 1. POPULATE SELECTORS
        observe({
      req(data())
      vars <- names(data())
      updateSelectizeInput(session, "builder_items",    choices = vars)
      updateSelectizeInput(session, "builder_cov_items",choices = vars)
      updateSelectizeInput(session, "bf_group_items",   choices = vars)
    })

        # Bifactor state: tracks general name + group factors
        bf_state <- reactiveValues(
      general  = "g",
      groups   = list()   # list of named character vectors: list(S1=c("i1","i2"), ...)
    )

        # Helper: append line to syntax textarea
        append_syntax <- function(new_text) {
      cur <- input$cfa_model_syntax_input
      val <- if (is.null(cur) || nchar(trimws(cur)) == 0) new_text
             else paste(cur, new_text, sep = "\n")
      updateTextAreaInput(session, "cfa_model_syntax_input", value = val)
    }

        # TAB 1: First-order factor  (=~)
        observeEvent(input$btn_add_factor, {
      f_name <- trimws(input$builder_factor_name %||% "")
      if (nchar(f_name) == 0) {
        showNotification("Factor name is required.", type = "warning"); return()
      }
      if (is.null(input$builder_items) || length(input$builder_items) == 0) {
        showNotification("Select at least one indicator.", type = "warning"); return()
      }
      append_syntax(paste0(f_name, " =~ ", paste(input$builder_items, collapse = " + ")))
      updateTextInput(session, "builder_factor_name", value = "")
      updateSelectizeInput(session, "builder_items", selected = character(0))
    })

    # Residual covariance (~~)
    observeEvent(input$btn_add_cov, {
      if (length(input$builder_cov_items) != 2) {
        showNotification("Select exactly 2 variables.", type = "warning"); return()
      }
      append_syntax(paste0(input$builder_cov_items[1], " ~~ ", input$builder_cov_items[2]))
      updateSelectizeInput(session, "builder_cov_items", selected = character(0))
    })

        # TAB 2: Higher-order factor
        observeEvent(input$btn_add_ho, {
      g_name <- trimws(input$ho_factor_name %||% "")
      fo     <- input$ho_first_order
      if (nchar(g_name) == 0) {
        showNotification("General factor name is required.", type = "warning"); return()
      }
      if (is.null(fo) || length(fo) < 2) {
        showNotification("Select at least 2 first-order factors.", type = "warning"); return()
      }
      append_syntax(paste0("\n# Higher-order factor\n", g_name, " =~ ",
                           paste(fo, collapse = " + ")))
      updateTextInput(session, "ho_factor_name", value = "")
      updateSelectizeInput(session, "ho_first_order", selected = character(0))
      showNotification(paste0("Higher-order factor ", g_name, " added."), type = "message")
    })

    # Populate ho_first_order choices from syntax (extract LHS of =~)
    observe({
      syn <- input$cfa_model_syntax_input %||% ""
      lines <- strsplit(syn, "\n")[[1]]
      fo_factors <- unique(trimws(sub("\\s*=~.*", "", grep("=~", lines, value = TRUE))))
      fo_factors <- fo_factors[nchar(fo_factors) > 0]
      updateSelectizeInput(session, "ho_first_order",
                           choices  = fo_factors,
                           selected = character(0))
    })

        # TAB 3: Bifactor
    
    # Add group factor to bifactor state
    observeEvent(input$btn_add_bf_group, {
      g_name  <- trimws(input$bf_general_name %||% "g")
      s_name  <- trimws(input$bf_group_name   %||% "")
      s_items <- input$bf_group_items
      if (nchar(s_name) == 0) {
        showNotification("Group factor name is required.", type = "warning"); return()
      }
      if (is.null(s_items) || length(s_items) == 0) {
        showNotification("Select at least one item for this group.", type = "warning"); return()
      }
      bf_state$general <- g_name
      bf_state$groups[[s_name]] <- s_items
      updateTextInput(session, "bf_group_name", value = "")
      updateSelectizeInput(session, "bf_group_items", selected = character(0))
      showNotification(
        paste0("Group factor ", s_name, " added (", length(s_items), " items). ",
               length(bf_state$groups), " group(s) defined so far."),
        type = "message"
      )
    })

    # Finalize bifactor syntax
    observeEvent(input$btn_finalize_bf, {
      if (length(bf_state$groups) < 1) {
        showNotification("Add at least one group factor first.", type = "warning"); return()
      }
      g   <- bf_state$general
      grps <- bf_state$groups
      all_items <- unique(unlist(grps))

      lines <- character(0)
      lines <- c(lines, "# --- Bifactor Model ---")

      # General factor
      lines <- c(lines, paste0(g, " =~ ", paste(all_items, collapse = " + ")))

      # Group factors
      for (s_name in names(grps)) {
        lines <- c(lines, paste0(s_name, " =~ ", paste(grps[[s_name]], collapse = " + ")))
      }

      # Orthogonality: g ~~ 0*Si and Si ~~ 0*Sj
      all_factors <- c(g, names(grps))
      orth_pairs <- utils::combn(all_factors, 2, simplify = FALSE)
      lines <- c(lines, "# Orthogonality constraints")
      for (pair in orth_pairs) {
        lines <- c(lines, paste0(pair[1], " ~~ 0*", pair[2]))
      }

      append_syntax(paste(lines, collapse = "\n"))
      # Reset state
      bf_state$groups  <- list()
      bf_state$general <- "g"
      showNotification("Bifactor syntax added. Check estimator: ML or WLSMV.", type = "message")
    })

        # TAB 4: Templates
        observeEvent(input$tpl_correlated, {
      tpl <- paste(
        "# Correlated factors model",
        "F1 =~ item1 + item2 + item3",
        "F2 =~ item4 + item5 + item6",
        "# F1 and F2 are allowed to correlate (default in lavaan CFA)",
        sep = "\n"
      )
      updateTextAreaInput(session, "cfa_model_syntax_input", value = tpl)
    })

    observeEvent(input$tpl_ho, {
      tpl <- paste(
        "# Higher-order (second-order) factor model",
        "# Step 1: first-order factors",
        "F1 =~ item1 + item2 + item3",
        "F2 =~ item4 + item5 + item6",
        "F3 =~ item7 + item8 + item9",
        "# Step 2: general factor loads on first-order factors",
        "g =~ F1 + F2 + F3",
        sep = "\n"
      )
      updateTextAreaInput(session, "cfa_model_syntax_input", value = tpl)
    })

    observeEvent(input$tpl_bifactor, {
      tpl <- paste(
        "# Bifactor model",
        "# General factor loads on ALL items",
        "g  =~ item1 + item2 + item3 + item4 + item5 + item6",
        "# Group (specific) factors",
        "S1 =~ item1 + item2 + item3",
        "S2 =~ item4 + item5 + item6",
        "# Orthogonality constraints (all factors uncorrelated)",
        "g  ~~ 0*S1",
        "g  ~~ 0*S2",
        "S1 ~~ 0*S2",
        sep = "\n"
      )
      updateTextAreaInput(session, "cfa_model_syntax_input", value = tpl)
    })

    observeEvent(input$btn_clear_syntax, {
      updateTextAreaInput(session, "cfa_model_syntax_input", value = "")
      showNotification("Syntax cleared.", type = "message")
    })

    # Dynamic estimator selection
    observeEvent(input$cfa_correlation_type_radio, {
      req(input$cfa_correlation_type_radio)
      estimator_choices <- if (input$cfa_correlation_type_radio == "pea") {
        c("Default (MLR)" = "default", "MLR", "ML", "GLS")
      } else {
        c("Default (WLSMV)" = "default", "WLSMV", "ULSMV", "DWLS")
      }
      updateSelectInput(session, "cfa_estimator_select", choices = estimator_choices, selected = "default")
    })

    # Run analysis
    observeEvent(input$run_cfa_button, {
      validate(
        need(data(), "Upload data."),
        need(input$cfa_model_syntax_input, "Define model.")
      )

      current_data <- data()
      model_syntax <- input$cfa_model_syntax_input
      corr_type <- input$cfa_correlation_type_radio
      est_sel <- input$cfa_estimator_select

      final_estimator <- est_sel
      if (est_sel == "default") {
        final_estimator <- if (corr_type == "poly") "WLSMV" else "MLR"
      }

      manifest_vars <- unique(unlist(lavaan::lavaanify(model_syntax)$rhs[lavaan::lavaanify(model_syntax)$op == "=~"]))
      ordered_arg <- if (corr_type == "poly") manifest_vars else FALSE

      progress_id <- showNotification("Running CFA...", duration = NULL, type = "message")
      on.exit(removeNotification(progress_id), add = TRUE)

      tryCatch({
        fit <- lavaan::cfa(
          model = model_syntax,
          data = current_data,
          ordered = ordered_arg,
          estimator = final_estimator,
          missing = "listwise",
          mimic = "Mplus" # Critical for Mplus match
        )
        cfa_analysis_results_rv$lavaan_object <- fit

        # Fit Measures
        fm_raw <- tryCatch(lavaan::fitMeasures(fit), error = function(e) NULL)
        if(!is.null(fm_raw)) {
          chi <- fm_raw["chisq.scaled"] %||% fm_raw["chisq"]
          df  <- fm_raw["df.scaled"] %||% fm_raw["df"]
          p   <- fm_raw["pvalue.scaled"] %||% fm_raw["pvalue"]
          cfi <- fm_raw["cfi.scaled"] %||% fm_raw["cfi.robust"] %||% fm_raw["cfi"]
          tli <- fm_raw["tli.scaled"] %||% fm_raw["tli.robust"] %||% fm_raw["tli"]
          rmsea     <- fm_raw["rmsea.scaled"] %||% fm_raw["rmsea.robust"] %||% fm_raw["rmsea"]
          rmsea_low <- fm_raw["rmsea.ci.lower.scaled"] %||% fm_raw["rmsea.ci.lower.robust"] %||% fm_raw["rmsea.ci.lower"]
          rmsea_upp <- fm_raw["rmsea.ci.upper.scaled"] %||% fm_raw["rmsea.ci.upper.robust"] %||% fm_raw["rmsea.ci.upper"]
          srmr <- fm_raw["srmr"] %||% fm_raw["srmr_bentler"]

          cfa_analysis_results_rv$fit_measures_df <- data.frame(
            Measure = c("Chi-Square", "Degrees of Freedom (df)", "p-value",
                        "CFI", "TLI (NNFI)", "RMSEA", "RMSEA 90% CI Lower", "RMSEA 90% CI Upper", "SRMR"),
            Value = round(c(chi, df, p, cfi, tli, rmsea, rmsea_low, rmsea_upp, srmr), 3)
          )
          if(!is.na(p)) cfa_analysis_results_rv$fit_measures_df$Value[3] <- format.pval(p, digits=3, eps=0.001)
        }

        # Loadings
        std_sol <- lavaan::standardizedSolution(fit)
        loadings <- std_sol[std_sol$op == "=~", c("lhs", "rhs", "est.std", "se", "pvalue")]
        colnames(loadings) <- c("Factor", "Item", "Std. Estimate", "SE", "p-value")
        loadings[,3:4] <- round(loadings[,3:4], 3)
        loadings[,5]   <- format.pval(loadings[,5], digits=3, eps=0.001)
        cfa_analysis_results_rv$factor_loadings_df <- loadings

        # Mod Indices
        mod_ind <- lavaan::modificationIndices(fit, sort. = TRUE, minimum.value = 3.84)
        cfa_analysis_results_rv$modification_indices_df <- head(mod_ind[, c("lhs","op","rhs","mi","epc")], 20)

        showNotification("CFA Analysis Complete!", type = "message")

      }, error = function(e) {
        showNotification(paste("Error:", e$message), type = "error", duration = 8)
        cfa_analysis_results_rv$lavaan_object <- NULL
      })
    })

    # Render outputs

    output$cfa_path_diagram_output <- renderPlot({
      req(cfa_analysis_results_rv$lavaan_object)

      # UI Controls
      selected_layout <- input$plot_layout %||% "tree"
      rotation_val    <- input$plot_rotation %||% 2
      box_width       <- input$plot_man_size %||% 10
      label_cex       <- input$plot_edge_label_cex %||% 0.8

      if(selected_layout == "tree2") {
        selected_layout <- "tree"
        rotation_val <- 2
      }

      bifactor_layout <- build_bifactor_layout(cfa_analysis_results_rv$lavaan_object)
      if (!is.null(bifactor_layout) && selected_layout %in% c("tree", "tree2")) {
        selected_layout <- bifactor_layout
      }

      what_labels <- if(!is.null(input$plot_show_labels) && input$plot_show_labels) "std" else "hide"

      is_poly <- (input$cfa_correlation_type_radio == "poly")
      custom_edge_width <- if(is_poly) 0.5 else 1.2
      # For polychoric/ordinal: residuals are fixed to 1.0 (WLSMV identification
      # constraint) and overlap badly. Hide them - loadings carry the key info.
      show_residuals <- !is_poly

      semPlot::semPaths(
        object = cfa_analysis_results_rv$lavaan_object,
        what = "std",
        whatLabels = what_labels,
        layout = selected_layout,
        rotation = rotation_val,

        shapeMan      = "rectangle",
        sizeMan       = box_width,
        sizeMan2      = box_width / 2,
        sizeLat       = box_width,
        sizeLat2      = box_width / 2,

        label.cex     = 1.2,
        edge.label.cex = label_cex,
        edge.width    = custom_edge_width,
        edge.color    = "black",
        style         = "lisrel",

        intercepts    = FALSE,
        thresholds    = FALSE,
        residuals     = show_residuals,
        residScale    = 15,

        reorder          = FALSE,
        optimizeLatRes   = TRUE,
        curve            = 2.5,
        mar              = c(5, 5, 5, 5),
        nCharNodes       = 0,
        theme            = "gray"
      )
    })

    output$cfa_fit_measures_table <- renderTable({ cfa_analysis_results_rv$fit_measures_df }, striped = TRUE, bordered = TRUE)
    output$cfa_factor_loadings_table <- renderTable({ cfa_analysis_results_rv$factor_loadings_df }, striped = TRUE)
    output$cfa_modification_indices_table <- renderTable({ cfa_analysis_results_rv$modification_indices_df }, striped = TRUE)

    # Downloads
    output$download_fit_measures_button <- downloadHandler(
      filename = "cfa_fit_measures.csv", content = function(file) write.csv(cfa_analysis_results_rv$fit_measures_df, file)
    )
    output$download_factor_loadings_button <- downloadHandler(
      filename = "cfa_factor_loadings.csv", content = function(file) write.csv(cfa_analysis_results_rv$factor_loadings_df, file)
    )
    output$download_path_diagram_button <- downloadHandler(
      filename = "cfa_path_diagram.svg",
      content = function(file) {
        svg(file, width = 12, height = 8)

        selected_layout <- input$plot_layout %||% "tree"
        rotation_val <- input$plot_rotation %||% 2
        box_width <- input$plot_man_size %||% 10
        if(selected_layout == "tree2") { selected_layout <- "tree"; rotation_val <- 2 }
        bifactor_layout <- build_bifactor_layout(cfa_analysis_results_rv$lavaan_object)
        if (!is.null(bifactor_layout) && selected_layout %in% c("tree", "tree2")) {
          selected_layout <- bifactor_layout
        }
        is_poly <- (input$cfa_correlation_type_radio == "poly")
        custom_edge_width <- if(is_poly) 0.5 else 1.2

        is_poly_dl <- (input$cfa_correlation_type_radio == "poly")
        semPlot::semPaths(
          object = cfa_analysis_results_rv$lavaan_object,
          what = "std", whatLabels = "std",
          layout = selected_layout, rotation = rotation_val,
          shapeMan = "rectangle", sizeMan = box_width, sizeMan2 = box_width/2,
          edge.width    = custom_edge_width,
          thresholds    = FALSE,
          reorder       = FALSE,
          residScale    = 15,
          residuals     = !is_poly_dl,
          edge.color    = "black",
          style         = "lisrel",
          intercepts    = FALSE,
          nCharNodes    = 0
        )
        dev.off()
      }
    )
  })
}

#' EFA Replicability UI Module
#' @import shiny
#' @import bslib
#' @importFrom bsicons bs_icon
#' @noRd
efa_ui_replicability <- function(id) {
  ns <- NS(id)
  tagList(
    layout_columns(
      col_widths = c(4, 8),
      card(
        card_header("Replicability Configuration", class = "bg-primary text-white", bs_icon("shuffle")),
        card_body(
          div(
            class = "d-grid gap-3",
            div(
              class = "p-3 rounded",
              style = "background:#f8fafc; border:1px solid #e2e8f0;",
              radioButtons(
                ns("cor_kind"), "Correlation:",
                choices = c("Pearson" = "pea", "Polychoric" = "poly"),
                selected = "poly",
                inline = TRUE
              ),
              tags$p(
                class = "text-muted small mb-0",
                "Use the same correlation choice as the main EFA module."
              )
            ),
            numericInput(ns("number_factor"), "No. of Factors:", value = 1, min = 1),
            selectizeInput(
              ns("fact_method"), "Extraction:",
              choices = c(
                "Minimum Residuals"         = "minres",
                "Maximum Likelihood"        = "ml",
                "Principal Axis"            = "pa",
                "Unweighted Least Squares"  = "uls",
                "Weighted Least Squares"    = "wls",
                "Minimum Rank"              = "minrank",
                "Minimum Chi-Square"        = "minchi",
                "Generalized Least Squares" = "gls"
              ),
              selected = "minres",
              options = list(dropdownParent = "body")
            ),
            selectizeInput(
              ns("rotating_method"), "Rotation:",
              choices = list(
                "None" = list("None" = "none"),
                "Oblique" = list(
                  "Oblimin"    = "oblimin",
                  "Promax"     = "promax",
                  "Quartimin"  = "quartimin",
                  "BiquartMin" = "biquartimin",
                  "GeominQ"    = "geominQ",
                  "BentlerQ"   = "bentlerQ",
                  "Simplimax"  = "simplimax",
                  "Cluster"    = "cluster"
                ),
                "Orthogonal" = list(
                  "Varimax"    = "varimax",
                  "Quartimax"  = "quartimax",
                  "Equamax"    = "equamax",
                  "BentlerT"   = "bentlerT",
                  "GeominT"    = "geominT",
                  "Bifactor"   = "bifactor"
                )
              ),
              selected = "oblimin",
              options = list(dropdownParent = "body")
            ),
            sliderInput(
              ns("split_prop"), "Split 1 proportion:",
              min = 0.30, max = 0.70, value = 0.50, step = 0.05
            ),
            numericInput(ns("seed"), "Random seed:", value = 1234, min = 1, step = 1)
          ),
          hr(),
          actionButton(
            ns("run_replicability"), "Run Replicability",
            icon = icon("play"), class = "btn-success w-100 btn-lg"
          )
        )
      ),
      card(
        card_header("Replicability Results", bs_icon("table")),
        card_body(
          navset_card_tab(
            nav_panel(
              title = "Loadings Comparison", icon = bs_icon("columns-gap"),
              tableOutput(ns("loadings_comparison")) %>%
                shinycssloaders::withSpinner(type = 8, color = "#2C3E50"),
              downloadButton(ns("download_replicability"), "Download CSV", class = "btn-sm mt-2")
            ),
            nav_panel(
              title = "Summary", icon = bs_icon("clipboard-data"),
              tableOutput(ns("summary_table")),
              tableOutput(ns("factor_congruence"))
            )
          )
        )
      )
    )
  )
}

#' Item Weighting UI
#' @noRd
item_weighting_ui <- function(id) {
  ns <- NS(id)
  tagList(
    card(
      card_header("Item Weighting (Kilic & Dogan)", class = "bg-primary text-white"),
      card_body(
        p("Applies psychometric weighting based on item difficulty and discrimination."),
        actionButton(ns("calculate_weighted_scores_button"), "Calculate Scores", icon = icon("balance-scale"), class = "btn-primary"),
        hr(),
        h5("Preview"),
        tableOutput(ns("weighted_scores_table_output")),
        downloadButton(ns("download_weighted_data_button"), "Download Weighted Data")
      )
    )
  )
}

#' Confirmatory Factor Analysis (CFA) UI Module
#' @import shiny
#' @import bslib
#' @importFrom bsicons bs_icon
#' @importFrom shinycssloaders withSpinner
#' @noRd
cfa_ui <- function(id) {
  ns <- NS(id)
  tagList(
    layout_columns(
      col_widths = c(4, 8),

      # -- LEFT: Model Builder ----------------------------------
      card(
        card_header(class = "bg-info text-white",
                    bs_icon("magic"), " Model Builder"),
        card_body(
          navset_pill(
            id = ns("builder_tabs"),

            # -- Tab 1: First-order factors --
            nav_panel("1st-Order", value = "tab_fo",
              p(class = "text-muted small mt-2 mb-1",
                "Define each latent factor and its indicators."),
              textInput(ns("builder_factor_name"), "Factor Name:",
                        placeholder = "e.g. F1, Anxiety ..."),
              selectizeInput(
                ns("builder_items"), "Indicators:",
                choices = NULL, multiple = TRUE,
                options = list(placeholder = "Select observed variables...")
              ),
              actionButton(ns("btn_add_factor"), "Add to Syntax",
                           icon = icon("code"),
                           class = "btn-secondary btn-sm w-100 mb-3"),
              hr(class = "my-2"),
              h6("Add Residual Covariance (~~)", class = "text-muted"),
              selectizeInput(
                ns("builder_cov_items"), NULL,
                choices = NULL, multiple = TRUE,
                options = list(maxItems = 2,
                               placeholder = "Select 2 variables...")
              ),
              actionButton(ns("btn_add_cov"), "Add Covariance",
                           icon = icon("link"),
                           class = "btn-outline-secondary btn-sm w-100")
            ),

            # -- Tab 2: Higher-order --
            nav_panel("Higher-Order", value = "tab_ho",
              div(
                class = "alert alert-info p-2 mt-2",
                style = "font-size:0.78rem;",
                bs_icon("info-circle"), " ",
                "Higher-order (second-order) model: a general factor ",
                tags$b("loads on first-order latent factors"),
                " (not directly on items). ",
                "First-order factors must already be defined in the syntax."
              ),
              textInput(ns("ho_factor_name"), "General Factor Name:",
                        placeholder = "e.g. g, General, HO ..."),
              selectizeInput(
                ns("ho_first_order"), "First-Order Factors:",
                choices = NULL, multiple = TRUE,
                options = list(placeholder = "Select latent factors ...")
              ),
              p(class = "text-muted small",
                "Tip: Type factor names manually if not yet in syntax."),
              actionButton(ns("btn_add_ho"), "Add Higher-Order Factor",
                           icon = icon("layer-group"),
                           class = "btn-primary btn-sm w-100")
            ),

            # -- Tab 3: Bifactor --
            nav_panel("Bifactor", value = "tab_bf",
              div(
                class = "alert alert-warning p-2 mt-2",
                style = "font-size:0.78rem;",
                bs_icon("info-circle"), " ",
                "Bifactor model: a ",
                tags$b("general factor (g)"),
                " loads on ALL items; specific group factors load on ",
                "subsets of items. All factors are ",
                tags$b("orthogonal"),
                " to each other."
              ),
              textInput(ns("bf_general_name"), "General Factor Name:",
                        value = "g", placeholder = "e.g. g"),
              hr(class = "my-2"),
              h6("Group Factor", class = "text-primary"),
              textInput(ns("bf_group_name"), "Group Factor Name:",
                        placeholder = "e.g. S1, Specific1 ..."),
              selectizeInput(
                ns("bf_group_items"), "Items for this Group:",
                choices = NULL, multiple = TRUE,
                options = list(placeholder = "Select observed variables...")
              ),
              actionButton(ns("btn_add_bf_group"), "Add Group Factor",
                           icon = icon("plus"),
                           class = "btn-secondary btn-sm w-100 mb-2"),
              hr(class = "my-2"),
              actionButton(ns("btn_finalize_bf"), "Finalize Bifactor Syntax",
                           icon = icon("check-circle"),
                           class = "btn-success btn-sm w-100"),
              p(class = "text-muted small mt-1",
                "Finalize: adds g =~ all items, all orthogonality constraints (~~0*) to syntax.")
            ),

            # -- Tab 4: Templates --
            nav_panel("Templates", value = "tab_tpl",
              p(class = "text-muted small mt-2",
                "Load a template into the syntax editor, then modify."),
              actionButton(ns("tpl_correlated"), "Correlated Factors",
                           class = "btn-outline-primary btn-sm w-100 mb-2"),
              actionButton(ns("tpl_ho"), "Higher-Order (2nd-order)",
                           class = "btn-outline-primary btn-sm w-100 mb-2"),
              actionButton(ns("tpl_bifactor"), "Bifactor",
                           class = "btn-outline-primary btn-sm w-100 mb-2"),
              actionButton(ns("btn_clear_syntax"), "Clear Syntax",
                           icon = icon("trash"),
                           class = "btn-outline-danger btn-sm w-100 mt-3")
            )
          )
        )
      ),

      # -- RIGHT: Syntax + Settings + Results ------------------
      tagList(
        card(
          card_header(class = "bg-primary text-white",
                      bs_icon("sliders"), " CFA Analysis Setup"),
          card_body(
            textAreaInput(
              ns("cfa_model_syntax_input"),
              label = "Model Syntax (lavaan):",
              placeholder = paste(
                "# First-order:",
                "F1 =~ item1 + item2 + item3",
                "F2 =~ item4 + item5 + item6",
                "",
                "# Higher-order:",
                "g =~ F1 + F2",
                "",
                "# Bifactor orthogonality:",
                "g ~~ 0*F1",
                sep = "\n"
              ),
              rows = 8, resize = "vertical"
            ),
            layout_columns(
              col_widths = c(6, 6),
              radioButtons(
                ns("cfa_correlation_type_radio"), "Data Type:",
                choices  = c("Continuous (Pearson)" = "pea",
                             "Ordinal (Polychoric)" = "poly"),
                inline   = TRUE, selected = "pea"
              ),
              selectInput(
                ns("cfa_estimator_select"), "Estimator:",
                choices  = c("ML", "MLR", "MLM", "GLS", "WLS",
                             "WLSMV", "DWLS"),
                selected = "ML"
              )
            ),
            div(
              class = "alert alert-secondary p-2",
              style = "font-size:0.76rem;",
              bs_icon("lightbulb"), " ",
              tags$b("Ordinal/Polychoric:"), " WLSMV recommended.  ",
              tags$b("Bifactor:"), " add ",
              tags$code("orthogonal = TRUE"),
              " constraint or use the Bifactor builder tab.  ",
              tags$b("Higher-order:"), " g =~ F1 + F2 in lavaan syntax."
            ),
            actionButton(ns("run_cfa_button"), "Run CFA",
                         icon = icon("play"),
                         class = "btn-success w-100 btn-lg")
          )
        ),

        # Results tabs
        navset_card_tab(
          full_screen = TRUE,
          nav_panel("Path Diagram", icon = bs_icon("diagram-3"),
            layout_sidebar(
              sidebar = sidebar(
                title = "Plot Settings", width = 250,
                selectInput(ns("plot_layout"), "Layout:",
                  choices = c("Tree (top-down)"   = "tree",
                              "Tree (left-right)"  = "tree2",
                              "Spring"             = "spring",
                              "Circle"             = "circle")),
                sliderInput(ns("plot_rotation"),       "Rotation:",    1, 4, 2, 1),
                sliderInput(ns("plot_man_size"),        "Box Width:",   4, 20, 10),
                sliderInput(ns("plot_edge_label_cex"), "Label Size:",  0.5, 1.5, 0.8, 0.1),
                checkboxInput(ns("plot_show_labels"), "Show Estimates", TRUE)
              ),
              plotOutput(ns("cfa_path_diagram_output"), height = "600px") %>%
                withSpinner(type = 8, color = "#2C3E50"),
              downloadButton(ns("download_path_diagram_button"),
                             "Download (.svg)", class = "btn-sm")
            )
          ),
          nav_panel("Fit Measures", icon = bs_icon("table"),
            tableOutput(ns("cfa_fit_measures_table")),
            downloadButton(ns("download_fit_measures_button"), "Download CSV", class = "btn-sm")
          ),
          nav_panel("Factor Loadings", icon = bs_icon("list-ol"),
            tableOutput(ns("cfa_factor_loadings_table")),
            downloadButton(ns("download_factor_loadings_button"), "Download CSV", class = "btn-sm")
          ),
          nav_panel("Modification Indices", icon = bs_icon("tools"),
            tableOutput(ns("cfa_modification_indices_table"))
          )
        )
      )
    )
  )
}

#' Exploratory Graph Analysis (EGA) Server Module Logic
#'
#' Handles the server-side logic for performing EGA, including input validation,
#' running the EGAnet::EGA function, rendering results (network matrix, plot,
#' dimensionality summary, item-community assignments), and providing download options.
#'
#' @param id Module namespace ID.
#' @param data A reactive expression returning the current dataset.
#'
#' @import shiny
#' @importFrom EGAnet EGA
#' @importFrom utils write.csv
#' @importFrom grDevices svg dev.off
#' @importFrom graphics plot text
#' @importFrom stats var na.omit
#' @noRd
ega_server <- function(id, data) {
  moduleServer(id, function(input, output, session) {

    # Helper for safe access
    `%||%` <- function(x, y) if (is.null(x) || length(x) == 0 || all(is.na(x))) y else x

    # reactiveValues to store EGA results
    ega_analysis_results_rv <- reactiveValues(
      ega_object = NULL,
      network_matrix = NULL,
      dimensionality_summary = NULL,
      item_community_assignments = NULL
    )

    observeEvent(input$run_ega_button, {
      # --- Input Validations ---
      validate(
        need(data(), "Please upload your dataset to run EGA."),
        need(input$ega_estimation_method_select, "Please select an EGA estimation method."),
        need(input$ega_correlation_type_radio, "Please select a correlation type for EGA.")
      )

      current_data <- data()

      validate(
        need(all(sapply(current_data, is.numeric)), "All columns in the dataset must be numeric for EGA."),
        need(ncol(current_data) > 1, "Dataset must have at least two variables for EGA."),
        need(nrow(current_data) > ncol(current_data), "Sample size should ideally be greater than the number of variables for stable network estimation.")
      )

      col_variances <- apply(current_data, 2, var, na.rm = TRUE)
      validate(
        need(all(col_variances > 1e-6), "One or more variables have zero or near-zero variance. Please remove them or check your data.")
      )

      progress_id <- showNotification("Running Exploratory Graph Analysis...", duration = NULL, type = "message")
      on.exit(removeNotification(progress_id), add = TRUE)

      # --- Perform EGA Analysis ---
      tryCatch({
        correlation_method_for_ega <- input$ega_correlation_type_radio

        ega_output <- EGAnet::EGA(
          data      = current_data,
          model     = input$ega_estimation_method_select,
          algorithm = input$ega_algorithm_select %||% "walktrap",
          corr      = correlation_method_for_ega,
          plot.EGA = TRUE,
          plot.type = "qgraph",
          plot.args = list(
            vsize = 7,
            label.cex = 1,
            edge.width = 1.5,
            layout = "spring",
            theme = "TeamFortress",
            legend.cex = 0.7,
            GLratio = 1.5
          ),
          verbose = FALSE
        )

        ega_analysis_results_rv$ega_object <- ega_output

        if (!is.null(ega_output$network)) {
          ega_analysis_results_rv$network_matrix <- as.data.frame(as.matrix(ega_output$network))
        } else {
          ega_analysis_results_rv$network_matrix <- data.frame(Message = "Network matrix not available from EGA output.")
        }

        dim_summary_text <- paste0(
          "Number of Dimensions (Communities) Identified: ", ega_output$n.dim %||% "N/A", "\n\n",
          "Item to Community Assignments:\n"
        )
        ega_analysis_results_rv$dimensionality_summary <- dim_summary_text

        if(!is.null(ega_output$wc)){
          item_comm_df <- data.frame(
            Item = names(ega_output$wc),
            Community = ega_output$wc,
            stringsAsFactors = FALSE
          )
          ega_analysis_results_rv$item_community_assignments <- item_comm_df
        } else {
          ega_analysis_results_rv$item_community_assignments <- data.frame(Message="Community assignments (wc) not available.")
        }

        # --- Render Outputs ---
        output$ega_network_table_output <- renderTable({
          validate(need(!is.null(ega_analysis_results_rv$network_matrix) && nrow(ega_analysis_results_rv$network_matrix) > 0 && !("Message" %in% colnames(ega_analysis_results_rv$network_matrix)),
                        "Network matrix is not available or empty."))
          round(ega_analysis_results_rv$network_matrix, 3)
        }, rownames = TRUE, striped = TRUE, hover = TRUE, bordered = TRUE)

        output$ega_network_plot_output <- renderPlot({
          validate(need(!is.null(ega_analysis_results_rv$ega_object), "EGA result object is not available for plotting."))
          plot(ega_analysis_results_rv$ega_object)
        }, width = 750, height = 550)

        output$ega_dimensionality_summary_output <- renderPrint({
          req(ega_analysis_results_rv$dimensionality_summary)
          cat(ega_analysis_results_rv$dimensionality_summary)
        })

        output$ega_item_community_table_output <- renderTable({
          req(ega_analysis_results_rv$item_community_assignments)
          validate(need(!("Message" %in% colnames(ega_analysis_results_rv$item_community_assignments)), ""))
          ega_analysis_results_rv$item_community_assignments
        }, striped = TRUE, hover = TRUE, bordered = TRUE)

        showNotification("EGA analysis completed successfully!", type = "message", duration = 4)

      }, error = function(e) {
        user_error_message <- if (!is.null(conditionMessage(e))) conditionMessage(e) else "An unspecified error occurred."
        showNotification(paste("Error (EGA):", user_error_message), type = "error", duration = 10)

        ega_analysis_results_rv$ega_object <- NULL
        ega_analysis_results_rv$network_matrix <- data.frame(Error = paste("Analysis failed:", user_error_message))
        ega_analysis_results_rv$dimensionality_summary <- paste("Analysis failed:", user_error_message)
        ega_analysis_results_rv$item_community_assignments <- data.frame(Error = paste("Analysis failed:", user_error_message))

        output$ega_network_table_output <- renderTable({ ega_analysis_results_rv$network_matrix })
        output$ega_network_plot_output <- renderPlot({ plot(NULL, xlim=c(0,1),ylim=c(0,1),main="Plot Error"); text(0.5,0.5,user_error_message)})
        output$ega_dimensionality_summary_output <- renderPrint({ cat(ega_analysis_results_rv$dimensionality_summary) })
        output$ega_item_community_table_output <- renderTable({ ega_analysis_results_rv$item_community_assignments })
      })
    })

    output$download_ega_network_button <- downloadHandler(
      filename = function() {
        paste0("ega_network_matrix_", input$ega_estimation_method_select, "_", Sys.Date(), ".csv")
      },
      content = function(file) {
        validate(need(!is.null(ega_analysis_results_rv$network_matrix) && !("Error" %in% colnames(ega_analysis_results_rv$network_matrix)) && nrow(ega_analysis_results_rv$network_matrix) > 0,
                      "Network matrix is not available for download or contains an error."))
        write.csv(ega_analysis_results_rv$network_matrix, file, row.names = TRUE)
      }
    )

    output$download_ega_plot_button <- downloadHandler(
      filename = function() {
        paste0("ega_network_plot_", input$ega_estimation_method_select, "_", Sys.Date(), ".svg")
      },
      content = function(file) {
        validate(need(!is.null(ega_analysis_results_rv$ega_object), "EGA results are not available for plot download."))
        svg(file, width = 10, height = 7.5)
        plot(ega_analysis_results_rv$ega_object)
        dev.off()
      }
    )
  })
}

# Internal ItemRest-style item removal engine for FAfA.
# Required imports: psych, gtools, qgraph, stats, utils.

.efa_cache <- new.env(hash = TRUE, parent = emptyenv())
.alpha_cache <- new.env(hash = TRUE, parent = emptyenv())

clear_efa_cache <- function() {
  rm(list = ls(envir = .efa_cache), envir = .efa_cache)
  rm(list = ls(envir = .alpha_cache), envir = .alpha_cache)
}

howard <- function(primary, secondary) {
  !(primary >= 0.40 && secondary < 0.30 && (primary - secondary) >= 0.20)
}

descriptive_stats <- function(data) {
  list(
    n_items   = ncol(data),
    n_obs     = nrow(data),
    min_value = min(data, na.rm = TRUE),
    max_value = max(data, na.rm = TRUE)
  )
}

cor_matrix_custom <- function(data, method = "polychoric") {
  if (method == "polychoric") {
    suppressMessages(suppressWarnings(qgraph::cor_auto(data)))
  } else {
    stats::cor(data, use = "pairwise.complete.obs", method = method)
  }
}

get_cor_matrix <- function(data, cor_method = "pearson") {
  cor_method <- match.arg(tolower(cor_method), c("pearson", "spearman", "polychoric"))
  colnames(data) <- gsub("\\.", "_", colnames(data))
  cor_matrix_custom(data, cor_method)
}

determine_n_factors <- function(data, cor_method = "pearson", cor_mat = NULL) {
  if (is.null(cor_mat)) {
    cor_mat <- cor_matrix_custom(data, cor_method)
  }
  fa_parallel <- suppressMessages(suppressWarnings(
    psych::fa.parallel(
      cor_mat,
      fa = "fa",
      n.iter = 20,
      n.obs = nrow(data),
      show.legend = FALSE,
      plot = FALSE,
      main = NULL
    )
  ))
  fa_parallel$nfact
}

alpha_custom <- function(data, use_cache = TRUE) {
  cache_key <- paste(sort(colnames(data)), collapse = "|")
  if (use_cache && exists(cache_key, envir = .alpha_cache, inherits = FALSE)) {
    return(get(cache_key, envir = .alpha_cache, inherits = FALSE))
  }

  alpha_obj <- NULL
  invisible(utils::capture.output({
    alpha_obj <- suppressMessages(suppressWarnings(
      psych::alpha(data, check.keys = FALSE)
    ))
  }))
  alpha_val <- alpha_obj$total$raw_alpha
  if (use_cache) {
    assign(cache_key, alpha_val, envir = .alpha_cache, inherits = FALSE)
  }
  alpha_val
}

efa_custom <- function(data, n_factors = 1, cor_method = "polychoric",
                       extract = "uls", rotate = "oblimin",
                       use_cache = TRUE, cor_mat = NULL) {
  if (use_cache) {
    cache_key <- paste(
      n_factors, cor_method, extract, rotate,
      paste(sort(colnames(data)), collapse = "|"),
      sep = "||"
    )
    if (exists(cache_key, envir = .efa_cache, inherits = FALSE)) {
      return(get(cache_key, envir = .efa_cache, inherits = FALSE))
    }
  }

  if (is.null(cor_mat)) {
    cor_mat <- cor_matrix_custom(data, cor_method)
  } else {
    cor_mat <- cor_mat[colnames(data), colnames(data), drop = FALSE]
  }

  efa <- NULL
  invisible(utils::capture.output({
    efa <- psych::fa(
      r = cor_mat,
      nfactors = n_factors,
      rotate = rotate,
      fm = extract
    )
  }))

  alpha_val <- alpha_custom(data, use_cache = use_cache)
  loading <- efa$loadings[]
  explained_var <- sum(efa$Vaccounted["SS loadings", seq_len(n_factors)]) / ncol(data)

  result <- list(
    efa = efa,
    alpha = alpha_val,
    explained_var = explained_var,
    loadings = loading
  )

  if (use_cache) {
    assign(cache_key, result, envir = .efa_cache, inherits = FALSE)
  }
  result
}

sort_item_ids <- function(x) {
  gtools::mixedsort(x)
}

identify_problem_items <- function(efa_res, min_loading = 0.30,
                                   loading_diff = 0.10) {
  loadings <- abs(efa_res$loadings)
  items <- rownames(loadings)
  cross_3 <- character(0)
  cross_2 <- character(0)
  low_loading <- character(0)

  if (ncol(loadings) == 0) {
    return(list(cross_3 = character(0), cross_2 = character(0), low_loading = character(0)))
  }

  for (i in seq_len(nrow(loadings))) {
    item_loads <- loadings[i, ]
    primary <- max(item_loads, na.rm = TRUE)

    if (all(item_loads < min_loading, na.rm = TRUE)) {
      low_loading <- c(low_loading, items[i])
      next
    }

    if (ncol(loadings) > 1) {
      sorted_loads <- sort(item_loads, decreasing = TRUE)
      secondary <- sorted_loads[2]

      if (identical(loading_diff, "howard")) {
        is_cross <- howard(primary, secondary)
      } else {
        is_cross <- (primary - secondary) <= as.numeric(loading_diff)
      }

      if (is_cross) {
        n_sig <- sum(item_loads >= min_loading, na.rm = TRUE)
        if (n_sig >= 3) {
          cross_3 <- c(cross_3, items[i])
        } else if (n_sig == 2) {
          cross_2 <- c(cross_2, items[i])
        } else {
          low_loading <- c(low_loading, items[i])
        }
      }
    }
  }

  list(
    cross_3 = unique(cross_3),
    cross_2 = unique(cross_2),
    low_loading = unique(low_loading)
  )
}

get_combinations <- function(items, max_size = NULL) {
  n <- length(items)
  if (n == 0) return(list())
  upper <- if (is.null(max_size)) n else min(n, max_size)
  unlist(
    lapply(seq_len(upper), function(i) utils::combn(items, i, simplify = FALSE)),
    recursive = FALSE
  )
}

build_summary_row <- function(remaining_items, removed_items, efa_out,
                              min_loading, loading_diff,
                              iteration = NA_integer_,
                              prior_removed = character(0)) {
  all_removed <- sort_item_ids(unique(c(prior_removed, removed_items)))
  prob <- identify_problem_items(efa_out, min_loading, loading_diff)
  cross_items <- sort_item_ids(unique(c(prob$cross_2, prob$cross_3)))
  low_items <- sort_item_ids(unique(prob$low_loading))

  load_mat <- as.matrix(efa_out$loadings)
  valid_loads <- abs(load_mat[abs(load_mat) >= min_loading])
  load_min <- if (length(valid_loads) > 0) formatC(min(valid_loads), digits = 2, format = "f") else NA_character_
  load_max <- if (length(valid_loads) > 0) formatC(max(valid_loads), digits = 2, format = "f") else NA_character_
  loading_range <- if (is.na(load_min)) "N/A" else paste0(load_min, "/", load_max)

  data.frame(
    Iteration = iteration,
    Removed_This_Step = if (length(removed_items) == 0) "None" else paste(sort_item_ids(removed_items), collapse = "-"),
    Removed_Items = if (length(all_removed) == 0) "None" else paste(all_removed, collapse = "-"),
    N_Removed = length(all_removed),
    Remaining_Items = paste(sort_item_ids(remaining_items), collapse = "-"),
    N_Remaining = length(remaining_items),
    Total_Explained_Var = efa_out$explained_var,
    Factor_Loading_Range = loading_range,
    Cronbachs_Alpha = efa_out$alpha,
    Has_Cross_Loading = ifelse(length(cross_items) > 0, "Yes", "No"),
    Cross_Loading_Items = ifelse(length(cross_items) > 0, paste(cross_items, collapse = ", "), "None"),
    Has_Low_Loading = ifelse(length(low_items) > 0, "Yes", "No"),
    Low_Loading_Items = ifelse(length(low_items) > 0, paste(low_items, collapse = ", "), "None"),
    stringsAsFactors = FALSE
  )
}

test_removals <- function(data, base_items, combs, n_factors, cor_method,
                          extract, rotate, min_loading, loading_diff,
                          min_items_per_factor = 3, early_exit = FALSE,
                          iteration = NA_integer_,
                          prior_removed = character(0),
                          cor_mat = NULL) {
  min_required <- n_factors * min_items_per_factor
  summary_list <- vector("list", length(combs))

  for (i in seq_along(combs)) {
    removed_items <- unlist(combs[[i]])
    remaining_items <- setdiff(base_items, removed_items)

    if (length(remaining_items) >= min_required) {
      efa_out <- tryCatch(
        efa_custom(
          data[, remaining_items, drop = FALSE],
          n_factors,
          cor_method,
          extract,
          rotate,
          use_cache = TRUE,
          cor_mat = cor_mat
        ),
        error = function(e) NULL
      )

      if (!is.null(efa_out)) {
        row <- build_summary_row(
          remaining_items,
          removed_items,
          efa_out,
          min_loading,
          loading_diff,
          iteration,
          prior_removed = prior_removed
        )

        summary_list[[i]] <- row

        if (early_exit && row$Has_Cross_Loading == "No" && row$Has_Low_Loading == "No") {
          return(row)
        }
      }
    }
  }

  result <- do.call(rbind, summary_list[!vapply(summary_list, is.null, logical(1))])
  if (is.null(result) || nrow(result) == 0) return(data.frame())

  result <- result[!duplicated(result$Removed_Items), ]
  result <- result[order(
    result$Has_Cross_Loading,
    result$Has_Low_Loading,
    -result$Total_Explained_Var,
    result$N_Removed
  ), ]
  row.names(result) <- NULL
  result
}

.parse_item_string <- function(s) {
  if (is.na(s) || s == "None" || nchar(trimws(s)) == 0) return(character(0))
  unlist(strsplit(s, "-"))
}

clean_result_rows <- function(tbl) {
  if (is.null(tbl) || nrow(tbl) == 0) return(data.frame())
  tbl[tbl$Has_Cross_Loading == "No" & tbl$Has_Low_Loading == "No", , drop = FALSE]
}

strategy_row_to_list <- function(row) {
  if (is.null(row) || nrow(row) == 0) return(NULL)
  row <- row[1, , drop = FALSE]
  list(
    row = row,
    removed_items = .parse_item_string(row$Removed_Items),
    remaining_items = .parse_item_string(row$Remaining_Items),
    n_removed = row$N_Removed,
    n_remaining = row$N_Remaining,
    explained_var = row$Total_Explained_Var,
    cronbachs_alpha = row$Cronbachs_Alpha,
    has_cross_loading = row$Has_Cross_Loading,
    cross_loading = .parse_item_string(gsub(", ", "-", row$Cross_Loading_Items)),
    has_low_loading = row$Has_Low_Loading,
    low_loading = .parse_item_string(gsub(", ", "-", row$Low_Loading_Items))
  )
}

build_recommended_strategy <- function(clean_solution, results_all,
                                       initial_efa, current_items) {
  if (!is.null(clean_solution) && !is.null(clean_solution$summary_row)) {
    out <- strategy_row_to_list(clean_solution$summary_row)
    out$source <- "clean_solution"
    return(out)
  }

  results_clean <- clean_result_rows(results_all)
  if (nrow(results_clean) > 0) {
    out <- strategy_row_to_list(results_clean[1, , drop = FALSE])
    out$source <- "clean_results"
    return(out)
  }

  if (!is.null(clean_solution)) {
    efa_out <- if (!is.null(clean_solution$efa)) clean_solution$efa else initial_efa
    return(list(
      row = NULL,
      source = "initial_or_iterative_clean",
      removed_items = clean_solution$removed_items,
      remaining_items = clean_solution$remaining_items,
      explained_var = efa_out$explained_var,
      cronbachs_alpha = efa_out$alpha
    ))
  }

  NULL
}

all_subsets_search <- function(data, base_items, problem_items, n_factors,
                               cor_method, extract, rotate, min_loading,
                               loading_diff, max_removed_per_iter,
                               min_items_per_factor, cor_mat = NULL) {
  if (length(problem_items) == 0) {
    return(list(summary = data.frame(), clean_solution = NULL))
  }

  combs <- get_combinations(problem_items, max_size = max_removed_per_iter)
  tbl <- test_removals(
    data, base_items, combs, n_factors, cor_method, extract, rotate,
    min_loading, loading_diff, min_items_per_factor,
    early_exit = FALSE, iteration = 1L, cor_mat = cor_mat
  )

  clean <- NULL
  if (nrow(tbl) > 0) {
    cand <- tbl[tbl$Has_Cross_Loading == "No" & tbl$Has_Low_Loading == "No", , drop = FALSE]
    if (nrow(cand) > 0) clean <- cand[1, , drop = FALSE]
  }

  list(summary = tbl, clean_solution = clean)
}

stepwise_search <- function(data, current_items, n_factors, cor_method,
                            extract, rotate, min_loading, loading_diff,
                            max_iter, max_removed_total, max_removed_per_iter,
                            min_items_per_factor, cor_mat = NULL) {
  already_removed <- character(0)
  all_iter_results <- list()
  clean_solution <- NULL
  visited <- character(0)

  for (iter in seq_len(max_iter)) {
    current_key <- paste(sort_item_ids(current_items), collapse = "-")
    if (current_key %in% visited) break
    visited <- c(visited, current_key)

    if (!is.null(max_removed_total) && length(already_removed) >= max_removed_total) break
    if (length(current_items) < n_factors * min_items_per_factor) break

    efa_now <- tryCatch(
      efa_custom(data[, current_items, drop = FALSE], n_factors, cor_method, extract, rotate, use_cache = TRUE, cor_mat = cor_mat),
      error = function(e) NULL
    )
    if (is.null(efa_now)) break

    prob <- identify_problem_items(efa_now, min_loading, loading_diff)
    problem_now <- sort_item_ids(unique(c(prob$cross_2, prob$cross_3, prob$low_loading)))

    if (length(problem_now) == 0) {
      clean_solution <- list(
        iteration = iter,
        removed_items = already_removed,
        remaining_items = current_items,
        efa = efa_now
      )
      break
    }

    max_this_round <- max_removed_per_iter
    if (!is.null(max_removed_total)) {
      quota <- max_removed_total - length(already_removed)
      if (quota <= 0) break
      max_this_round <- min(max_removed_per_iter, quota)
    }

    combs <- get_combinations(problem_now, max_size = max_this_round)
    if (length(combs) == 0) break

    iter_tbl <- test_removals(
      data, current_items, combs, n_factors, cor_method, extract, rotate,
      min_loading, loading_diff, min_items_per_factor,
      early_exit = FALSE, iteration = as.integer(iter),
      prior_removed = already_removed, cor_mat = cor_mat
    )

    if (nrow(iter_tbl) == 0) break
    all_iter_results[[iter]] <- iter_tbl

    cand <- iter_tbl[iter_tbl$Has_Cross_Loading == "No" & iter_tbl$Has_Low_Loading == "No", , drop = FALSE]
    if (nrow(cand) > 0) {
      best_clean <- cand[1, , drop = FALSE]
      clean_solution <- list(
        iteration = iter,
        summary_row = best_clean,
        removed_items = .parse_item_string(best_clean$Removed_Items),
        remaining_items = .parse_item_string(best_clean$Remaining_Items)
      )
      break
    }

    best_mid <- iter_tbl[1, , drop = FALSE]
    already_removed <- .parse_item_string(best_mid$Removed_Items)
    current_items <- .parse_item_string(best_mid$Remaining_Items)
  }

  list(
    clean_solution = clean_solution,
    iteration_results = all_iter_results,
    final_removed = already_removed,
    final_remaining = current_items
  )
}

backward_search <- function(data, current_items, problem_items, n_factors,
                            cor_method, extract, rotate, min_loading,
                            loading_diff, max_iter, max_removed_total,
                            min_items_per_factor, cor_mat = NULL) {
  already_removed <- character(0)
  all_iter_results <- list()
  clean_solution <- NULL
  visited <- character(0)

  for (iter in seq_len(max_iter)) {
    current_key <- paste(sort_item_ids(current_items), collapse = "-")
    if (current_key %in% visited) break
    visited <- c(visited, current_key)

    if (!is.null(max_removed_total) && length(already_removed) >= max_removed_total) break
    if (length(current_items) < n_factors * min_items_per_factor) break

    efa_now <- tryCatch(
      efa_custom(data[, current_items, drop = FALSE], n_factors, cor_method, extract, rotate, use_cache = TRUE, cor_mat = cor_mat),
      error = function(e) NULL
    )
    if (is.null(efa_now)) break

    prob_now <- identify_problem_items(efa_now, min_loading, loading_diff)
    problem_now <- sort_item_ids(intersect(
      unique(c(prob_now$cross_2, prob_now$cross_3, prob_now$low_loading)),
      current_items
    ))

    if (length(problem_now) == 0) {
      clean_solution <- list(
        iteration = iter,
        removed_items = already_removed,
        remaining_items = current_items,
        efa = efa_now
      )
      break
    }

    round_tbl <- test_removals(
      data, current_items, as.list(problem_now),
      n_factors, cor_method, extract, rotate,
      min_loading, loading_diff, min_items_per_factor,
      early_exit = FALSE, iteration = as.integer(iter),
      prior_removed = already_removed, cor_mat = cor_mat
    )

    if (nrow(round_tbl) == 0) break
    all_iter_results[[iter]] <- round_tbl
    best <- round_tbl[1, , drop = FALSE]

    if (best$Has_Cross_Loading == "No" && best$Has_Low_Loading == "No") {
      clean_solution <- list(
        iteration = iter,
        summary_row = best,
        removed_items = .parse_item_string(best$Removed_Items),
        remaining_items = .parse_item_string(best$Remaining_Items)
      )
      break
    }

    newly <- .parse_item_string(best$Removed_This_Step)
    if (length(newly) == 0 || any(newly %in% already_removed)) break

    already_removed <- c(already_removed, newly)
    current_items <- .parse_item_string(best$Remaining_Items)
  }

  list(
    clean_solution = clean_solution,
    iteration_results = all_iter_results,
    final_removed = already_removed,
    final_remaining = current_items
  )
}

itemrest <- function(data,
                     cor_method = "pearson",
                     n_factors = NULL,
                     extract = "uls",
                     rotate = "oblimin",
                     min_loading = 0.30,
                     loading_diff = 0.10,
                     search = "all_subsets",
                     max_iter = 20,
                     max_removed_total = NULL,
                     max_removed_per_iter = 3,
                     min_items_per_factor = 3) {
  search <- match.arg(search, c("all_subsets", "stepwise", "backward"))
  cor_method <- match.arg(tolower(cor_method), c("pearson", "spearman", "polychoric"))

  data <- as.data.frame(data)
  colnames(data) <- gsub("\\.", "_", colnames(data))

  clear_efa_cache()
  full_cor_mat <- cor_matrix_custom(data, cor_method)

  auto_n_factors <- is.null(n_factors)
  if (auto_n_factors) {
    n_factors <- determine_n_factors(data, cor_method, cor_mat = full_cor_mat)
  }
  n_factors <- as.integer(n_factors)

  descriptives <- descriptive_stats(data)
  initial_efa <- efa_custom(data, n_factors, cor_method, extract, rotate, use_cache = TRUE, cor_mat = full_cor_mat)
  problem_items <- identify_problem_items(initial_efa, min_loading, loading_diff)
  all_problem <- sort_item_ids(unique(c(problem_items$cross_3, problem_items$cross_2, problem_items$low_loading)))

  repro_snippet <- if (cor_method == "polychoric") {
    paste0(
      "cor_mat <- get_cor_matrix(data, cor_method = \"polychoric\")\n",
      "psych::fa(r = cor_mat, nfactors = ", n_factors, ", ",
      "rotate = \"", rotate, "\", fm = \"", extract, "\")"
    )
  } else {
    NULL
  }

  removal_summary <- NULL
  clean_solution <- NULL
  search_result <- NULL
  current_items <- colnames(data)

  if (length(all_problem) > 0) {
    if (search == "all_subsets") {
      search_result <- all_subsets_search(
        data, current_items, all_problem, n_factors, cor_method,
        extract, rotate, min_loading, loading_diff,
        max_removed_per_iter, min_items_per_factor, cor_mat = full_cor_mat
      )
      removal_summary <- search_result$summary
      clean_solution <- search_result$clean_solution
    } else if (search == "stepwise") {
      search_result <- stepwise_search(
        data, current_items, n_factors, cor_method, extract, rotate,
        min_loading, loading_diff, max_iter, max_removed_total,
        max_removed_per_iter, min_items_per_factor, cor_mat = full_cor_mat
      )
      removal_summary <- do.call(rbind, search_result$iteration_results)
      clean_solution <- search_result$clean_solution
    } else if (search == "backward") {
      search_result <- backward_search(
        data, current_items, all_problem, n_factors, cor_method,
        extract, rotate, min_loading, loading_diff,
        max_iter, max_removed_total, min_items_per_factor, cor_mat = full_cor_mat
      )
      removal_summary <- do.call(rbind, search_result$iteration_results)
      clean_solution <- search_result$clean_solution
    }
  } else {
    clean_solution <- list(
      iteration = 0L,
      removed_items = character(0),
      remaining_items = current_items,
      efa = initial_efa
    )
  }

  if (is.null(removal_summary)) {
    removal_summary <- data.frame()
  }

  results_all <- removal_summary
  results_clean <- clean_result_rows(results_all)
  recommended <- build_recommended_strategy(clean_solution, results_all, initial_efa, current_items)

  initial_summary <- list(
    cronbachs_alpha = initial_efa$alpha,
    explained_var = initial_efa$explained_var,
    cross_loading_items = sort_item_ids(unique(c(problem_items$cross_2, problem_items$cross_3))),
    low_loading_items = sort_item_ids(problem_items$low_loading),
    all_problem_items = all_problem
  )

  output <- list(
    all = results_all,
    clean = recommended,
    results_all = results_all,
    results_clean = results_clean,
    recommended = recommended,
    initial = initial_summary,
    descriptive_stats = descriptives,
    initial_efa = initial_efa,
    problem_items = problem_items,
    all_problem_items = all_problem,
    all_problem_items_combined = all_problem,
    removal_summary = results_all,
    clean_solution = clean_solution,
    search_result = search_result,
    replication_code = repro_snippet,
    settings = list(
      n_factors = n_factors,
      auto_n_factors = auto_n_factors,
      cor_method = cor_method,
      extract = extract,
      rotate = rotate,
      min_loading = min_loading,
      loading_diff = loading_diff,
      search = search,
      max_iter = max_iter,
      max_removed_total = max_removed_total,
      max_removed_per_iter = max_removed_per_iter,
      min_items_per_factor = min_items_per_factor
    )
  )
  class(output) <- "itemrest_result"
  output
}

#' Print an itemrest result.
#' @param x Object of class itemrest_result.
#' @param report Report scope: clean or all.
#' @param ... Not used.
#' @return Invisibly returns x.
#' @export
print.itemrest_result <- function(x, report = "clean", ...) {
  report <- match.arg(report, c("clean", "all"))
  results_all <- if (!is.null(x$results_all)) x$results_all else x$removal_summary

  if (report == "clean") {
    if (is.null(x$recommended)) {
      cat("No clean strategy was found.\n")
    } else if (is.null(x$recommended$row)) {
      cat("Recommended strategy:\n")
      cat("Items removed:", if (length(x$recommended$removed_items) == 0) "None" else paste(sort_item_ids(x$recommended$removed_items), collapse = ", "), "\n")
      cat("Alpha:", formatC(x$recommended$cronbachs_alpha, digits = 3, format = "f"), "\n")
      cat("Explained var:", paste0(formatC(x$recommended$explained_var * 100, format = "f", digits = 2), "%"), "\n")
    } else {
      print(x$recommended$row, row.names = FALSE)
    }
  } else if (is.null(results_all) || nrow(results_all) == 0) {
    cat("No removal strategies evaluated.\n")
  } else {
    print(results_all, row.names = FALSE)
  }

  invisible(x)
}

#' About UI Module
#' @param id Module namespace ID.
#' @import shiny
#' @import bslib
#' @importFrom bsicons bs_icon
#' @noRd
about_ui <- function(id) {
  ns <- NS(id)

  tagList(
    # 1. Uygulama Hakkında (En üstte geniş kart)
    card(
      card_header(
        class = "bg-dark text-white",
        "About FAfA",
        bs_icon("info-circle")
      ),
      card_body(
        htmlOutput(ns("application_description_html"))
      )
    ),

    # 2. Alt Kısım: Geliştirici, Teşekkürler ve Atıf (3 Sütunlu Yapı)
    layout_columns(
      col_widths = c(4, 4, 4),

      # Geliştirici Bilgisi
      card(
        card_header(
          class = "bg-primary text-white",
          "Developer",
          bs_icon("person-workspace")
        ),
        card_body(
          htmlOutput(ns("developer_info_html"))
        )
      ),

      # Teşekkürler & Versiyon (EKSİK OLAN KISIM EKLENDİ)
      card(
        card_header(
          class = "bg-success text-white",
          "Acknowledgements & Version",
          bs_icon("people")
        ),
        card_body(
          htmlOutput(ns("contributors_version_html"))
        )
      ),

      # Atıf Bilgisi
      card(
        card_header(
          class = "bg-info text-white",
          "How to Cite",
          bs_icon("quote")
        ),
        card_body(
          htmlOutput(ns("citation_info_html"))
        )
      )
    )
  )
}

#' Missing Value Handling Server Module
#'
#' @param id Module namespace ID.
#' @param data Reactive containing the input dataset.
#' @return Reactive containing the processed (imputed) dataset.
#' @import shiny
#' @importFrom naniar vis_miss mcar_test
#' @importFrom ggplot2 theme element_text
#' @importFrom stats na.omit median
#' @importFrom utils write.csv
#' @importFrom graphics image
#' @export
mod_missing_server <- function(id, data) {
  moduleServer(id, function(input, output, session) {

    processed_data_rv <- reactiveVal(NULL)

    # Initialize with original data
    observeEvent(data(), {
      req(data())
      processed_data_rv(data())
    })

    # 1. Visualization
    output$missing_plot <- renderPlot({
      req(processed_data_rv())
      current_df <- processed_data_rv()

      if (requireNamespace("naniar", quietly = TRUE)) {
        naniar::vis_miss(current_df) +
          ggplot2::theme(axis.text.x = ggplot2::element_text(angle = 90))
      } else {
        image(is.na(current_df), main = "Black areas represent missing values", axes = FALSE)
      }
    })

    # 2. MCAR Test (Button Triggered)
    mcar_result_trigger <- eventReactive(input$run_mcar_button, {
      req(data())
      raw_df <- data()

      if (sum(is.na(raw_df)) == 0) {
        return("No missing values found. MCAR test not applicable.")
      }

      showNotification("Running Little's MCAR Test...", type = "message")

      if (requireNamespace("naniar", quietly = TRUE)) {
        tryCatch({
          res <- naniar::mcar_test(raw_df)
          p_val <- res$p.value
          interp <- if(!is.na(p_val) && p_val < 0.05) {
            "RESULT: p < 0.05\nInterpretation: Data is likely NOT MCAR (Systematic missingness)."
          } else {
            "RESULT: p > 0.05\nInterpretation: Data is likely Missing Completely at Random (MCAR)."
          }
          paste0("Statistic: ", round(res$statistic, 2), "\ndf: ", res$df, "\np-value: ", format.pval(p_val, eps=0.001), "\n\n", interp)
        }, error = function(e) paste("Error running MCAR test. Ensure all columns are numeric.\nDetails:", e$message))
      } else {
        "Package 'naniar' is required."
      }
    })

    output$mcar_output <- renderPrint({
      req(mcar_result_trigger())
      cat(mcar_result_trigger())
    })

    # 3. Summary Table
    output$missing_summary_table <- renderTable({
      req(processed_data_rv())
      df <- processed_data_rv()
      miss_counts <- colSums(is.na(df))
      summary_df <- data.frame(Variable = names(df), Missing_Count = miss_counts, Missing_Percentage = round((miss_counts / nrow(df)) * 100, 2))
      summary_df[summary_df$Missing_Count > 0, ]
    }, rownames = FALSE, striped = TRUE)

    # 4. Imputation Logic (UPDATED FOR 2 RF METHODS)
    observeEvent(input$apply_imputation, {
      req(data())
      raw_df <- data()
      method <- input$imputation_method

      # Uzun süren işlemler için bildirim
      if(grepl("missForest", method) || method %in% c("amelia", "mice")) {
        showNotification("Running advanced imputation... Please wait.", type = "message", duration = 5)
      }

      clean_df <- tryCatch({

        if (method == "listwise") {
          stats::na.omit(raw_df)

        } else if (method == "mean") {
          as.data.frame(lapply(raw_df, function(x) {
            if(is.numeric(x)) x[is.na(x)] <- mean(x, na.rm = TRUE)
            return(x)
          }))

        } else if (method == "median") {
          as.data.frame(lapply(raw_df, function(x) {
            if(is.numeric(x)) x[is.na(x)] <- stats::median(x, na.rm = TRUE)
            return(x)
          }))

        } else if (method == "amelia") {
          if (!requireNamespace("Amelia", quietly = TRUE)) stop("Package 'Amelia' required.")
          Amelia::amelia(raw_df, m = 1, p2s = 0, idvars = NULL)$imputations[[1]]

        } else if (method == "mice") {
          if (!requireNamespace("mice", quietly = TRUE)) stop("Package 'mice' required.")
          mice::complete(mice::mice(raw_df, m = 1, printFlag = FALSE), 1)

        } else if (method == "missForest_cont") {
          # SÜREKLİ ATAMA (Regresyon)
          if (!requireNamespace("missForest", quietly = TRUE)) stop("Package 'missForest' required.")
          # Doğrudan nümerik olarak veriyoruz
          missForest::missForest(raw_df, verbose = FALSE)$ximp

        } else if (method == "missForest_cat") {
          # KATEGORİK ATAMA (Sınıflandırma)
          if (!requireNamespace("missForest", quietly = TRUE)) stop("Package 'missForest' required.")

          # Hepsini faktöre çevir ki RF sınıflandırma yapsın
          df_factors <- raw_df
          df_factors[] <- lapply(df_factors, as.factor)

          # missForest çalıştır
          rf_out <- missForest::missForest(df_factors, verbose = FALSE)$ximp

          # Tekrar nümeriğe çevir
          rf_out[] <- lapply(rf_out, function(x) as.numeric(as.character(x)))
          rf_out

        } else {
          raw_df
        }

      }, error = function(e) {
        showNotification(paste("Imputation Error:", e$message), type = "error", duration = 8)
        return(raw_df)
      })

      processed_data_rv(clean_df)

      if(method != "none") {
        showNotification(paste("Applied:", method, "| Data updated."), type = "message")
      }
    })

    # 5. Download Handler
    output$download_data <- downloadHandler(
      filename = function() paste0("processed_data_", input$imputation_method, "_", Sys.Date(), ".csv"),
      content = function(file) {
        req(processed_data_rv())
        utils::write.csv(processed_data_rv(), file, row.names = FALSE)
      }
    )

    return(processed_data_rv)
  })
}

#' The application User-Interface
#'
#' @param request Internal parameter for {shiny}.
#'     DO NOT REMOVE.
#' @import shiny
#' @import bslib
#' @importFrom bsicons bs_icon
#' @noRd

# ---- Helper: sidebar group label (dark background safe) ----
.sidebar_label <- function(icon_name, label) {
  div(
    class = "fafa-sidebar-label mt-4 mb-1",
    bs_icon(icon_name), " ", label
  )
}

app_ui <- function(request) {
  tagList(
    golem_add_external_resources(),

    page_fillable(
      title = "FAfA: Factor Analysis for All",
      theme = bs_theme(
        version      = 5,
        base_font    = font_google("Inter"),
        heading_font = font_google("Inter"),
        primary      = "#2563EB",
        secondary    = "#64748B",
        success      = "#16a34a",
        info         = "#0891B2",
        warning      = "#D97706",
        danger       = "#DC2626",
        "enable-rounded"    = TRUE,
        "font-size-base"    = "0.875rem",
        "body-bg"           = "#f1f5f9",
        "body-color"        = "#1e293b",
        "card-border-color" = "#e2e8f0",
        "card-cap-bg"       = "#f8fafc",
        "card-cap-color"    = "#334155",
        "border-radius"     = "0.5rem"
      ) |>
        bs_add_rules("

/* ================================================================
   FAfA Modern Design System
   ================================================================ */

/* ---- Sidebar logo ---- */
.fafa-logo {
  display: flex;
  justify-content: center;
  padding: 16px 8px 4px 8px;
}
.fafa-logo img {
  width: 108px;
  height: auto;
  filter: drop-shadow(0 2px 6px rgba(0,0,0,0.35));
  border-radius: 4px;
}

/* ---- Sidebar: dark panel ---- */
.row > .col-sm-3:has(#tabs) {
  background: #0f172a;
  border-radius: 14px;
  padding: 14px 10px 20px 10px !important;
  min-height: calc(100vh - 80px);
}

#tabs .nav-link {
  color: #94a3b8;
  border-radius: 8px;
  font-size: 0.815rem;
  padding: 7px 11px;
  margin-bottom: 1px;
  transition: background 0.14s ease, color 0.14s ease;
  font-weight: 400;
  line-height: 1.4;
}

#tabs .nav-link:hover {
  background: rgba(255,255,255,0.07);
  color: #e2e8f0;
}

#tabs .nav-link.active {
  background: linear-gradient(135deg,#1d4ed8,#3b82f6);
  color: #fff !important;
  font-weight: 500;
  box-shadow: 0 2px 10px rgba(37,99,235,0.38);
}

/* Sidebar group labels */
.fafa-sidebar-label {
  color: #4b5563;
  font-size: 0.67rem;
  font-weight: 700;
  letter-spacing: 0.09em;
  text-transform: uppercase;
  padding: 12px 11px 3px 11px;
  display: block;
}

/* ---- Cards ---- */
.card {
  border: 1px solid #e2e8f0 !important;
  border-radius: 12px !important;
  box-shadow: 0 1px 3px rgba(0,0,0,0.05),0 1px 2px rgba(0,0,0,0.04) !important;
}

.card-header {
  border-radius: 11px 11px 0 0 !important;
  padding: 11px 16px;
  font-size: 0.84rem;
  font-weight: 600;
  letter-spacing: 0.01em;
  border-bottom: 1px solid rgba(0,0,0,0.08) !important;
}

.card-header.bg-primary { background: linear-gradient(135deg,#1d4ed8,#2563eb) !important; color:#fff !important; }
.card-header.bg-info    { background: linear-gradient(135deg,#0369a1,#0891b2) !important; color:#fff !important; }
.card-header.bg-success { background: linear-gradient(135deg,#15803d,#16a34a) !important; color:#fff !important; }
.card-header.bg-danger  { background: linear-gradient(135deg,#b91c1c,#dc2626) !important; color:#fff !important; }
.card-header.bg-warning { background: linear-gradient(135deg,#b45309,#d97706) !important; color:#fff !important; }
.card-header.bg-dark    { background: linear-gradient(135deg,#0f172a,#1e293b) !important; color:#fff !important; }

.card-header:not([class*='bg-']) {
  background: #f8fafc !important;
  color: #334155 !important;
  border-bottom: 1px solid #e2e8f0 !important;
}

.card-body { padding: 16px; }

/* ---- Buttons ---- */
.btn {
  border-radius: 8px;
  font-size: 0.84rem;
  font-weight: 500;
  letter-spacing: 0.01em;
  padding: 7px 15px;
  transition: all 0.14s ease;
}

.btn-primary { box-shadow: 0 1px 5px rgba(37,99,235,0.32); }
.btn-success { box-shadow: 0 1px 5px rgba(22,163,74,0.32);  }
.btn-danger  { box-shadow: 0 1px 5px rgba(220,38,38,0.32);  }
.btn-info    { box-shadow: 0 1px 5px rgba(8,145,178,0.32);  }

.btn:hover:not(:disabled) { transform: translateY(-1px); filter: brightness(1.07); }
.btn:active { transform: translateY(0) !important; }

.btn-outline-secondary {
  color: #475569 !important;
  border-color: #94a3b8 !important;
  background: transparent !important;
}
.btn-outline-secondary:hover {
  background: #64748b !important;
  border-color: #64748b !important;
  color: #fff !important;
}

/* ---- Tables ---- */
.table { font-size: 0.825rem; }

.table > thead > tr > th {
  background: #f1f5f9;
  color: #475569;
  font-weight: 600;
  font-size: 0.75rem;
  text-transform: uppercase;
  letter-spacing: 0.06em;
  border-bottom: 2px solid #e2e8f0 !important;
  padding: 9px 13px;
  white-space: nowrap;
}

.table > tbody > tr > td {
  padding: 8px 13px;
  vertical-align: middle;
  border-color: #f1f5f9;
}

.table-striped > tbody > tr:nth-of-type(odd) > td { background: #f8fafc; }
.table-bordered { border-color: #e2e8f0 !important; }

/* ---- Form controls ---- */
.form-control, .form-select {
  border-radius: 8px;
  border-color: #e2e8f0;
  font-size: 0.84rem;
  background: #fff;
}

.form-control:focus, .form-select:focus {
  border-color: #3b82f6;
  box-shadow: 0 0 0 3px rgba(59,130,246,0.12);
}

.form-label {
  font-weight: 500;
  font-size: 0.81rem;
  color: #374151;
  margin-bottom: 4px;
}

/* ---- Shiny notifications ---- */
.shiny-notification {
  border-radius: 10px !important;
  border: none !important;
  border-left: 4px solid #3b82f6 !important;
  box-shadow: 0 8px 24px rgba(0,0,0,0.13) !important;
  font-size: 0.83rem;
  background: #fff !important;
  color: #1e293b !important;
}

.shiny-notification-error   { border-left-color: #dc2626 !important; }
.shiny-notification-warning { border-left-color: #d97706 !important; }
.shiny-notification-message { border-left-color: #16a34a !important; }

/* ---- Value boxes ---- */
.bslib-value-box {
  border-radius: 12px !important;
  border: none !important;
  box-shadow: 0 2px 10px rgba(0,0,0,0.07) !important;
}

/* ---- Scrollbars ---- */
::-webkit-scrollbar           { width: 5px; height: 5px; }
::-webkit-scrollbar-track     { background: transparent; }
::-webkit-scrollbar-thumb     { background: #cbd5e1; border-radius: 4px; }
::-webkit-scrollbar-thumb:hover { background: #94a3b8; }

/* ---- Misc ---- */
hr           { border-color: #e2e8f0; opacity: 0.7; }
.text-muted  { color: #94a3b8 !important; }
small,.small { font-size: 0.78rem; }
code         { font-size: 0.8rem; background: #f1f5f9; border-radius: 4px; padding: 1px 5px; }

        "),

      # ---- Navigation ----
      navset_pill_list(
        id     = "tabs",
        widths = c(3, 9),

        # Logo
        nav_item(
          div(
            class = "fafa-logo",
            tags$img(src = "www/logo.png", alt = "FAfA Logo")
          )
        ),

        # Data & Wrangling
        nav_item(.sidebar_label("database-gear", "Data & Wrangling")),
        nav_panel("Select Data",       value = "data",        icon = bs_icon("file-earmark-arrow-up"), data_selection_ui("data_selection")),
        nav_panel("Exclude Variables", value = "ex_var",      icon = bs_icon("x-circle"),              wrangling_ui_ex_var("wrangling_ex_var")),
        nav_panel("Missing Values",    value = "missing_val", icon = bs_icon("patch-question"),         mod_missing_ui("missing_val")),
        nav_panel("Split Dataset",     value = "split_data",  icon = bs_icon("scissors"),              wrangling_ui_split("wrangling_split")),
        nav_panel("Manage Outliers",   value = "outliers",    icon = bs_icon("exclamation-triangle"),   wrangling_ui_outliers("wrangling_outliers")),

        # Assumption Checks
        nav_item(.sidebar_label("clipboard-check", "Assumption Checks")),
        nav_panel("Assumptions", value = "assumptions", icon = bs_icon("check2-square"), assumptions_ui("assumptions")),

        # Exploratory Factor Analysis
        nav_item(.sidebar_label("search", "Exploratory Factor Analysis")),
        nav_panel("Factor Retention",     value = "fac_ret",    icon = bs_icon("graph-up"),   efa_ui_fac_ret("efa_fac_ret")),
        nav_panel("EFA Setup & Analysis", value = "efa",        icon = bs_icon("sliders"),    efa_ui_analysis("efa_analysis")),
        nav_panel("Item Drop Out",        value = "item_rest",  icon = bs_icon("list-check"), mod_itemrest_ui("item_rest")),
        nav_panel("EFA Reporting",        value = "report_efa", icon = bs_icon("file-text"),  efa_ui_report("efa_report")),

        # Advanced Analysis
        nav_item(.sidebar_label("cpu", "Advanced Analysis")),
        nav_panel("Exploratory Graph Analysis",   value = "ega",         icon = bs_icon("diagram-3"),        ega_ui("ega")),
        nav_panel("Confirmatory Factor Analysis", value = "cfa",         icon = bs_icon("diagram-2"),        cfa_ui("cfa")),
        nav_panel("Measurement Invariance",       value = "inv",         icon = bs_icon("arrow-left-right"), inv_ui("inv")),
        nav_panel("Reliability Analysis",         value = "reliability", icon = bs_icon("award"),            reliability_ui("reliability")),
        nav_panel("Item Weighting",               value = "item_weight", icon = bs_icon("calculator"),       item_weighting_ui("item_weighting")),

        # Info
        nav_item(.sidebar_label("info-circle", "Info")),
        nav_panel("About", value = "about", icon = bs_icon("question-circle"), about_ui("about"))
      )
    )
  )
}

#' Add external Resources to the Application
#'
#' @import shiny
#' @importFrom golem add_resource_path activate_js favicon bundle_resources
#' @noRd
golem_add_external_resources <- function() {
  add_resource_path("www", app_sys("app/www"))
  tags$head(
    favicon(),
    bundle_resources(path = app_sys("app/www"), app_title = "FAfA")
  )
}

#' Measurement Invariance UI
#' @noRd
inv_ui <- function(id) {
  ns <- NS(id)
  tagList(
    layout_columns(
      col_widths = c(4, 8),
      tagList(
        card(
          card_header("Model Builder", class = "bg-info text-white", bs_icon("magic")),
          card_body(
            h6("Define Factor (=~)", class = "text-primary"),
            textInput(ns("builder_factor_name"), "Factor Name:", placeholder = "e.g. F1, F2, Factor1 ..."),
            selectizeInput(
              ns("builder_items"),
              "Select Indicators:",
              choices = NULL,
              multiple = TRUE,
              options = list(placeholder = "Select variables...")
            ),
            actionButton(
              ns("btn_add_to_model"),
              "Add to Syntax",
              icon = icon("code"),
              class = "btn-secondary btn-sm w-100 mb-3"
            ),

            h6("Add Covariance (~~)", class = "text-primary"),
            selectizeInput(
              ns("builder_cov_items"),
              "Select 2 Variables:",
              choices = NULL,
              multiple = TRUE,
              options = list(maxItems = 2, placeholder = "Select 2 vars...")
            ),
            actionButton(
              ns("btn_add_cov"),
              "Add Covariance",
              icon = icon("link"),
              class = "btn-secondary btn-sm w-100"
            )
          )
        ),

        card(
          card_header("Invariance Setup", class = "bg-primary text-white"),
          card_body(
            textAreaInput(
              ns("inv_model_syntax"),
              "Model Syntax (lavaan):",
              rows = 6,
              placeholder = "Use the builder above or type syntax here..."
            ),
            selectInput(ns("grouping_variable_select"), "Grouping Variable:", choices = ""),
            checkboxGroupInput(
              ns("invariance_levels_checkbox"),
              "Levels:",
              choices = c("configural", "metric", "scalar", "strict"),
              selected = c("configural", "metric")
            ),
            actionButton(ns("run_invariance_button"), "Run Analysis", class = "btn-success w-100")
          )
        )
      ),


      card(
        card_header("Results"),
        navset_card_tab(
          nav_panel("Fit Measures", tableOutput(ns("invariance_fit_measures_table"))),
          nav_panel("Comparison (LRT)", tableOutput(ns("model_comparison_table")))
        )
      )
    )
  )
}

#' Assumptions Server Logic (Modern)
#'
#' @param id Module namespace ID.
#' @param data Input data (reactive).
#' @export
assumptions_server <- function(id, data) {
  moduleServer(id, function(input, output, session) {
    results_rv <- reactiveValues(desc = NULL, multi = NULL, norm = NULL)

    observeEvent(input$run_descriptives_button, {
      req(data())
      res <- assumptions(data())
      results_rv$desc <- res$descriptives
    })

    observeEvent(input$run_collinearity_button, {
      req(data())
      res <- assumptions(data())
      results_rv$multi <- res$multicollinearity
    })

    observeEvent(input$run_normality_tests_button, {
      req(data())
      progress_id <- showNotification("Running normality tests...",
                                      duration = NULL, type = "message")
      on.exit(removeNotification(progress_id), add = TRUE)

      tryCatch({
        res            <- assumptions(data())
        norm_df        <- res$mvn_table

        # Format p-values for display
        norm_df[["p-value"]] <- sapply(norm_df[["p-value"]], function(p) {
          if (is.na(p)) return(NA_character_)
          if (p < 0.001) "< .001" else as.character(round(p, 3))
        })

        results_rv$norm <- norm_df
      }, error = function(e) {
        showNotification(paste("Error:", e$message), type = "error", duration = 8)
      })
    })

    output$descriptives_table_output          <- renderTable({ results_rv$desc  }, rownames = TRUE)
    output$collinearity_table_output          <- renderTable({ results_rv$multi })
    output$multivariate_normality_table_output <- renderTable({
      validate(need(results_rv$norm, "Click 'Run Normality Tests' to compute results."))
      results_rv$norm
    }, striped = TRUE, bordered = TRUE, na = "-")

    output$download_descriptives_button <- downloadHandler(
      filename = "descriptives.csv",
      content  = function(f) write.csv(results_rv$desc, f)
    )
  })
}

#' Wrangling Server Modules (Modern)
#' @import shiny
#' @export
#' @param id Module namespace ID.
#' @param data Input data (reactive)

# 1. Exclude Variables Server
wrangling_server_ex_var <- function(id, data) {
  moduleServer(id, function(input, output, session) {

    # Tracks names of currently excluded variables
    excluded_vars_rv <- reactiveVal(character(0))

    # Active dataset: NULL when nothing excluded (so fallback chain in app_server works),
    # otherwise the original data minus excluded columns.
    data_after_exclusion_rv <- reactive({
      excl <- excluded_vars_rv()
      if (length(excl) == 0) return(NULL)
      req(data())
      remaining <- setdiff(names(data()), excl)
      if (length(remaining) == 0) return(NULL)
      data()[, remaining, drop = FALSE]
    })

    # When upstream data changes, drop any excluded names that no longer exist.
    # This handles new file uploads without wiping valid exclusions on re-imputation.
    observeEvent(data(), {
      req(data())
      excl <- excluded_vars_rv()
      if (length(excl) > 0) {
        valid_excl <- intersect(excl, names(data()))
        excluded_vars_rv(valid_excl)
      }
    }, ignoreNULL = TRUE)

    # Keep both checkbox lists in sync with current exclusion state
    observe({
      req(data())
      excl  <- excluded_vars_rv()
      avail <- setdiff(names(data()), excl)
      updateCheckboxGroupInput(session, "available_vars_checkbox",
                               choices = avail, selected = NULL)
      updateCheckboxGroupInput(session, "excluded_vars_checkbox",
                               choices = excl, selected = NULL)
    })

    # Summary counts
    output$variable_summary <- renderUI({
      req(data())
      excl    <- excluded_vars_rv()
      total   <- length(names(data()))
      n_excl  <- length(excl)
      n_act   <- total - n_excl
      div(
        class = "d-flex justify-content-around text-center py-1",
        div(h5(total, class = "mb-0 text-primary"), tags$small("Total")),
        div(h5(n_excl, class = "mb-0 text-danger"),  tags$small("Excluded")),
        div(h5(n_act,  class = "mb-0 text-success"), tags$small("Active"))
      )
    })

    # --- Exclude ---
    observeEvent(input$exclude_button, {
      req(data())
      selected <- input$available_vars_checkbox
      if (is.null(selected) || length(selected) == 0) {
        showNotification("Please select at least one variable to exclude.",
                         type = "warning")
        return()
      }
      new_excl   <- union(excluded_vars_rv(), selected)
      n_remaining <- length(names(data())) - length(new_excl)
      if (n_remaining == 0) {
        showNotification("Cannot exclude all variables.", type = "error")
        return()
      }
      excluded_vars_rv(new_excl)
      showNotification(
        paste0(length(selected), " variable(s) excluded. ",
               "Active dataset: ", n_remaining, " variable(s)."),
        type     = "message",
        duration = 4
      )
    })

    # --- Recover selected ---
    observeEvent(input$recover_button, {
      selected <- input$excluded_vars_checkbox
      if (is.null(selected) || length(selected) == 0) {
        showNotification("Please select at least one variable to recover.",
                         type = "warning")
        return()
      }
      new_excl <- setdiff(excluded_vars_rv(), selected)
      excluded_vars_rv(new_excl)
      req(data())
      n_act <- length(names(data())) - length(new_excl)
      showNotification(
        paste0(length(selected), " variable(s) recovered. ",
               "Active dataset: ", n_act, " variable(s)."),
        type     = "message",
        duration = 4
      )
    })

    # --- Reset all ---
    observeEvent(input$reset_button, {
      excluded_vars_rv(character(0))
      showNotification("All variables restored to active dataset.",
                       type = "message")
    })

    # --- Download active data ---
    output$download_excluded_data_button <- downloadHandler(
      filename = function() "active_dataset.csv",
      content  = function(file) {
        d <- data_after_exclusion_rv()
        if (is.null(d)) d <- data()
        write.csv(d, file, row.names = FALSE)
      }
    )

    return(data_after_exclusion_rv)
  })
}

# 2. Split Data
wrangling_server_split <- function(id, data) {
  moduleServer(id, function(input, output, session) {
    split_datasets_rv <- reactiveValues(first_half = NULL, second_half = NULL)

    observeEvent(input$split_data_button, {
      req(data())
      tryCatch({
        df <- data()
        n1 <- floor((input$split_percentage_slider / 100) * nrow(df))
        idx <- sample.int(nrow(df), n1)
        split_datasets_rv$first_half  <- df[ idx, , drop = FALSE]
        split_datasets_rv$second_half <- df[-idx, , drop = FALSE]
        showNotification("Data split successfully!", type = "message")
      }, error = function(e) showNotification(e$message, type = "error"))
    })

    output$download_first_subset_button <- downloadHandler(
      filename = "subset1.csv",
      content  = function(file) write.csv(split_datasets_rv$first_half,  file, row.names = FALSE)
    )
    output$download_second_subset_button <- downloadHandler(
      filename = "subset2.csv",
      content  = function(file) write.csv(split_datasets_rv$second_half, file, row.names = FALSE)
    )
  })
}

# 3. Manage Outliers
wrangling_server_outliers <- function(id, data) {
  moduleServer(id, function(input, output, session) {
    outlier_info_rv <- reactiveValues(table = NULL, count = NULL,
                                      data_clean = NULL, indices = NULL)

    observeEvent(input$check_outliers_button, {
      req(data())
      tryCatch({
        res <- assumptions(data(), mah_p_threshold = input$mah_p_value_threshold_input)
        outlier_info_rv$table   <- res$Mah_significant
        outlier_info_rv$count   <- res$n_outlier
        outlier_info_rv$indices <- res$Mah_significant$Row_Number_In_Complete_Data
        showNotification("Outlier check complete.", type = "message")
      }, error = function(e) showNotification(e$message, type = "error"))
    })

    observeEvent(input$remove_outliers_button, {
      req(data(), outlier_info_rv$indices)
      clean <- stats::na.omit(data())
      if (length(outlier_info_rv$indices) > 0) {
        outlier_info_rv$data_clean <- clean[-outlier_info_rv$indices, , drop = FALSE]
        showNotification("Outliers removed!", type = "message")
      }
    })

    output$outliers_table     <- renderTable({ outlier_info_rv$table })
    output$outlier_count_text <- renderText({ paste("Outliers found:", outlier_info_rv$count) })

    output$download_data_no_outliers_button <- downloadHandler(
      filename = "data_no_outliers.csv",
      content  = function(file) write.csv(outlier_info_rv$data_clean, file, row.names = FALSE)
    )

    return(reactive(outlier_info_rv$data_clean))
  })
}

#' Item Weighting Server Module
#' @param id Module namespace ID.
#' @param data Input data (reactive)
#' @export
item_weighting_server <- function(id, data) {
  moduleServer(id, function(input, output, session) {
    weighted_data_rv <- reactiveVal()

    observeEvent(input$calculate_weighted_scores_button, {
      req(data())
      # utils.R içindeki fonksiyonu kontrol et
      if (!exists("item_weighting")) {
        showNotification("Function 'item_weighting' missing.", type="error"); return()
      }

      tryCatch({
        res <- item_weighting(data())
        weighted_data_rv(res)
        showNotification("Calculated!", type="message")
      }, error = function(e) showNotification(e$message, type="error"))
    })

    output$weighted_scores_table_output <- renderTable({
      utils::head(weighted_data_rv(), 10)
    }, striped=TRUE)

    output$download_weighted_data_button <- downloadHandler(
      filename = "weighted_scores.csv",
      content = function(file) write.csv(weighted_data_rv(), file, row.names = FALSE)
    )
  })
}

#' Wrangling UI Modules
#' @import shiny
#' @import bslib
#' @importFrom bsicons bs_icon
#' @noRd

# 1. Exclude Variables
wrangling_ui_ex_var <- function(id) {
  ns <- NS(id)
  tagList(
    layout_columns(
      col_widths = c(6, 6),

      # --- LEFT: Available Variables ---
      card(
        card_header(
          class = "bg-success text-white",
          bs_icon("check2-square"), " Available Variables"
        ),
        card_body(
          p(class = "text-muted small mb-2",
            "Tick the variables you want to remove from the active dataset, then click Exclude."),
          div(
            style = paste(
              "max-height: 380px; overflow-y: auto;",
              "border: 1px solid #dee2e6; border-radius: 6px; padding: 8px 12px;"
            ),
            checkboxGroupInput(
              ns("available_vars_checkbox"),
              label    = NULL,
              choices  = NULL,
              selected = NULL
            )
          ),
          hr(class = "my-2"),
          actionButton(
            ns("exclude_button"),
            label = "Exclude Selected",
            icon  = icon("minus-circle"),
            class = "btn-danger w-100"
          )
        )
      ),

      # --- RIGHT: Excluded Variables + Summary + Actions ---
      card(
        card_header(
          class = "bg-danger text-white",
          bs_icon("x-circle"), " Excluded Variables"
        ),
        card_body(
          # Variable count summary
          uiOutput(ns("variable_summary")),
          hr(class = "my-2"),

          # List of excluded vars (tick to recover)
          p(class = "text-muted small mb-2",
            "Tick variables to bring back, then click Recover."),
          div(
            style = paste(
              "max-height: 240px; overflow-y: auto;",
              "border: 1px solid #dee2e6; border-radius: 6px; padding: 8px 12px;"
            ),
            checkboxGroupInput(
              ns("excluded_vars_checkbox"),
              label    = NULL,
              choices  = NULL,
              selected = NULL
            )
          ),
          hr(class = "my-2"),

          # Recover / Reset
          div(
            class = "d-flex gap-2 mb-2",
            actionButton(
              ns("recover_button"),
              label = "Recover Selected",
              icon  = icon("undo"),
              class = "btn-success flex-fill"
            ),
            actionButton(
              ns("reset_button"),
              label = "Reset All",
              icon  = icon("refresh"),
              class = "btn-outline-secondary flex-fill"
            )
          ),

          # Download active dataset
          downloadButton(
            ns("download_excluded_data_button"),
            label = "Download Active Data",
            class = "w-100"
          )
        )
      )
    )
  )
}

# 2. Split Data
wrangling_ui_split <- function(id) {
  ns <- NS(id)
  tagList(
    layout_columns(
      col_widths = 8,
      style = "margin: 0 auto;",
      card(
        card_header("Split Dataset", class = "bg-warning text-dark", bs_icon("scissors")),
        card_body(
          p("Randomly split data into two subsets (e.g., for EFA/CFA)."),
          sliderInput(ns("split_percentage_slider"), "First Subset %:",
                      min = 10, max = 90, value = 50, step = 5, post = "%"),
          actionButton(ns("split_data_button"), "Split Data",
                       icon = icon("random"), class = "btn-warning w-100"),
          hr(),
          div(class = "d-flex gap-2",
              downloadButton(ns("download_first_subset_button"),  "Save Part 1", class = "flex-fill"),
              downloadButton(ns("download_second_subset_button"), "Save Part 2", class = "flex-fill")
          )
        )
      )
    )
  )
}

# 3. Manage Outliers
wrangling_ui_outliers <- function(id) {
  ns <- NS(id)
  tagList(
    layout_columns(
      col_widths = c(4, 8),
      card(
        card_header("Outlier Detection", class = "bg-dark text-white", bs_icon("search")),
        card_body(
          p("Detect multivariate outliers using Mahalanobis Distance."),
          numericInput(ns("mah_p_value_threshold_input"), "P-value Threshold:",
                       value = 0.001, min = 0.00001, max = 0.05, step = 0.0001),
          actionButton(ns("check_outliers_button"), "Find Outliers",
                       icon = icon("search"), class = "btn-primary w-100 mb-2"),
          actionButton(ns("remove_outliers_button"), "Remove & Update",
                       icon = icon("user-minus"), class = "btn-danger w-100"),
          hr(),
          downloadButton(ns("download_data_no_outliers_button"), "Download Clean Data", class = "w-100")
        )
      ),
      card(
        card_header("Outlier Results"),
        card_body(
          textOutput(ns("outlier_count_text")),
          tableOutput(ns("outliers_table")) %>% withSpinner(type = 8, color = "#2C3E50")
        )
      )
    )
  )
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mod_efa_server.r
\name{efa_server_analysis}
\alias{efa_server_analysis}
\title{EFA Analysis Server Module}
\usage{
efa_server_analysis(id, data)
}
\arguments{
\item{id}{Module namespace ID.}

\item{data}{Input data (reactive).}
}
\description{
EFA Analysis Server Module
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/itemrest_internal.R
\name{print.itemrest_result}
\alias{print.itemrest_result}
\title{Print an itemrest result.}
\usage{
\method{print}{itemrest_result}(x, report = "clean", ...)
}
\arguments{
\item{x}{Object of class itemrest_result.}

\item{report}{Report scope: clean or all.}

\item{...}{Not used.}
}
\value{
Invisibly returns x.
}
\description{
Print an itemrest result.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.r
\name{op_null_or}
\alias{op_null_or}
\alias{\%||\%}
\title{Default Value Operator}
\usage{
x \%||\% y
}
\arguments{
\item{x}{Left hand side}

\item{y}{Right hand side}
}
\description{
Helper for default NULL values (coalesce)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mod_itemrest_ui.r
\name{mod_itemrest_ui}
\alias{mod_itemrest_ui}
\title{ItemRest Analysis UI Module}
\usage{
mod_itemrest_ui(id)
}
\arguments{
\item{id}{Module namespace ID.}
}
\description{
ItemRest Analysis UI Module
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mod_efa_server.r
\name{efa_server_report}
\alias{efa_server_report}
\title{EFA Reporting Server Module}
\usage{
efa_server_report(id, data, efa_output_reactive, efa_settings_reactive)
}
\arguments{
\item{id}{Module namespace ID.}

\item{data}{Input data (reactive).}

\item{efa_output_reactive}{Reactive containing the EFA results.}

\item{efa_settings_reactive}{Reactive containing the EFA settings.}
}
\description{
EFA Reporting Server Module
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mod_missing_server.r
\name{mod_missing_server}
\alias{mod_missing_server}
\title{Missing Value Handling Server Module}
\usage{
mod_missing_server(id, data)
}
\arguments{
\item{id}{Module namespace ID.}

\item{data}{Reactive containing the input dataset.}
}
\value{
Reactive containing the processed (imputed) dataset.
}
\description{
Missing Value Handling Server Module
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mod_item_weighting_server.r
\name{item_weighting_server}
\alias{item_weighting_server}
\title{Item Weighting Server Module}
\usage{
item_weighting_server(id, data)
}
\arguments{
\item{id}{Module namespace ID.}

\item{data}{Input data (reactive)}
}
\description{
Item Weighting Server Module
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mod_cfa_server.r
\name{cfa_server}
\alias{cfa_server}
\title{Confirmatory Factor Analysis (CFA) Server Module}
\usage{
cfa_server(id, data)
}
\arguments{
\item{id}{Module namespace ID.}

\item{data}{Reactive containing the input dataset.}
}
\description{
Confirmatory Factor Analysis (CFA) Server Module
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mod_reliability_server.r
\name{reliability_server}
\alias{reliability_server}
\title{Reliability Analysis Server Module}
\usage{
reliability_server(id, data)
}
\arguments{
\item{id}{Module namespace ID.}

\item{data}{Input data (reactive)}
}
\description{
Reliability Analysis Server Module
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/run_app.R
\name{run_app}
\alias{run_app}
\title{Run the Shiny Application}
\usage{
run_app()
}
\description{
This function launches the Shiny application.
It uses default options for \code{shinyApp} and golem.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mod_inv_server.r
\name{inv_server}
\alias{inv_server}
\title{Measurement Invariance Server Module}
\usage{
inv_server(id, data)
}
\arguments{
\item{id}{Module namespace ID.}

\item{data}{Input data (reactive)}
}
\description{
Measurement Invariance Server Module
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mod_data_selection_server.r
\name{data_selection_server}
\alias{data_selection_server}
\title{Data Selection Server Logic (Modern)}
\usage{
data_selection_server(id, data)
}
\arguments{
\item{id}{Module namespace ID.}

\item{data}{Input data (reactive).}
}
\description{
Data Selection Server Logic (Modern)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mod_about_server.r
\name{about_server}
\alias{about_server}
\title{About Server Module}
\usage{
about_server(id)
}
\arguments{
\item{id}{Module namespace ID.}
}
\description{
About Server Module
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mod_missing_ui.r
\name{mod_missing_ui}
\alias{mod_missing_ui}
\title{Missing Value Handling UI Module}
\usage{
mod_missing_ui(id)
}
\arguments{
\item{id}{Module namespace ID.}
}
\description{
Missing Value Handling UI Module
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mod_cfa_ui.r
\name{cfa_ui}
\alias{cfa_ui}
\title{Confirmatory Factor Analysis (CFA) UI Module}
\usage{
cfa_ui(id)
}
\arguments{
\item{id}{Module namespace ID.}
}
\description{
Confirmatory Factor Analysis (CFA) UI Module
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mod_wrangling_server.r
\name{wrangling_server_ex_var}
\alias{wrangling_server_ex_var}
\title{Wrangling Server Modules (Modern)}
\usage{
wrangling_server_ex_var(id, data)
}
\arguments{
\item{id}{Module namespace ID.}

\item{data}{Input data (reactive)}
}
\description{
Wrangling Server Modules (Modern)
}
